@@ -7,7 +7,7 @@
 
 /* default list of mibs to load */
 
-#define DEFAULT_MIBS "IP-MIB:IF-MIB:TCP-MIB:UDP-MIB:SNMPv2-MIB:RFC1213-MIB:UCD-SNMP-MIB"
+#define DEFAULT_MIBS "IP-MIB:IF-MIB:TCP-MIB:UDP-MIB:SNMPv2-MIB:RFC1213-MIB"
 
 /* default location to look for mibs to load using the above tokens
    and/or those in the MIBS envrionment variable*/
@@ -18,35 +18,58 @@
 
 #undef DEFAULT_MIBFILES
 
+/* should we compile to use special opaque types: float, double,
+   counter64, i64, ui64, union? */
+#undef OPAQUE_SPECIAL_TYPES
+
+/* comment the next line if you are compiling with libsnmp.h 
+   and are not using the UC-Davis SNMP library. */
+#define UCD_SNMP_LIBRARY 1
+
+/* comment the next line if you do not want SNMPv2 party-based auth. */
+#define USE_V2PARTY_PROTOCOL 1
+
+/* add in recent CMU library extensions (not complete) */
+#undef CMU_COMPATIBLE
+
+/* should "--" comments in mibs be a comment till the end of the line
+   or also until another "--", the latter being the technically
+   correct. */
+#undef MIB_COMMENT_IS_EOL_TERMINATED
+
+/* debugging stuff */
+#undef SNMP_NO_DEBUGGING           /* if defined, we optimize the code
+                                      to exclude all debugging calls. */
+#define SNMP_ALWAYS_DEBUG 0        /* Always print debugging information and
+                                      ignore the -D flag passed to the cmds */
+
 @TOP@
 
+/* define the system type include file here */
+#define SYSTEM_INCLUDE_FILE "s/generic.h"
+
+/* define the machine (cpu) type include file here */
+#define MACHINE_INCLUDE_FILE "m/generic.h"
+
 /* SNMPLIBDIR contains important files */
 
 #undef SNMPLIBPATH
+#undef SNMPSHAREPATH
 
 /* LOGFILE:  If defined it closes stdout/err/in and opens this in out/err's
    place.  (stdin is closed so that sh scripts won't wait for it) */
 
 #undef LOGFILE
 
-/* to hack in forced V2 security, I had to reserve the left byte of
-   the ACL Mib word for V2.  Do NOT define more than 5 V1 communities
-   else they will roll into these definitions (see snmp_vars.c:340) 
-   If GLOBALSECURITY is defined, it sets the default SNMP access type
-   for the extensible mibs to the setting type described. */
-
-#define GLOBALSECURITY SNMPV2AUTH    /* only authenticated snmpv2 requests
-                                        permited */
-
-/* configuration files.  I rdist the first one and leave machine
-   specific stuff in the second one */
-
 /* default system contact */
 #undef SYS_CONTACT
 
 /* system location */
 #undef SYS_LOC
 
+/* Use libwrap to handle allow/deny hosts? */
+#undef USE_LIBWRAP
+
 /* location of UNIX kernel */
 #define KERNEL_LOC "/vmunix"
 
@@ -64,9 +87,6 @@
 /* Where is the uname command */
 #define UNAMEPROG "/bin/uname"
 
-/* debugging stuff */
-#undef DODEBUG
-
 /* If you don't have root access don't exit upon kmem errors */
 #undef NO_ROOT_ACCESS
 
@@ -76,11 +96,12 @@
 
 @BOTTOM@
 
-/* define if sys/cdefs.h doesn't define the __P() macro */
-#undef SYS_CDEFS_DEFINES___P
+/* define if you are using linux and /proc/net/dev has the compressed
+   field, which exists in linux kernels 2.2 and greater. */
+#undef PROC_NET_DEV_HAS_COMPRESSED
 
 /* define rtentry to ortentry on SYSV machines (alphas) */
-#define RTENTRY rtentry;
+#undef RTENTRY
 
 /* Use BSD 4.4 routing table entries? */
 #undef RTENTRY_4_4
@@ -95,6 +116,7 @@
 
 /* ifnet structure tests */
 #undef STRUCT_IFNET_HAS_IF_BAUDRATE
+#undef STRUCT_IFNET_HAS_IF_SPEED
 #undef STRUCT_IFNET_HAS_IF_TYPE
 #undef STRUCT_IFNET_HAS_IF_IMCASTS
 #undef STRUCT_IFNET_HAS_IF_IQDROPS
@@ -124,6 +146,12 @@
 /* swdevt.sw_nblksenabled */
 #undef STRUCT_SWDEVT_HAS_SW_NBLKSENABLED
 
+/* nlist.n_value */
+#undef STRUCT_NLIST_HAS_N_VALUE
+
+/* vfsstat.f_frsize */
+#undef STRUCT_STATVFS_HAS_F_FRSIZE
+
 /* ifnet needs to have _KERNEL defined */
 #undef IFNET_NEEDS_KERNEL
 
@@ -136,6 +164,9 @@
 /* define if SIOCGIFADDR exists in sys/ioctl.h */
 #undef SYS_IOCTL_H_HAS_SIOCGIFADDR
 
+/* define if your compiler (processor) defines __FUNCTION__ for you */
+#undef HAVE_CPP_UNDERBAR_FUNCTION_DEFINED
+
 /* mib pointer to the top of the extensible tree.  This has been
  assigned to UCDavis by the iana group.  Optionally, point this to the
  location in the tree your company/organization has been allocated. */
@@ -147,12 +178,6 @@
 /* count the above numbers */
 #define EXTENSIBLENUM 7
 
-/* the ErrorFlag is V1 accessable because HP Openview does not support
-   V2.  You can make this list of pairs as long as you want, just make
-   sure to end it in -1.*/
-
-#define SECURITYEXCEPTIONS {100,SNMPV1,-1} /* the ErrorFlag is V1 */
-
 /* Mib-2 tree Info */
 /* These are the system information variables. */
 
@@ -163,40 +188,26 @@
    these: (See README for details) */
 
 /*   proc PROCESSNAME [MAX] [MIN] */
-#define PROCMIBNUM 1
-#define USEPROCMIB
+#define PROCMIBNUM 2
 
 /*   exec/shell NAME COMMAND      */
-#define SHELLMIBNUM 3
-#define USESHELLMIB
+#define SHELLMIBNUM 8
 
 /*   swap MIN                     */
 #define MEMMIBNUM 4
-#if defined(hpux9) || defined(bsdi2) || defined(linux) || defined(freebsd2)
-#define USEMEMMIB
-#endif
 
 /*   disk DISK MINSIZE            */
-#define DISKMIBNUM 6
-#if (HAVE_FSTAB_H || HAVE_SYS_STATVFS_H)
-#define USEDISKMIB
-#endif
+#define DISKMIBNUM 9
 
 /*   load 1 5 15                  */
-#define LOADAVEMIBNUM 7
-#define USELOADAVEMIB
-
-/*   pass MIBOID command */
-#define USEPASSMIB
+#define LOADAVEMIBNUM 10
 
 /* which version are you using? This mibloc will tell you */
 #define VERSIONMIBNUM 100
-#define USEVERSIONMIB    
 
 /* Reports errors the agent runs into */
 /* (typically its "can't fork, no mem" problems) */
 #define ERRORMIBNUM 101
-#define USEERRORMIB    
 
 /* The sub id of EXENSIBLEMIB returned to queries of
    .iso.org.dod.internet.mgmt.mib-2.system.sysObjectID.0 */
@@ -217,6 +228,7 @@
 #define IRIXID 9
 #define LINUXID 10
 #define BSDIID 11
+#define OPENBSDID 12
 #define UNKNOWNID 255
 
 #ifdef hpux9
@@ -250,7 +262,10 @@
 #define OSTYPE LINUXID
 #endif
 #if defined(bsdi2) || defined(bsdi3)
-#define OSTYPE IRIXID
+#define OSTYPE BSDIID
+#endif
+#ifdef openbsd2
+#define OSTYPE OPENBSDID
 #endif
 /* unknown */
 #ifndef OSTYPE
@@ -288,9 +303,9 @@
 
 #define DEFMAXLOADAVE 12.0      /* default maximum load average before error */
 
-#define MAXREADCOUNT 20   /* max times to loop reading output from
-                             execs.  Because of sleep(1)s, this will also
-                             be time to wait (in seconds) for exec to finish */
+#define MAXREADCOUNT 100   /* max times to loop reading output from
+                              execs.  Because of sleep(1)s, this will also
+                              be time to wait (in seconds) for exec to finish */
 
 #define SNMPBLOCK 1       /* Set to 1 if you want snmpgets to block and never
                              timeout.  Original CMU code had this
@@ -305,20 +320,16 @@
 /* Number of community strings to store */
 #define NUM_COMMUNITIES	5
 
+/* UNdefine to allow specifying zero-length community string */
+/* #define NO_ZEROLENGTH_COMMUNITY 1 */
+
 /* #define EXIT_ON_BAD_KLREAD  */
 /* define to exit the agent on a bad kernel read */
 
 #define LASTFIELD -1      /* internal define */
 
-/* debugging macros */
-
-#ifdef DODEBUG
-#define DEBUGP(x) fprintf(stderr,x);
-#define DEBUGP1(x,y) fprintf(stderr,x,y);
-#else
-#define DEBUGP(x)
-#define DEBUGP1(x,y)
-#endif
+/* configure options specified */
+#define CONFIGURE_OPTIONS ""
 
 #ifndef HAVE_STRCHR
 #ifdef HAVE_INDEX
@@ -379,43 +390,24 @@
 #endif
 #endif
 
-/* Watch out for compilers that don't handle void properly. */
-#undef void
+/* define if you have librpm and libdb */
+#undef HAVE_LIBRPM
+
+/* define if you have gethostbyname */
+#undef HAVE_GETHOSTBYNAME
+
+/* define if you have getservbyname */
+#undef HAVE_GETSERVBYNAME
 
 /* Not-to-be-compiled macros for use by configure only */
 #define config_require(x)
 #define config_arch_require(x,y)
 #define config_load_mib(x,y,z)
-#define config_parse_dot_conf(x,y,z)
+#define config_parse_dot_conf(w,x,y,z)
+#define config_add_mib(x)
   
 #include <mib_module_config.h>
 
-#ifndef DONT_INC_STRUCTS
-#include "agent/mibgroup/struct.h"
-#endif
-
-#ifndef linux
-#ifndef solaris2
-#define bsdlike
-#endif
-#endif
-
-#ifndef SYS_CDEFS_DEFINES___P
-#ifndef __P
-#ifdef __STDC__
-#define __P(params) params
-#else
-#define __P(params) ()
-#endif /* __STDC__ */
-#endif /* __P */
-#else /* SYS_CDEFS_DEFINES___P */
-#ifndef __P
-#ifdef HAVE_SYS_CDEFS_H
-#include <sys/cdefs.h>
-#endif /* HAVE_SYS_CDEFS_H */
-#endif /* __P */
-#endif /* SYS_CDEFS_DEFINES___P */
-
 #ifdef WIN32
 #define ENV_SEPARATOR ";"
 #define ENV_SEPARATOR_CHAR ';'
@@ -424,3 +416,10 @@
 #define ENV_SEPARATOR_CHAR ':'
 #define _CRTIMP
 #endif
+
+#include SYSTEM_INCLUDE_FILE
+#include MACHINE_INCLUDE_FILE
+
+#if defined(HAVE_NLIST) && defined(STRUCT_NLIST_HAS_N_VALUE) && !defined(DONT_USE_NLIST)
+#define CAN_USE_NLIST
+#endif
