@@ -1,6 +1,6 @@
 /*- This is a -*- C -*- compatible code file
  *
- * RCS $Id: kernel_sunos5.c,v 1.5 1997/11/12 18:27:46 hardaker Exp $
+ * RCS $Id: kernel_sunos5.c,v 1.13.4.3 1999/06/04 18:53:31 nba Exp $
  *
  * Code for SUNOS5_INSTRUMENTATION
  *
@@ -14,14 +14,11 @@
  */
 #include <config.h>
 #ifdef solaris2
-#define _SYS_PROC_H
 /*-
  * Includes of standard ANSI C header files 
  */
 
-#ifdef DODEBUG
 #include <stdio.h>
-#endif
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
@@ -33,9 +30,7 @@
 #include <fcntl.h>
 #include <stropts.h>
 #include <sys/types.h>
-#define _SYS_USER_H
 #include <kvm.h>
-#include <nlist.h>
 #include <sys/fcntl.h>
 #include <kstat.h>
 #include <errno.h>
@@ -57,6 +52,10 @@
  * Includes of local application header files 
  */
 
+#include "../../snmplib/system.h"
+#include "asn1.h"
+#include "snmp_debug.h"
+
 #include "kernel_sunos5.h"
 
 /*-
@@ -75,7 +74,7 @@ mibcache Mibcache[MIBCACHE_SIZE] = {
 {MIB_IP,		sizeof(mib2_ip_t),			(void *)-1, 0, 20, 0, 0},
 {MIB_IP_ADDR,		20*sizeof(mib2_ipAddrEntry_t),		(void *)-1, 0, 20, 0, 0},
 {MIB_IP_ROUTE,		200*sizeof(mib2_ipRouteEntry_t),	(void *)-1, 0, 10, 0, 0},
-{MIB_IP_NET,		20*sizeof(mib2_ipNetToMediaEntry_t),	(void *)-1, 0, 100, 0, 0},
+{MIB_IP_NET,		100*sizeof(mib2_ipNetToMediaEntry_t),	(void *)-1, 0, 100, 0, 0},
 {MIB_ICMP,		sizeof(mib2_icmp_t),			(void *)-1, 0, 20, 0, 0},
 {MIB_TCP,		sizeof(mib2_tcp_t),			(void *)-1, 0, 20, 0, 0},
 {MIB_TCP_CONN,		1000*sizeof(mib2_tcpConnEntry_t),	(void *)-1, 0, 15, 0, 0},
@@ -160,6 +159,25 @@ extern "C" {
    interface. :-(*/
 
 
+int getKstatInt(char *classname, char *statname, char *varname, int *value)
+{
+    kstat_ctl_t *ksc = kstat_open();
+    kstat_t *ks;
+    kid_t kid;
+    kstat_named_t *named;
+
+    if (ksc == NULL) return 0;
+    ks = kstat_lookup (ksc, classname, -1, statname);
+    if (ks == NULL) return 0;
+    kid = kstat_read (ksc, ks, NULL);
+    if (kid == -1) return 0;
+    named = kstat_data_lookup(ks, varname);
+    if (named == NULL) return 0;
+    *value = named->value.ul;
+    kstat_close(ksc);
+    return 1;
+}
+
 int
 getKstat(char *statname, char *varname, void *value)
 {
@@ -198,12 +216,11 @@ getKstat(char *statname, char *varname, void *value)
 
   /* Now, look for the name of our stat in the headers buf */
   for (i = 0; i < ks->ks_ndata; i++) {
-#ifdef DODEBUG
-    printf ("module: %s instance: %d name: %s class: %s type: %d flags: %x\n",
-	    kstat_data[i].ks_module, kstat_data[i].ks_instance, 
-	    kstat_data[i].ks_name, kstat_data[i].ks_class,
-	    kstat_data[i].ks_type, kstat_data[i].ks_flags);
-#endif
+    DEBUGMSGTL(("kernel_sunos5",
+                "module: %s instance: %d name: %s class: %s type: %d flags: %x\n",
+                kstat_data[i].ks_module, kstat_data[i].ks_instance, 
+                kstat_data[i].ks_name, kstat_data[i].ks_class,
+                kstat_data[i].ks_type, kstat_data[i].ks_flags));
     if (strcmp(statname, kstat_data[i].ks_name) == 0) {
       strcpy(module_name, kstat_data[i].ks_module);
       instance = kstat_data[i].ks_instance;
@@ -230,54 +247,44 @@ getKstat(char *statname, char *varname, void *value)
     goto Return;		/* Invalid stat type */
   }
   for (i = 0, d = KSTAT_NAMED_PTR(ks); i < ks->ks_ndata; i++, d++) {
-#ifdef DODEBUG
-    printf ("variable: %s %d\n", d->name, d->data_type);
-#endif
+    DEBUGMSGTL(("kernel_sunos5", "variable: %s %d\n", d->name, d->data_type));
     if (strcmp(d->name, varname) == 0) {
       switch (d->data_type) {
       case KSTAT_DATA_CHAR:
 	*(char *)v = (int)d->value.c;
-#ifdef DODEBUG
-	printf ("value: %d\n", (int)d->value.c);
-#endif
+	DEBUGMSGTL(("kernel_sunos5", "value: %d\n", (int)d->value.c));
 	break;
       case KSTAT_DATA_LONG:
 	*(long *)v = d->value.l;
-#ifdef DODEBUG
-	printf ("value: %ld\n", d->value.l);
-#endif
+	DEBUGMSGTL(("kernel_sunos5", "value: %ld\n", d->value.l));
 	break;
       case KSTAT_DATA_ULONG:
 	*(ulong_t *)v = d->value.ul;
-#ifdef DODEBUG
-	printf ("value: %lu\n", d->value.ul);
-#endif
+	DEBUGMSGTL(("kernel_sunos5", "value: %lu\n", d->value.ul));
 	break;
+#if KSTAT_DATA_LONGLONG != KSTAT_DATA_LONG
       case KSTAT_DATA_LONGLONG:
 	*(longlong_t *)v = d->value.ll;
-#ifdef DODEBUG
-	printf ("value: %ld\n", (long)d->value.ll);
-#endif
+	DEBUGMSGTL(("kernel_sunos5", "value: %ld\n", (long)d->value.ll));
 	break;
+#endif
+#if KSTAT_DATA_ULONGLONG != KSTAT_DATA_ULONG
       case KSTAT_DATA_ULONGLONG:
 	*(u_longlong_t *)v = d->value.ull;
-#ifdef DODEBUG
-	printf ("value: %lu\n", (unsigned long)d->value.ul);
-#endif
+	DEBUGMSGTL(("kernel_sunos5", "value: %lu\n", (unsigned long)d->value.ul));
 	break;
+#endif
       case KSTAT_DATA_FLOAT:
 	*(float *)v = d->value.f;
-#ifdef DODEBUG
-	printf ("value: %f\n", d->value.f);
-#endif
+	DEBUGMSGTL(("kernel_sunos5", "value: %f\n", d->value.f));
 	break;
       case KSTAT_DATA_DOUBLE:
 	*(double *)v = d->value.d;
-#ifdef DODEBUG
-	printf ("value: %f\n", d->value.d);
-#endif
+        DEBUGMSGTL(("kernel_sunos5", "value: %f\n", d->value.d));
 	break;
       default:
+	fprintf(stderr, "Unknown type in kstat data: %s %s %d\n",
+		statname, varname, d->data_type);
 	ret = -3;
 	goto Return;		/* Invalid data type */
       }
@@ -313,9 +320,7 @@ getMibstat(mibgroup_e grid,  void *resp, int entrysize,
      * We assume that Mibcache is initialized in mibgroup_e enum order
      * so we don't check the validity of index here.
      */
-#ifdef DODEBUG
-    printf ("getMibstat (%d, *, %d, %d, *, *)\n", grid, entrysize, req_type);
-#endif
+    DEBUGMSGTL(("kernel_sunos5", "getMibstat (%d, *, %d, %d, *, *)\n", grid, entrysize, req_type));
     cachep = &Mibcache[grid];
     mibgr = Mibmap[grid].group;
     mibtb = Mibmap[grid].table;
@@ -330,10 +335,9 @@ getMibstat(mibgroup_e grid,  void *resp, int entrysize,
     if (req_type != GET_NEXT)
       cachep->cache_last_found = 0;
     cache_valid = (time(NULL) - cachep->cache_time) > cachep->cache_ttl ? 0 : 1;
-#ifdef DODEBUG
-    printf ("... cache_valid %d time %ld ttl %d now %ld\n",
-            cache_valid, cachep->cache_time, cachep->cache_ttl, time (NULL));
-#endif
+    DEBUGMSGTL(("kernel_sunos5", "... cache_valid %d time %ld ttl %d now %ld\n",
+                cache_valid, cachep->cache_time, cachep->cache_ttl,
+                time (NULL)));
     if (cache_valid) {
 	/* Is it really? */
 	if (cachep->cache_comp != (void *) comp ||
@@ -381,9 +385,7 @@ getMibstat(mibgroup_e grid,  void *resp, int entrysize,
 	    cachep->cache_arg = NULL;
 	}
     }
-#ifdef DODEBUG
-    printf ("... result %d rc %d\n", result, rc);
-#endif
+    DEBUGMSGTL(("kernel_sunos5", "... result %d rc %d\n", result, rc));
     if (result == FOUND || rc == 0 || rc == 1) {
 	/* Entry has been found, deliver it */
 	if (resp != (void *)NULL) 
@@ -392,9 +394,7 @@ getMibstat(mibgroup_e grid,  void *resp, int entrysize,
 	cachep->cache_last_found = ((char *) ep - (char *) cachep->cache_addr) / entrysize;
     } else
 	ret = 1;		/* Not found */
-#ifdef DODEBUG
-    printf ("... getMibstat returns %d\n", ret);
-#endif
+    DEBUGMSGTL(("kernel_sunos5", "... getMibstat returns %d\n", ret));
     return (ret);
 }
 
@@ -420,13 +420,14 @@ getentry(req_e req_type, void *bufaddr, int len, int entrysize,
 	    ((req_type == GET_NEXT) && previous_found)) {
 	    return (FOUND);
 	}
-	if ((*comp)(arg, bp) == 0)
+	if ((*comp)(arg, bp) == 0) {
 	    if (req_type == GET_EXACT) {
 		return (FOUND);
 	    } else {			/* GET_NEXT */
 		previous_found++;
 		continue;
 	    }
+        }
     }
     if (previous_found)
 	return (NEED_NEXT);
@@ -486,9 +487,7 @@ getmib(int groupname, int subgroupname, void *statbuf, size_t size, int entrysiz
   struct 	opthdr *req;
   found_e	result = FOUND;
 
-#ifdef DODEBUG
-  printf ("...... getmib (%d, %d, ...)\n", groupname, subgroupname);
-#endif
+  DEBUGMSGTL(("kernel_sunos5", "...... getmib (%d, %d, ...)\n", groupname, subgroupname));
 
   /* Open the stream driver and push all MIB-related modules */
 
@@ -509,9 +508,7 @@ getmib(int groupname, int subgroupname, void *statbuf, size_t size, int entrysiz
       ret = -1;
       goto Return;
     }
-#ifdef DODEBUG
-    printf ("...... modules pushed OK\n");
-#endif
+    DEBUGMSGTL(("kernel_sunos5", "...... modules pushed OK\n"));
   }
 
   /* First, use bigger buffer, to accelerate skipping
@@ -596,7 +593,7 @@ getmib(int groupname, int subgroupname, void *statbuf, size_t size, int entrysiz
 
       case MOREDATA:
       case 0:
-	if ((req_type == GET_NEXT)  && (result == NEED_NEXT))
+	if (req_type == GET_NEXT  && result == NEED_NEXT)
 	    /* End of buffer, so "next" is the first item in the next buffer */
 	    req_type = GET_FIRST;
 	result = getentry(req_type, (void *)strbuf.buf, strbuf.len, entrysize,
@@ -604,7 +601,7 @@ getmib(int groupname, int subgroupname, void *statbuf, size_t size, int entrysiz
 	*length = strbuf.len;	/* To use in caller for cacheing */
 	break;
       }
-    } while ((rc == MOREDATA) && (result != FOUND));
+    } while (rc == MOREDATA && result != FOUND);
     if (result == FOUND) {	/* Search is successful */
 	if (rc != MOREDATA)
 	    ret = 0;		/* Found and no more data */
@@ -618,9 +615,7 @@ getmib(int groupname, int subgroupname, void *statbuf, size_t size, int entrysiz
   }
  Return:
   (void)ioctl(sd, I_FLUSH, FLUSHRW);
-#ifdef DODEBUG
-  printf ("...... getmib returns %d\n", ret);
-#endif
+  DEBUGMSGTL(("kernel_sunos5", "...... getmib returns %d\n", ret));
   return(ret);
 }
 
@@ -634,7 +629,7 @@ getif(mib2_ifEntry_t *ifbuf, size_t size, req_e req_type,
 {
     int 		i, ret, idx = 1;
     int 		sd;
-    char 		buf[1024];
+    char 		buf[10240];
     struct ifconf 	ifconf;
     struct ifreq 	*ifrp;
     mib2_ifEntry_t	*ifp;
@@ -647,6 +642,7 @@ getif(mib2_ifEntry_t *ifbuf, size_t size, req_e req_type,
     ifconf.ifc_buf = buf;
     ifconf.ifc_len = sizeof(buf);
     if (ioctl(sd, SIOCGIFCONF, &ifconf) == -1) {
+	fprintf (stderr, "cannot SIOCGIFCONF - increase buffer size\n");
 	ret = -1;
 	goto Return;
     }
@@ -654,14 +650,10 @@ getif(mib2_ifEntry_t *ifbuf, size_t size, req_e req_type,
     for (i = 0, ifp = (mib2_ifEntry_t *)ifbuf, ifrp = ifconf.ifc_req;
 	 ((char *)ifrp < ((char *)ifconf.ifc_buf + ifconf.ifc_len)) && (i < nentries);
 	 i++, ifp++, ifrp++, idx++) {
-#ifdef DODEBUG
-	printf ("...... getif %s\n", ifrp->ifr_name);
-#endif
+        DEBUGMSGTL(("kernel_sunos5", "...... getif %s\n", ifrp->ifr_name));
 	if (ioctl(sd, SIOCGIFFLAGS, ifrp) < 0) {
 	    ret = -1;
-#ifdef DODEBUG
-	    printf ("...... SIOCGIFFLAGS failed\n");
-#endif
+	    DEBUGMSGTL(("kernel_sunos5", "...... SIOCGIFFLAGS failed\n"));
 	    goto Return;
 	}
 	(void)memset(ifp, 0, sizeof(mib2_ifEntry_t));
@@ -673,9 +665,7 @@ getif(mib2_ifEntry_t *ifbuf, size_t size, req_e req_type,
 	ifp->ifLastChange = 0;		/* Who knows ...  */
 	if (ioctl(sd, SIOCGIFMTU, ifrp) < 0) {
 	    ret = -1;
-#ifdef DODEBUG
-	    printf ("...... SIOCGIFMTU failed\n");
-#endif
+	    DEBUGMSGTL(("kernel_sunos5", "...... SIOCGIFMTU failed\n"));
 	    goto Return;
 	}
 	ifp->ifMtu = ifrp->ifr_metric;
@@ -819,7 +809,7 @@ Name_cmp(void *ifrp, void *ep)
 
 #ifdef _GETKSTAT_TEST
 
-void
+int
 main (int argc, char **argv)
 {
   int rc = 0;
@@ -830,12 +820,14 @@ main (int argc, char **argv)
     exit(1);
   }
 
+  snmp_set_do_debugging(1);
   rc = getKstat(argv[1], argv[2], &val);
 
   if (rc == 0)
     fprintf(stderr, "%s = %lu\n", argv[2], val);
   else
     fprintf(stderr, "rc =%d\n", rc);
+  return 0;
 }
 #endif /*_GETKSTAT_TEST */
 
@@ -852,6 +844,9 @@ ip20comp(void *ifname, void *ipp)
 int
 ARP_Cmp_Addr(void *addr, void *ep)
 {
+  fprintf(stderr, "ARP: %lx <> %lx\n",
+      ((mib2_ipNetToMediaEntry_t *)ep)->ipNetToMediaNetAddress,
+      *(IpAddress *)addr);
   if (((mib2_ipNetToMediaEntry_t *)ep)->ipNetToMediaNetAddress ==
       *(IpAddress *)addr)
     return (0);
@@ -867,7 +862,7 @@ IF_cmp(void *addr, void *ep)
 	return (1);
 }
 
-void
+int
 main (int argc, char **argv)
 {
   int rc = 0, i, idx;
@@ -875,13 +870,14 @@ main (int argc, char **argv)
   mib2_ipNetToMediaEntry_t entry, *ep = &entry;
   mib2_ifEntry_t	ifstat;
   req_e 		req_type;
-  IpAddress		LastAddr = -1;
+  IpAddress		LastAddr = 0;
 
   if (argc != 3) {
     fprintf(stderr, "Usage: %s if_name req_type (0 first, 1 exact, 2 next) \n",
 	    argv[0]);
     exit(1);
   }
+
   switch (atoi(argv[2])) {
   case 0:
     req_type = GET_FIRST;
@@ -893,6 +889,8 @@ main (int argc, char **argv)
     req_type = GET_NEXT;
     break;
   };
+
+  snmp_set_do_debugging(0);
   while ((rc = getMibstat(MIB_INTERFACES, &ifstat, sizeof(mib2_ifEntry_t),
 			  req_type, &IF_cmp, &idx)) == 0) {
       idx = ifstat.ifIndex;
@@ -913,7 +911,7 @@ main (int argc, char **argv)
     fprintf(stdout, "Ipaddr = %lX\n", (u_long)LastAddr);
     req_type = GET_NEXT;
   }
-
+  return 0;
 }	
 #endif /*_GETMIBSTAT_TEST */
 #endif /* SUNOS5 */
