@@ -48,48 +48,46 @@ SOFTWARE.
 #endif
 
 #include "asn1.h"
-#include "snmp_impl.h"
 #include "snmp_api.h"
+#include "snmp_impl.h"
 #include "snmp.h"
 #include "acl.h"
 #include "party.h"
 #include "context.h"
 #include "mib.h"
-#include "mibgroup/snmp_mib.h"
+#include "snmp_vars.h"
+#if USING_MIBII_SNMP_MIB_MODULE
+#include "mibgroup/mibII/snmp_mib.h"
+#endif
 #include "snmpd.h"
+#include "mibgroup/struct.h"
 #include "mibgroup/util_funcs.h"
 #include "snmp_agent.h"
+#include "var_struct.h"
 #include "read_config.h"
+#include "snmp_debug.h"
 #include "mib_module_config.h"
+#if USING_MIBII_VACM_VARS_MODULE
+#include "mibgroup/mibII/vacm_vars.h"
+#endif
 
-static int create_identical __P((u_char *, u_char *, int, long, long, struct packet_info *));
-static int parse_var_op_list __P((u_char *, int, u_char *, int, long *, struct packet_info *, int));
-static int snmp_access __P((u_short, int, int));
+static int create_identical (u_char *, u_char *, int, long, long, struct packet_info *);
+static int parse_var_op_list (u_char *, int, u_char *, int, long *, struct packet_info *, int);
 static int snmp_vars_inc;
-static int get_community __P((u_char *, int));
-static int bulk_var_op_list __P((u_char *, int, u_char *, int, int, int, long *, struct packet_info *));
-static int create_toobig __P((u_char *, int, long, struct packet_info *));
-static int goodValue __P((u_char, int, u_char, int));
-static void setVariable __P((u_char *, u_char, int, u_char *, int));
-static void dump_var __P((oid *, int, int, void *, int));
+static int bulk_var_op_list (u_char *, int, u_char *, int, int, int, long *, struct packet_info *);
+static int create_toobig (u_char *, int, long, struct packet_info *);
+static int goodValue (u_char, int, u_char, int);
+static void setVariable (u_char *, u_char, int, u_char *, int);
+static void dump_var (oid *, int, int, void *, int);
 
 
-char	communities[NUM_COMMUNITIES][COMMUNITY_MAX_LEN] = {
-    "public", 
-    "private",
-    "regional",
-    "proxy",
-    "core"
-};
-
-static void dump_var (var_name, var_name_len, statType, statP, statLen)
-    oid *var_name;
-    int var_name_len;
-    int statType;
-    void *statP;
-    int statLen;
+static void dump_var (oid *var_name,
+		      int var_name_len,
+		      int statType,
+		      void *statP,
+		      int statLen)
 {
-    char buf [2560];
+    char buf [SPRINT_MAX_LEN];
     struct variable_list temp_var;
 
     temp_var.type = statType;
@@ -100,12 +98,11 @@ static void dump_var (var_name, var_name_len, statType, statP, statLen)
 }
 
 int
-snmp_agent_parse(data, length, out_data, out_length, sourceip)
-    register u_char	*data;
-    int			length;
-    register u_char	*out_data;
-    int			*out_length;
-    u_long		sourceip;	/* possibly for authentication */
+snmp_agent_parse(u_char	*data,
+		 int length,
+		 u_char *out_data,
+		 int *out_length,
+		 u_long sourceip)	/* possibly for authentication */
 {
     struct packet_info packet, *pi = &packet;
     u_char	    type;
@@ -120,12 +117,24 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
     len = length;
     (void)asn_parse_header(data, &len, &type);
 
+    pi->source.sin_addr.s_addr = sourceip;
     if (type == (ASN_SEQUENCE | ASN_CONSTRUCTOR)){
         /* authenticates message and returns length if valid */
 	pi->community_len = COMMUNITY_MAX_LEN;
         data = snmp_comstr_parse(data, &length,
 			       pi->community, &pi->community_len,
-                               (long *) &pi->version);
+                               &pi->version);
+	switch (pi->version) {
+	case SNMP_VERSION_1:
+	    pi->mp_model = SNMP_MP_MODEL_SNMPv1;
+	    pi->sec_model = SNMP_SEC_MODEL_SNMPv1;
+	    break;
+	case SNMP_VERSION_2c:
+	    pi->mp_model = SNMP_MP_MODEL_SNMPv2c;
+	    pi->sec_model = SNMP_SEC_MODEL_SNMPv2c;
+	    break;
+	}
+	pi->sec_level = SNMP_SEC_LEVEL_NOAUTH;
     } else if (type == (ASN_CONTEXT | ASN_CONSTRUCTOR | 1)){
         pi->srcPartyLength = sizeof(pi->srcParty)/sizeof(oid);
         pi->dstPartyLength = sizeof(pi->dstParty)/sizeof(oid);
@@ -138,7 +147,7 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
 				  pi->context, &pi->contextLength,
 				  FIRST_PASS);
     } else {
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
         snmp_inbadversions++;
 #endif
         ERROR_MSG("unknown auth header type");
@@ -147,29 +156,11 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
 
     if (data == NULL){
 	ERROR_MSG("bad authentication");
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	snmp_inasnparseerrors++;
 #endif
 	return 0;
     }
-    if (pi->version == SNMP_VERSION_1 || pi->version == SNMP_VERSION_2c){
-	pi->community_id = get_community(pi->community, pi->community_len);
-	if (pi->community_id == -1) {
-#ifdef USING_SNMP_MODULE       
-	    snmp_inbadcommunitynames++;
-#endif
-	    send_easy_trap(4);
-	    return 0;
-	}
-    } else if (pi->version == SNMP_VERSION_2p){
-	pi->community_id = 0;
-    } else {
-	ERROR_MSG("Bad Version");
-#ifdef USING_SNMP_MODULE       
-	snmp_inbadversions++;
-#endif
-	return 0;
-    }
 
     data = asn_parse_header(data, &length, &pi->pdutype);
     if (data == NULL){
@@ -178,39 +169,39 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
     }
 
     switch (pi->pdutype) {
-    case GET_REQ_MSG:
-#ifdef USING_SNMP_MODULE       
+    case SNMP_MSG_GET:
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	snmp_ingetrequests++;
 #endif
 	break;
-    case BULK_REQ_MSG:
-#ifdef USING_SNMP_MODULE       
+    case SNMP_MSG_GETBULK:
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	snmp_ingetrequests++;
 #endif
 	break;
-    case GETNEXT_REQ_MSG:
-#ifdef USING_SNMP_MODULE       
+    case SNMP_MSG_GETNEXT:
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	snmp_ingetnexts++;
 #endif
 	break;
-    case SET_REQ_MSG:
-#ifdef USING_SNMP_MODULE       
+    case SNMP_MSG_SET:
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	snmp_insetrequests++;
 #endif
 	break;
-    case GET_RSP_MSG:
-#ifdef USING_SNMP_MODULE       
+    case SNMP_MSG_RESPONSE:
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
         snmp_ingetresponses++;
 #endif
 	return 0;
-    case TRP_REQ_MSG:
-    case TRP2_REQ_MSG:
-#ifdef USING_SNMP_MODULE       
+    case SNMP_MSG_TRAP:
+    case SNMP_MSG_TRAP2:
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
         snmp_intraps++;
 #endif
 	return 0;
     default:
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
         snmp_inasnparseerrors++;
 #endif
 	return 0;
@@ -284,7 +275,7 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
     data = asn_parse_int(data, &length, &type, &reqid, sizeof(reqid));
     if (data == NULL){
 	ERROR_MSG("bad parse of reqid");
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	snmp_inasnparseerrors++;
 #endif
 	return 0;
@@ -292,7 +283,7 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
     data = asn_parse_int(data, &length, &type, &errstat, sizeof(errstat));
     if (data == NULL){
 	ERROR_MSG("bad parse of errstat");
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	snmp_inasnparseerrors++;
 #endif
 	return 0;
@@ -300,7 +291,7 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
     data = asn_parse_int(data, &length, &type, &errindex, sizeof(errindex));
     if (data == NULL){
 	ERROR_MSG("bad parse of errindex");
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	snmp_inasnparseerrors++;
 #endif
 	return 0;
@@ -309,17 +300,17 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
     if (verbose) {
 	fprintf (stdout, "    ");
 	switch (pi->pdutype) {
-	case GET_REQ_MSG:
+	case SNMP_MSG_GET:
 	    fprintf (stdout, "GET");
 	    break;
-	case GETNEXT_REQ_MSG:
+	case SNMP_MSG_GETNEXT:
 	    fprintf (stdout, "GETNEXT");
 	    break;
-	case BULK_REQ_MSG:
+	case SNMP_MSG_GETBULK:
 	    fprintf (stdout, "GETBULK non-rep = %ld, max-rep = %ld",
 		     errstat, errindex);
 	    break;
-	case SET_REQ_MSG:
+	case SNMP_MSG_SET:
 	    fprintf (stdout, "SET");
 	    break;
 	}
@@ -328,7 +319,7 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
 
     /* create the requid, errstatus, errindex for the output packet */
     out_reqid = asn_build_sequence(out_header, out_length,
-				 (u_char)GET_RSP_MSG, 0);
+				 (u_char)SNMP_MSG_RESPONSE, 0);
     if (out_reqid == NULL){
 	ERROR_MSG("");
 	return 0;
@@ -357,13 +348,13 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
 	return 0;
     }
 
-    if (pi->pdutype == BULK_REQ_MSG)
+    if (pi->pdutype == SNMP_MSG_GETBULK)
 	errstat = bulk_var_op_list(data, length, out_data, *out_length,
 				    errstat, errindex, &errindex, pi);
     else
 	errstat = parse_var_op_list(data, length, out_data, *out_length,
 			    &errindex, pi, RESERVE1);
-    if (pi->pdutype == SET_REQ_MSG){
+    if (pi->pdutype == SNMP_MSG_SET){
 	if (errstat == SNMP_ERR_NOERROR)
 	    errstat = parse_var_op_list(data, length, out_data, *out_length,
 					&errindex, pi, RESERVE2);
@@ -371,21 +362,23 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
     	    /*
 	     * SETS require 3-4 passes through the var_op_list.  The first two
 	     * passes verify that all types, lengths, and values are valid
-	     * and may reserve resources and the third does the set and a
-	     * fourth executes any actions.  Then the identical GET RESPONSE
+	     * and may reserve resources and the third does the set provisionally
+	     * and a fourth confirms this.  Then the identical GET RESPONSE
 	     * packet is returned.
 	     * If either of the first two passes returns an error, another
 	     * pass is made so that any reserved resources can be freed.
+	     * If the third passes returns an error, another pass is made so
+	     * that the changes can be backed out.
 	     */
               errstat = parse_var_op_list(data, length, out_data, *out_length,
-				&dummyindex, pi, COMMIT);
+				&dummyindex, pi, ACTION);
 	      parse_var_op_list(data, length, out_data, *out_length,
 				&dummyindex, pi,
-                                (errstat == SNMP_ERR_NOERROR) ? ACTION : FREE);
+                                (errstat == SNMP_ERR_NOERROR) ? COMMIT : UNDO);
               if (errstat == SNMP_ERR_NOERROR) {
                 if (create_identical(startData, out_auth, startLength, 0L, 0L, pi)){
 		  *out_length = pi->packet_end - out_auth;
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 		  snmp_outgetresponses++;
 		  snmp_intotalsetvars += snmp_vars_inc;
 #endif
@@ -402,7 +395,7 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
 	case SNMP_ERR_NOERROR:
 	    /* re-encode the headers with the real lengths */
 	    *out_length = pi->packet_end - out_header;
-	    out_data = asn_build_sequence(out_header, out_length, GET_RSP_MSG,
+	    out_data = asn_build_sequence(out_header, out_length, SNMP_MSG_RESPONSE,
 					pi->packet_end - out_reqid);
 	    if (out_data != out_reqid){
 		ERROR_MSG("internal error: header");
@@ -432,13 +425,13 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
 	       to be fixed. */
 	    if (pi->version == SNMP_VERSION_2p)
 		pi->packet_end = out_auth + packet_len;
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	    snmp_intotalreqvars += snmp_vars_inc;
 	    snmp_outgetresponses++;
 #endif
 	    break;
 	case SNMP_ERR_TOOBIG:
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	    snmp_intoobigs++;
 #endif
 	    if (pi->version == SNMP_VERSION_2p){
@@ -460,22 +453,22 @@ snmp_agent_parse(data, length, out_data, out_length, sourceip)
 	case SNMP_ERR_NOTWRITABLE:
 	    goto reterr;
 	case SNMP_ERR_NOSUCHNAME:
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	    snmp_outnosuchnames++;
 #endif
 	    goto reterr;
 	case SNMP_ERR_BADVALUE:
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	    snmp_inbadvalues++;
 #endif
 	    goto reterr;
 	case SNMP_ERR_READONLY:
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	    snmp_inreadonlys++;
 #endif
 	    goto reterr;
 	case SNMP_ERR_GENERR:
-#ifdef USING_SNMP_MODULE       
+#ifdef USING_MIBII_SNMP_MIB_MODULE       
 	    snmp_ingenerrs++;
 #endif
 reterr:
@@ -503,33 +496,33 @@ reterr:
  * If any error occurs, an error code is returned.
  */
 static int
-parse_var_op_list(data, length, out_data, out_length, index, pi, action)
-    register u_char	*data;
-    int			length;
-    register u_char	*out_data;
-    int			out_length;
-    register long	*index;
-    struct packet_info	*pi;
-    int			action;
+parse_var_op_list(u_char *data,
+		  int length,
+		  u_char *out_data,
+		  int out_length,
+		  long *index,
+		  struct packet_info *pi,
+		  int action)
 {
     u_char  type;
-    oid	    var_name[MAX_NAME_LEN];
+    oid	    var_name[MAX_OID_LEN];
     int	    var_name_len, var_val_len;
     u_char  var_val_type, *var_val, statType;
     register u_char *statP;
     int	    statLen;
     u_short acl;
     int	    rw, exact, err;
-    int	    (*write_method) __P((int, u_char *, u_char, int, u_char *, oid *, int));
+    WriteMethod *write_method;
     u_char  *headerP, *var_list_start;
     int	    dummyLen;
     int	    noSuchObject;
+    char c_oid[SPRINT_MAX_LEN];
 
-    if (pi->pdutype == SET_REQ_MSG)
+    if (pi->pdutype == SNMP_MSG_SET)
 	rw = WRITE;
     else
 	rw = READ;
-    if (pi->pdutype == GETNEXT_REQ_MSG){
+    if (pi->pdutype == SNMP_MSG_GETNEXT){
 	exact = FALSE;
     } else {
 	exact = TRUE;
@@ -555,47 +548,44 @@ parse_var_op_list(data, length, out_data, out_length, index, pi, action)
     *index = 1;
     while((int)length > 0){
 	/* parse the name, value pair */
-	var_name_len = MAX_NAME_LEN;
+	var_name_len = MAX_OID_LEN;
 	data = snmp_parse_var_op(data, var_name, &var_name_len, &var_val_type,
 				 &var_val_len, &var_val, (int *)&length);
 	if (data == NULL)
 	    return PARSE_ERROR;
 
 	if (verbose && action == RESERVE1) {
-	    char buf [256];
-	    sprint_objid (buf, var_name, var_name_len);
-	    fprintf (stdout, "    -- %s\n", buf);
+	    sprint_objid (c_oid, var_name, var_name_len);
+	    fprintf (stdout, "    -- %s\n", c_oid);
 	}
 
 	/* now attempt to retrieve the variable on the local entity */
 	statP = getStatPtr(var_name, &var_name_len, &statType, &statLen, &acl,
 			   exact, &write_method, pi, &noSuchObject);
-	if (pi->version == SNMP_VERSION_1 && statP == NULL
-	      && (pi->pdutype != SET_REQ_MSG || !write_method)){
+	if (statP == NULL && pi->pdutype != SNMP_MSG_SET) {
 	    if (verbose) fprintf (stdout, "    >> noSuchName\n");
 	    else {
-		char buf [256];
-		sprint_objid(buf, var_name, var_name_len);
-		printf("%s --  OID Doesn't exist\n", buf);
+		sprint_objid(c_oid, var_name, var_name_len);
+		DEBUGMSGTL(("snmp_agent",
+                            "%s(%s) --  OID Doesn't exist or access is denied\n",
+                            exact ? "GET" : "GETNEXT", c_oid));
 	    }
 	    return SNMP_ERR_NOSUCHNAME; 
 	}
 
 	/* Effectively, check if this variable is read-only or read-write
 	   (in the MIB sense). */
-	if (pi->pdutype == SET_REQ_MSG && pi->version == SNMP_VERSION_1
-	      && !snmp_access(acl, pi->community_id, rw)){
-	    if (verbose) fprintf (stdout, "    >> noSuchName (read-only)\n");
-	    ERROR_MSG("read-only? (ignoring)");
-	    return SNMP_ERR_NOSUCHNAME;
-	}
-	if (pi->pdutype == SET_REQ_MSG &&
-            (pi->version == SNMP_VERSION_2p ||
-             pi->version == SNMP_VERSION_2c) && 
-            !snmp_access(acl, pi->community_id, rw)){
-	    if (verbose) fprintf (stdout, "    >> notWritable\n");
-	    ERROR_MSG("Not Writable");
-	    return SNMP_ERR_NOTWRITABLE;
+	if ((pi->pdutype == SNMP_MSG_SET && !(acl & 2))) {
+	    if (pi->version == SNMP_VERSION_1 || pi->pdutype != SNMP_MSG_SET) {
+		if (verbose) fprintf (stdout, "    >> noSuchName (read-only)\n");
+		ERROR_MSG("read-only");
+		return SNMP_ERR_NOSUCHNAME;
+	    }
+	    else {
+		if (verbose) fprintf (stdout, "    >> notWritable\n");
+		ERROR_MSG("Not Writable");
+		return SNMP_ERR_NOTWRITABLE;
+	    }
 	}
 
 	/* Its bogus to check here on getnexts - the whole packet shouldn't
@@ -603,7 +593,7 @@ parse_var_op_list(data, length, out_data, out_length, index, pi, action)
 	   luckily no objects are set unreadable.  This can still be
 	   useful for sets to determine which are intrinsically writable */
 
-	if (pi->pdutype == SET_REQ_MSG){
+	if (pi->pdutype == SNMP_MSG_SET){
 	    if (write_method == NULL){
 		if (statP != NULL){
 		    /* see if the type and value is consistent with this
@@ -616,14 +606,14 @@ parse_var_op_list(data, length, out_data, out_length, index, pi, action)
 			else
 			    return SNMP_ERR_BADVALUE;
 		    }
-		    /* actually do the set if necessary */
+		    /* actually do the set if nescessary */
 		    if (action == COMMIT)
 			setVariable(var_val, var_val_type, var_val_len,
 				    statP, statLen);
 		} else {
 		    if (pi->version == SNMP_VERSION_2p
                         || pi->version == SNMP_VERSION_2c)
-			return SNMP_ERR_NOCREATION;
+			return SNMP_ERR_NOACCESS; /* NOCREATION? */
 		    else
 			return SNMP_ERR_NOSUCHNAME;
 		}
@@ -663,7 +653,7 @@ parse_var_op_list(data, length, out_data, out_length, index, pi, action)
 	(*index)++;
 	snmp_vars_inc++;
     }
-    if (pi->pdutype != SET_REQ_MSG){
+    if (pi->pdutype != SNMP_MSG_SET){
 	/* save a pointer to the end of the packet */
         pi->packet_end = out_data;
 
@@ -680,7 +670,7 @@ parse_var_op_list(data, length, out_data, out_length, index, pi, action)
 }
 
 struct repeater {
-    oid	name[MAX_NAME_LEN];
+    oid	name[MAX_OID_LEN];
     int length;
 } repeaterList[10];
 
@@ -695,25 +685,23 @@ struct repeater {
  * If any error occurs, an error code is returned.
  */
 static int
-bulk_var_op_list(data, length, out_data, out_length, non_repeaters,
-		 max_repetitions, index, pi)
-    register u_char	*data;
-    int			length;
-    register u_char	*out_data;
-    int			out_length;
-    int			non_repeaters;
-    int			max_repetitions;
-    register long	*index;
-    struct packet_info	*pi;
+bulk_var_op_list(u_char *data,
+		 int length,
+		 u_char *out_data,
+		 int out_length,
+		 int non_repeaters,
+		 int max_repetitions,
+		 long *index,
+		 struct packet_info *pi)
 {
     u_char  type;
-    oid	    var_name[MAX_NAME_LEN];
+    oid	    var_name[MAX_OID_LEN];
     int	    var_name_len, var_val_len;
     u_char  var_val_type, *var_val, statType;
     register u_char *statP;
     int	    statLen;
     u_short acl;
-    int	    (*write_method) __P((int, u_char *, u_char, int, u_char *, oid *, int));
+    WriteMethod *write_method;
     u_char  *headerP, *var_list_start;
     int	    dummyLen;
     u_char  *repeaterStart, *out_data_save;
@@ -722,6 +710,7 @@ bulk_var_op_list(data, length, out_data, out_length, non_repeaters,
     int	    noSuchObject, useful;
     int repeaterIndex, repeaterCount;
     struct repeater *rl;
+    char c_oid[SPRINT_MAX_LEN];
 
     if (non_repeaters < 0)
 	non_repeaters = 0;
@@ -756,16 +745,15 @@ bulk_var_op_list(data, length, out_data, out_length, non_repeaters,
     while((int)length > 0 && non_repeaters > 0){
 	/* parse the name, value pair */
 	
-	var_name_len = MAX_NAME_LEN;
+	var_name_len = MAX_OID_LEN;
 	data = snmp_parse_var_op(data, var_name, &var_name_len, &var_val_type,
 				 &var_val_len, &var_val, (int *)&length);
 	if (data == NULL)
 	    return PARSE_ERROR;
 
 	if (verbose) {
-	    char buf [256];
-	    sprint_objid (buf, var_name, var_name_len);
-	    fprintf (stdout, "    non-rep -- %s\n", buf);
+	    sprint_objid (c_oid, var_name, var_name_len);
+	    fprintf (stdout, "    non-rep -- %s\n", c_oid);
 	}
 
 	/* now attempt to retrieve the variable on the local entity */
@@ -801,7 +789,7 @@ bulk_var_op_list(data, length, out_data, out_length, non_repeaters,
     useful = FALSE;
     while((int)length > 0){
 	/* parse the name, value pair */
-	rl->length = MAX_NAME_LEN;
+	rl->length = MAX_OID_LEN;
 	data = snmp_parse_var_op(data, rl->name, &rl->length,
 				 &var_val_type, &var_val_len, &var_val,
 				 (int *)&length);
@@ -809,9 +797,8 @@ bulk_var_op_list(data, length, out_data, out_length, non_repeaters,
 	    return PARSE_ERROR;
 
 	if (verbose) {
-	    char buf [256];
-	    sprint_objid (buf, rl->name, rl->length);
-	    fprintf (stdout, "    rep -- %s\n", buf);
+	    sprint_objid (c_oid, rl->name, rl->length);
+	    fprintf (stdout, "    rep -- %s\n", c_oid);
 	}
 
 	/* now attempt to retrieve the variable on the local entity */
@@ -855,7 +842,7 @@ bulk_var_op_list(data, length, out_data, out_length, non_repeaters,
 	while((repeaterIndex++ < repeaterCount) > 0 && !full){
 	    /* parse the name, value pair */
 #if 0
-	    var_name_len = MAX_NAME_LEN;
+	    var_name_len = MAX_OID_LEN;
 	    data = snmp_parse_var_op(data, var_name, &var_name_len,
 				     &var_val_type, &var_val_len, &var_val,
 				     (int *)&length);
@@ -910,12 +897,12 @@ bulk_var_op_list(data, length, out_data, out_length, non_repeaters,
  * Returns 1 upon success and 0 upon failure.
  */
 static int
-create_identical(snmp_in, snmp_out, snmp_length, errstat, errindex, pi)
-    u_char	    	*snmp_in;
-    u_char	    	*snmp_out;
-    int		    	snmp_length;
-    long	    	errstat, errindex;
-    struct packet_info 	*pi;
+create_identical(u_char *snmp_in,
+		 u_char *snmp_out,
+		 int snmp_length,
+		 long errstat,
+		 long errindex,
+		 struct packet_info *pi)
 {
     register u_char *data;
     u_char	    type;
@@ -935,7 +922,7 @@ create_identical(snmp_in, snmp_out, snmp_length, errstat, errindex, pi)
 	pi->community_len = COMMUNITY_MAX_LEN;
         headerPtr = snmp_comstr_parse(snmp_in, &length,
 				    pi->community, &pi->community_len,
-				    (long *)&pi->version);
+				    &pi->version);
     } else if (type == (ASN_CONTEXT | ASN_CONSTRUCTOR | 1)){
         pi->srcPartyLength = sizeof(pi->srcParty)/sizeof(oid);
         pi->dstPartyLength = sizeof(pi->dstParty)/sizeof(oid);
@@ -980,7 +967,7 @@ create_identical(snmp_in, snmp_out, snmp_length, errstat, errindex, pi)
 	return 0;
 
 #if 0
-    data = asn_build_header(headerPtr, &headerLength, GET_RSP_MSG,
+    data = asn_build_header(headerPtr, &headerLength, SNMP_MSG_RESPONSE,
 			    headerLength);
     if (data != reqidPtr)
 	return 0;
@@ -990,7 +977,7 @@ create_identical(snmp_in, snmp_out, snmp_length, errstat, errindex, pi)
      * this code as a long-term solution (we should re-encode the error/set
      * reply packet).
      */
-    *headerPtr = GET_RSP_MSG;
+    *headerPtr = SNMP_MSG_RESPONSE;
 #endif
     
     length = snmp_length;
@@ -1007,7 +994,7 @@ create_identical(snmp_in, snmp_out, snmp_length, errstat, errindex, pi)
     if (pi->version == SNMP_VERSION_1 || pi->version == SNMP_VERSION_2c){
 	data = snmp_comstr_build(snmp_out, (int *)&dummy,
 			       pi->community, &pi->community_len,
-			       (int *)&pi->version, messagelen);
+			       &pi->version, messagelen);
     } else if (pi->version == SNMP_VERSION_2p){
 	data = snmp_party_build(snmp_out, (int *)&dummy, pi, messagelen,
 				  pi->dstParty, pi->dstPartyLength,
@@ -1039,11 +1026,10 @@ create_identical(snmp_in, snmp_out, snmp_length, errstat, errindex, pi)
 }
 
 static int
-create_toobig(snmp_out, snmp_length, reqid, pi)
-    u_char	    	*snmp_out;
-    int		    	snmp_length;
-    long	    	reqid;
-    struct packet_info 	*pi;
+create_toobig(u_char *snmp_out,
+	      int snmp_length,
+	      long reqid,
+	      struct packet_info *pi)
 {
     register u_char *data;
     u_char	    type;
@@ -1062,7 +1048,7 @@ create_toobig(snmp_out, snmp_length, reqid, pi)
     if (data == NULL)
 	return 0;
     headerPtr = data;
-    data = asn_build_sequence(data, &length, GET_RSP_MSG, 16);
+    data = asn_build_sequence(data, &length, SNMP_MSG_RESPONSE, 16);
     if (data == NULL)
 	return 0;
     reqidPtr = data;
@@ -1083,7 +1069,7 @@ create_toobig(snmp_out, snmp_length, reqid, pi)
 	return 0;
 
     pi->packet_end = data;
-    data = asn_build_sequence(headerPtr, &length, GET_RSP_MSG,
+    data = asn_build_sequence(headerPtr, &length, SNMP_MSG_RESPONSE,
 			      data - reqidPtr);
     if (data != reqidPtr)
 	return 0;
@@ -1103,45 +1089,10 @@ create_toobig(snmp_out, snmp_length, reqid, pi)
 }
 
 static int
-snmp_access(acl, community, rw)
-    u_short 	acl;
-    int		community;
-    int		rw;
-{
-    /*
-     * Each group has 2 bits, the more significant one is for read access,
-     * the less significant one is for write access.
-     */
-
-    community <<= 1;	/* multiply by two to shift two bits at a time */
-    if (rw == READ){
-	return (acl & (2 << community));    /* return the correct bit */
-    } else {
-	return (acl & (1 << community));
-    }
-}
-
-static int
-get_community(community, community_len)
-    u_char	*community;
-    int		community_len;
-{
-    int	count;
-
-    for(count = 0; count < NUM_COMMUNITIES; count++){
-	if ((community_len == strlen(communities[count])
-	     && !memcmp(communities[count], community, community_len)))
-	    break;
-    }
-    if (count == NUM_COMMUNITIES)
-	return -1;
-    return count + 1;
-}
-
-static int
-goodValue(inType, inLen, actualType, actualLen)
-    u_char	inType, actualType;
-    int		inLen, actualLen;
+goodValue(u_char inType, 
+	  int inLen,
+	  u_char actualType,
+	  int actualLen)
 {
     if (inLen > actualLen)
 	return FALSE;
@@ -1149,12 +1100,11 @@ goodValue(inType, inLen, actualType, actualLen)
 }
 
 static void
-setVariable(var_val, var_val_type, var_val_len, statP, statLen)
-    u_char  *var_val;
-    u_char  var_val_type;
-    int	    var_val_len;
-    u_char  *statP;
-    int	    statLen;
+setVariable(u_char *var_val,
+	    u_char var_val_type,
+	    int var_val_len,
+	    u_char *statP,
+	    int statLen)
 {
     int	    buffersize = 1000;
 
@@ -1162,19 +1112,19 @@ setVariable(var_val, var_val_type, var_val_len, statP, statLen)
 	case ASN_INTEGER:
 	    asn_parse_int(var_val, &buffersize, &var_val_type, (long *)statP, statLen);
 	    break;
-	case COUNTER:
-	case GAUGE:
-	case TIMETICKS:
+	case ASN_COUNTER:
+	case ASN_GAUGE:
+	case ASN_TIMETICKS:
 	    asn_parse_unsigned_int(var_val, &buffersize, &var_val_type, (u_long *)statP, statLen);
 	    break;
-	case COUNTER64:
+	case ASN_COUNTER64:
 	    asn_parse_unsigned_int64(var_val, &buffersize, &var_val_type,
 				     (struct counter64 *)statP, statLen);
 	    break;
 	case ASN_OCTET_STR:
-	case IPADDRESS:
-	case ASNT_OPAQUE:
-	case NSAP:
+	case ASN_IPADDRESS:
+	case ASN_OPAQUE:
+	case ASN_NSAP:
 	    asn_parse_string(var_val, &buffersize, &var_val_type, statP, &statLen);
 	    break;
 	case ASN_OBJECT_ID:
@@ -1185,31 +1135,3 @@ setVariable(var_val, var_val_type, var_val_len, statP, statLen)
 	    break;
     }
 }
-
-void snmp_agent_parse_config(word,cptr)
-  char *word;
-  char *cptr;
-{
-  char tmpbuf[1024];
-  int i;
-    
-  i = atoi(cptr);
-  if (i > 0 && i <= NUM_COMMUNITIES) {
-    cptr = skip_not_white(cptr);
-    cptr = skip_white(cptr);
-    if (cptr != NULL) {
-      if (((int) strlen(cptr)) < COMMUNITY_MAX_LEN) {
-        copy_word(cptr,communities[i-1]);
-      } else {
-        sprintf(tmpbuf,"community name (#%d) \"%s\" too long", i, cptr);
-        config_perror(tmpbuf);
-      }
-    } else {
-      config_perror("no community name found");
-    }
-  } else {
-    sprintf(tmpbuf,"community number invalid:  %d, must be > 0 and < %d", i,
-            NUM_COMMUNITIES+1);
-    config_perror(tmpbuf);
-  }
-}
