@@ -33,19 +33,23 @@ SOFTWARE.
 
 #include <config.h>
 
-#if STDC_HEADERS
+#if HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#if HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 
 #include <sys/types.h>
-#include <sys/socket.h>
 
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -53,21 +57,29 @@ SOFTWARE.
 #include <stdio.h>
 #include <signal.h>
 
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#else
+#include <sys/socket.h>
+#include <netdb.h>
+#endif
+
 #include "main.h"
 #include "asn1.h"
 #include "snmp.h"
-#include "snmp_impl.h"
 #include "snmp_api.h"
+#include "snmp_impl.h"
 #include "snmp_client.h"
 #include "mib.h"
+#include "snmp_debug.h"
+#include "../../snmplib/system.h"
 
 #include "netstat.h"
 
 #define	YES	1
 #define	NO	0
 
-static void sidewaysintpr __P((unsigned int));
-RETSIGTYPE catchalarm __P((int));
+static void sidewaysintpr(unsigned int);
 
 static oid oid_ifname[] =	{1, 3, 6, 1, 2, 1, 2, 2, 1, 2, 1};
 static oid oid_ifinucastpkts[] ={1, 3, 6, 1, 2, 1, 2, 2, 1,11, 1};
@@ -96,10 +108,9 @@ static oid oid_ipadentaddr[] =	{1, 3, 6, 1, 2, 1, 4,20, 1, 1, 0, 0, 0, 0};
  * Print a description of the network interfaces.
  */
 void
-intpr(interval)
-	int interval;
+intpr(int interval)
 {
-	oid varname[MAX_NAME_LEN], *instance, *ifentry;
+	oid varname[MAX_OID_LEN], *instance, *ifentry;
 	int varname_len;
 	int ifnum, cfg_nnets;
 	oid curifip [4];
@@ -109,18 +120,20 @@ intpr(interval)
 	int ifindex;
 	struct _if_info {
 	    char name[128];
+	    char ip[128], route[128];
 	    int mtu;
 	    int ipkts, ierrs, opkts, oerrs, operstatus, outqueue;
 	    u_long netmask;
 	    struct in_addr ifip, ifroute;
 	} *if_table, *cur_if;
+	int max_name = 0, max_ip = 0, max_route = 0, i;
 
 	if (interval) {
 		sidewaysintpr((unsigned)interval);
 		return;
 	}
 	var = getvarbyname(Session, oid_cfg_nnets, sizeof(oid_cfg_nnets) / sizeof(oid));
-	if (var) {
+	if (var && var->val.integer) {
 	    cfg_nnets = *var->val.integer;
 	    snmp_free_var(var);
 	}
@@ -128,13 +141,7 @@ intpr(interval)
 	    fprintf (stderr, "No response when requesting number of interfaces.\n");
 	    return;
 	}
-#ifdef DODEBUG
-	printf ("cfg_nnets = %d\n", cfg_nnets);
-#endif
-	printf("%-7.7s %-4.4s %-15.15s %-15.15s %8.8s %5.5s %8.8s %5.5s %5.5s",
-		"Name", "Mtu", "Network", "Address", "Ipkts", "Ierrs",
-		"Opkts", "Oerrs", "Queue");
-	putchar('\n');
+	DEBUGMSGTL (("netstat:if", "cfg_nnets = %d\n", cfg_nnets));
 
 	memset (curifip, 0, sizeof (curifip));
 	if_table = (struct _if_info *) calloc (cfg_nnets, sizeof (*if_table));
@@ -143,7 +150,7 @@ intpr(interval)
 	for (ifnum = 1; ifnum <= cfg_nnets; ifnum++) {
 		register char *cp;
 
-		request = snmp_pdu_create (GETNEXT_REQ_MSG);
+		request = snmp_pdu_create (SNMP_MSG_GETNEXT);
 		memmove (varname, oid_ipadentaddr, sizeof (oid_ipadentaddr));
 		varname_len = sizeof (oid_ipadentaddr) / sizeof (oid);
 		instance = varname + 9;
@@ -162,9 +169,9 @@ intpr(interval)
 		    break;
 		}
 		for (var = response->variables; var; var = var->next_variable) {
-#ifdef DODEBUG
+                  if (snmp_get_do_debugging()) {
 		    print_variable (var->name, var->name_length, var);
-#endif
+                  }
 		    switch (var->name [9]) {
 		    case IFINDEX:
 			ifindex = *var->val.integer;
@@ -179,6 +186,14 @@ intpr(interval)
 		    }
 		}
 		cur_if->ifroute.s_addr = cur_if->ifip.s_addr & cur_if->netmask;
+		if (cur_if->ifroute.s_addr)
+			strcpy(cur_if->route, netname (cur_if->ifroute, 0));
+		else strcpy(cur_if->route, "none");
+		if ((i = strlen(cur_if->route)) > max_route) max_route = i;
+		if (cur_if->ifip.s_addr)
+			strcpy(cur_if->ip, routename (cur_if->ifip));
+		else strcpy(cur_if->ip, "none");
+		if ((i = strlen(cur_if->ip)) > max_ip) max_ip = i;
 
 		snmp_free_pdu (response);
 
@@ -186,7 +201,7 @@ intpr(interval)
 		varname_len = sizeof(oid_ifname) / sizeof(oid);
 		ifentry = varname + 9;
 		instance = varname + 10;
-		request = snmp_pdu_create (GET_REQ_MSG);
+		request = snmp_pdu_create (SNMP_MSG_GET);
 
 		*instance = ifnum;
 		*ifentry = IFNAME;
@@ -218,9 +233,10 @@ intpr(interval)
 		}
 		cur_if = if_table + ifnum - 1;
 		for (var = response->variables; var; var = var->next_variable) {
-#ifdef DODEBUG
+                  if (snmp_get_do_debugging()) {
 		    print_variable (var->name, var->name_length, var);
-#endif
+                  }
+		    if (var->val.integer)
 		    switch (var->name [9]) {
 		    case OUTQLEN:
 			cur_if->outqueue = *var->val.integer; break;
@@ -239,8 +255,11 @@ intpr(interval)
 		    case OUTNUCASTPKTS:
 			cur_if->opkts += *var->val.integer; break;
 		    case IFNAME:
+			if (var->val_len >= sizeof(cur_if->name))
+			    var->val_len = sizeof(cur_if->name) - 1;
 			memmove (cur_if->name, var->val.string, var->val_len);
 			cur_if->name [var->val_len] = 0;
+			if ((i = strlen(cur_if->name)+1) > max_name) max_name = i;
 			break;
 		    case IFOPERSTATUS:
 			cur_if->operstatus = *var->val.integer; break;
@@ -249,27 +268,29 @@ intpr(interval)
 
 		snmp_free_pdu (response);
 
-		cur_if->name[6] = '\0';
-                cp = (char *) strchr(cur_if->name, ' ');
-                if ( cp != NULL )
-                  *cp = '\0';
-		if (interface != NULL && strcmp(cur_if->name, interface) != 0) {
+		if (intrface != NULL && strcmp(cur_if->name, intrface) != 0) {
 			cur_if->name [0] = 0;
 			continue;
 		}
 		if (cur_if->operstatus != MIB_IFSTATUS_UP) {
-			cp = (char *) strchr(cur_if->name, '\0');
+			cp = strchr(cur_if->name, '\0');
 			*cp++ = '*';
 			*cp = '\0';
 		}
 	}
 
+	printf("%*.*s %5.5s %*.*s %*.*s %9.9s %5.5s %9.9s %5.5s %5.5s",
+		-max_name, max_name, "Name", "Mtu",
+		-max_route, max_route, "Network",
+		-max_ip, max_ip, "Address", "Ipkts", "Ierrs",
+		"Opkts", "Oerrs", "Queue");
+	putchar('\n');
 	for (ifnum = 0, cur_if = if_table; ifnum < cfg_nnets; ifnum++, cur_if++) {
 		if (cur_if->name [0] == 0) continue;
-		printf("%-7.7s %4d ", cur_if->name, cur_if->mtu);
-		printf("%-15.15s ", cur_if->ifroute.s_addr ? netname (cur_if->ifroute, 0) : "none");
-		printf("%-15.15s ", cur_if->ifip.s_addr ? routename (cur_if->ifip) : "none");
-		printf("%8d %5d %8d %5d %5d",
+		printf("%-*.*s %5d ", max_name, max_name, cur_if->name, cur_if->mtu);
+		printf("%-*.*s ", max_route, max_route, cur_if->route);
+		printf("%-*.*s ", max_ip, max_ip, cur_if->ip);
+		printf("%9d %5d %9d %5d %5d",
 		    cur_if->ipkts, cur_if->ierrs,
 		    cur_if->opkts, cur_if->oerrs,
 		    cur_if->outqueue);
@@ -281,10 +302,9 @@ intpr(interval)
  * Print a description of the network interfaces.
  */
 void
-intpro(interval)
-	int interval;
+intpro(int interval)
 {
-	oid varname[MAX_NAME_LEN], *instance, *ifentry;
+	oid varname[MAX_OID_LEN], *instance, *ifentry;
 	int varname_len;
 	int ifnum, cfg_nnets;
 	oid curifip [4];
@@ -294,17 +314,19 @@ intpro(interval)
 	int ifindex;
 	struct _if_info {
 	    char name[128];
+	    char ip[128], route[128];
 	    unsigned long ioctets, ierrs, ooctets, oerrs, operstatus, outqueue;
 	    u_long netmask;
 	    struct in_addr ifip, ifroute;
 	} *if_table, *cur_if;
+	int max_name = 0, max_route = 0, max_ip = 0, i;
 
 	if (interval) {
 		sidewaysintpr((unsigned)interval);
 		return;
 	}
 	var = getvarbyname(Session, oid_cfg_nnets, sizeof(oid_cfg_nnets) / sizeof(oid));
-	if (var) {
+	if (var && var->val.integer) {
 	    cfg_nnets = *var->val.integer;
 	    snmp_free_var(var);
 	}
@@ -312,12 +334,7 @@ intpro(interval)
 	    fprintf (stderr, "No response when requesting number of interfaces.\n");
 	    return;
 	}
-#ifdef DODEBUG
-	printf ("cfg_nnets = %d\n", cfg_nnets);
-#endif
-	printf("%-7.7s %-15.15s %-15.15s %12.12s %12.12s ",
-		"Name", "Network", "Address", "Ioctets", "Ooctets");
-	putchar('\n');
+	DEBUGMSGTL (("netstat:if", "cfg_nnets = %d\n", cfg_nnets));
 
 	memset (curifip, 0, sizeof (curifip));
 	if_table = (struct _if_info *) calloc (cfg_nnets, sizeof (*if_table));
@@ -326,7 +343,7 @@ intpro(interval)
 	for (ifnum = 1; ifnum <= cfg_nnets; ifnum++) {
 		register char *cp;
 
-		request = snmp_pdu_create (GETNEXT_REQ_MSG);
+		request = snmp_pdu_create (SNMP_MSG_GETNEXT);
 		memmove (varname, oid_ipadentaddr, sizeof (oid_ipadentaddr));
 		varname_len = sizeof (oid_ipadentaddr) / sizeof (oid);
 		instance = varname + 9;
@@ -345,9 +362,9 @@ intpro(interval)
 		    break;
 		}
 		for (var = response->variables; var; var = var->next_variable) {
-#ifdef DODEBUG
+                  if (snmp_get_do_debugging()) {
 		    print_variable (var->name, var->name_length, var);
-#endif
+                  }
 		    switch (var->name [9]) {
 		    case IFINDEX:
 			ifindex = *var->val.integer;
@@ -362,6 +379,14 @@ intpro(interval)
 		    }
 		}
 		cur_if->ifroute.s_addr = cur_if->ifip.s_addr & cur_if->netmask;
+		if (cur_if->ifroute.s_addr)
+			strcpy(cur_if->route, netname (cur_if->ifroute, 0));
+		else strcpy(cur_if->route, "none");
+		if ((i = strlen(cur_if->route)) > max_route) max_route = i;
+		if (cur_if->ifip.s_addr)
+			strcpy(cur_if->ip, routename (cur_if->ifip));
+		else strcpy(cur_if->ip, "none");
+		if ((i = strlen(cur_if->ip)) > max_ip) max_ip = i;
 
 		snmp_free_pdu (response);
 
@@ -369,7 +394,7 @@ intpro(interval)
 		varname_len = sizeof(oid_ifname) / sizeof(oid);
 		ifentry = varname + 9;
 		instance = varname + 10;
-		request = snmp_pdu_create (GET_REQ_MSG);
+		request = snmp_pdu_create (SNMP_MSG_GET);
 
 		*instance = ifnum;
 		*ifentry = IFNAME;
@@ -389,9 +414,9 @@ intpro(interval)
 		}
 		cur_if = if_table + ifnum - 1;
 		for (var = response->variables; var; var = var->next_variable) {
-#ifdef DODEBUG
+                  if (snmp_get_do_debugging()) {
 		    print_variable (var->name, var->name_length, var);
-#endif
+                  }
 		    switch (var->name [9]) {
 		    case OUTQLEN:
 			cur_if->outqueue = *var->val.integer; break;
@@ -404,8 +429,11 @@ intpro(interval)
 		    case OUTOCTETS:
 			cur_if->ooctets += *var->val.integer; break;
 		    case IFNAME:
+			if (var->val_len >= sizeof(cur_if->name))
+			    var->val_len = sizeof(cur_if->name) - 1;
 			memmove (cur_if->name, var->val.string, var->val_len);
 			cur_if->name [var->val_len] = 0;
+			if ((i = strlen(cur_if->name)+1) > max_name) max_name = i;
 			break;
 		    case IFOPERSTATUS:
 			cur_if->operstatus = *var->val.integer; break;
@@ -414,11 +442,7 @@ intpro(interval)
 
 		snmp_free_pdu (response);
 
-		cur_if->name[6] = '\0';
-                cp = (char *) strchr(cur_if->name, ' ');
-                if ( cp != NULL )
-                  *cp = '\0';
-		if (interface != NULL && strcmp(cur_if->name, interface) != 0) {
+		if (intrface != NULL && strcmp(cur_if->name, intrface) != 0) {
 			cur_if->name [0] = 0;
 			continue;
 		}
@@ -429,16 +453,22 @@ intpro(interval)
 		}
 	}
 
+	printf("%*.*s %*.*s %*.*s %12.12s %12.12s ",
+		-max_name, max_name, "Name",
+		-max_route, max_route, "Network",
+		-max_ip, max_ip, "Address", "Ioctets", "Ooctets");
+	putchar('\n');
 	for (ifnum = 0, cur_if = if_table; ifnum < cfg_nnets; ifnum++, cur_if++) {
 		if (cur_if->name [0] == 0) continue;
-		printf("%-7.7s ", cur_if->name);
-		printf("%-15.15s ", cur_if->ifroute.s_addr ? netname (cur_if->ifroute, 0) : "none");
-		printf("%-15.15s ", cur_if->ifip.s_addr ? routename (cur_if->ifip) : "none");
+		printf("%*.*s ", -max_name, max_name, cur_if->name);
+		printf("%*.*s ", -max_route, max_route, cur_if->route);
+		printf("%*.*s ", -max_ip, max_ip, cur_if->ip);
 		printf("%12lu %12lu",
 		    cur_if->ioctets,
 		    cur_if->ooctets);
 		putchar('\n');
 	}
+	free(if_table);
 }
 
 #define	MAXIF	128
@@ -453,6 +483,83 @@ struct	iftot {
 
 u_char	signalled;			/* set if alarm goes off "early" */
 
+/*
+ * timerSet sets or resets the timer to fire in "interval" seconds.
+ * timerPause waits only if the timer has not fired.
+ * timing precision is not considered important.
+ */
+
+#if (defined(WIN32) || defined(cygwin))
+static int sav_int;
+static time_t timezup;
+static void
+timerSet(int interval_seconds)
+{
+    sav_int = interval_seconds;
+	timezup = time(0) + interval_seconds;
+}
+/* you can do better than this ! */
+static void
+timerPause(void)
+{
+	time_t now;
+	while (time(&now) < timezup)
+#ifdef WIN32
+		Sleep(400);
+#else
+	{
+		struct timeval tx;
+		tx.tv_sec = 0;
+		tx.tv_usec = 400 * 1000; /* 400 milliseconds */
+		select( 0,0,0,0,&tx);
+	}
+#endif
+}
+
+#else
+
+/*
+ * Called if an interval expires before sidewaysintpr has completed a loop.
+ * Sets a flag to not wait for the alarm.
+ */
+RETSIGTYPE
+catchalarm(int sig)
+{
+	signalled = YES;
+}
+
+static void
+timerSet(int interval_seconds)
+{
+#ifdef HAVE_SIGSET
+	(void)sigset(SIGALRM, catchalarm);
+#else
+	(void)signal(SIGALRM, catchalarm);
+#endif
+	signalled = NO;
+	(void)alarm(interval_seconds);
+}
+
+static void
+timerPause(void)
+{
+#ifdef HAVE_SIGHOLD
+	sighold(SIGALRM);
+	if (! signalled) {
+		sigpause(SIGALRM);
+	}
+#else
+	int oldmask;
+	oldmask = sigblock(sigmask(SIGALRM));
+	if (! signalled) {
+		sigpause(0);
+	}
+	sigsetmask(oldmask);
+#endif
+}
+
+#endif /* !WIN32 && !cygwin */
+
 /*
  * Print a running summary of interface statistics.
  * Repeat display every interval seconds, showing statistics
@@ -460,17 +567,13 @@ u_char	signalled;			/* set if alarm goes off "early" */
  * First line printed at top of screen is always cumulative.
  */
 static void
-sidewaysintpr(interval)
-	unsigned interval;
+sidewaysintpr(unsigned int interval)
 {
 	register struct iftot *ip, *total;
 	register int line;
 	struct iftot *lastif, *sum, *interesting, ifnow, *now = &ifnow;
-#ifndef HAVE_SIGHOLD
-	int oldmask;
-#endif
 	struct variable_list *var;
-	oid varname[MAX_NAME_LEN], *instance, *ifentry;
+	oid varname[MAX_OID_LEN], *instance, *ifentry;
 	int varname_len;
 	int ifnum, cfg_nnets;
 
@@ -494,13 +597,15 @@ sidewaysintpr(interval)
 		varname[10] = ifnum;
 		var = getvarbyname(Session, varname, varname_len);
 		if (var){
+		    if (var->val_len >= (sizeof(ip->ift_name) - 3))
+		        var->val_len = (sizeof(ip->ift_name) - 4);
 		    memmove(ip->ift_name + 1, var->val.string, var->val_len);
 		    snmp_free_var(var);
 		}
                 cp = (char *) strchr(ip->ift_name, ' ');
                 if ( cp != NULL )
                   *cp = '\0';
-		if (interface && strcmp(ip->ift_name + 1, interface) == 0)
+		if (intrface && strcmp(ip->ift_name + 1, intrface) == 0)
 			interesting = ip;
 		ip->ift_name[15] = '\0';
 		cp = (char *) strchr(ip->ift_name, '\0');
@@ -511,13 +616,8 @@ sidewaysintpr(interval)
 	}
 	lastif = ip;
 
-#ifdef HAVE_SIGSET
-	(void)sigset(SIGALRM, catchalarm);
-#else
-	(void)signal(SIGALRM, catchalarm);
-#endif
-	signalled = NO;
-	(void)alarm(interval);
+	timerSet(interval);
+
 banner:
 	printf("    input   %-6.6s    output       ", interesting->ift_name);
 	if (lastif - iftot > 0)
@@ -617,39 +717,13 @@ loop:
 	putchar('\n');
 	fflush(stdout);
 	line++;
-#ifdef HAVE_SIGHOLD
-	sighold(SIGALRM);
-	if (! signalled) {
-		sigpause(SIGALRM);
-	}
-#else
-	oldmask = sigblock(sigmask(SIGALRM));
-	if (! signalled) {
-		sigpause(0);
-	}
-	sigsetmask(oldmask);
-#endif
-/* reset signal as many OSs require this, and the rest shouldn't be hurt */
-#ifdef HAVE_SIGSET
-	(void)sigset(SIGALRM, catchalarm);
-#else
-	(void)signal(SIGALRM, catchalarm);
-#endif
-	signalled = NO;
-	(void)alarm(interval);
+
+	timerPause();
+	timerSet(interval);
+
 	if (line == 21)
 		goto banner;
 	goto loop;
 	/*NOTREACHED*/
 }
 
-/*
- * Called if an interval expires before sidewaysintpr has completed a loop.
- * Sets a flag to not wait for the alarm.
- */
-RETSIGTYPE
-catchalarm(sig)
-    int sig;
-{
-	signalled = YES;
-}
