@@ -7,13 +7,13 @@
 
                       All Rights Reserved
 
-Permission to use, copy, modify, and distribute this software and its 
-documentation for any purpose and without fee is hereby granted, 
+Permission to use, copy, modify, and distribute this software and its
+documentation for any purpose and without fee is hereby granted,
 provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in 
+both that copyright notice and this permission notice appear in
 supporting documentation, and that the name of CMU not be
 used in advertising or publicity pertaining to distribution of the
-software without specific, written prior permission.  
+software without specific, written prior permission.
 
 CMU DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
 ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
@@ -25,27 +25,27 @@ SOFTWARE.
 ******************************************************************/
 #include <config.h>
 
-#include <ctype.h>
-#ifdef HAVE_STDLIB_H
+#if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#ifdef HAVE_UNISTD_H
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#else
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 #include <sys/types.h>
 #if HAVE_NETINET_IN_H
 # include <netinet/in.h>
 #endif
-#ifdef HAVE_ARPA_INET_H
-#include <arpa/inet.h>
-#endif
 #if TIME_WITH_SYS_TIME
-# include <sys/time.h>
+# ifdef WIN32
+#  include <sys/timeb.h>
+# else
+#  include <sys/time.h>
+# endif
 # include <time.h>
 #else
 # if HAVE_SYS_TIME_H
@@ -57,49 +57,31 @@ SOFTWARE.
 #if HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
-#include <netdb.h>
 #include <stdio.h>
-#include <sys/socket.h>
-#include <net/if.h>
-#if HAVE_SYS_IOCTL_H
-#include <sys/ioctl.h>
+#if HAVE_WINSOCK_H
+#include <winsock.h>
+#else
+#include <netdb.h>
 #endif
-#if HAVE_SYS_SOCKIO_H
-#include <sys/sockio.h>
-#endif
-#include <sys/file.h>
-#include <nlist.h>
-#if HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
-#if HAVE_SYS_SYSCTL_H
-#include <sys/sysctl.h>
+#if HAVE_ARPA_INET_H
+#include <arpa/inet.h>
 #endif
 
 #include "asn1.h"
-#include "snmp_impl.h"
 #include "snmp_api.h"
 #include "snmp_client.h"
 #include "mib.h"
 #include "snmp.h"
-#include "party.h"
+#include "snmp_impl.h"
 #include "system.h"
 #include "snmp_parse_args.h"
 
-int main __P((int, char **));
-int ascii_to_binary __P((u_char *, u_char *));
-int hex_to_binary __P((u_char *, u_char *));
-int snmp_input __P((int, struct snmp_session *, int, struct snmp_pdu *, void *));
-in_addr_t parse_address __P((char *));
-void snmp_add_var __P((struct snmp_pdu *, oid *, int, char, char *));
-
 oid objid_enterprise[] = {1, 3, 6, 1, 4, 1, 3, 1, 1};
 oid objid_sysdescr[]   = {1, 3, 6, 1, 2, 1, 1, 1, 0};
 oid objid_sysuptime[]  = {1, 3, 6, 1, 2, 1, 1, 3, 0};
 oid objid_snmptrap[]   = {1, 3, 6, 1, 6, 3, 1, 1, 4, 1, 0};
 
-void
-usage __P((void))
+void usage(void)
 {
     fprintf(stderr,"Usage:\n  snmptrap ");
     snmp_parse_args_usage(stderr);
@@ -110,18 +92,16 @@ usage __P((void))
     fprintf(stderr, "  -v 2 trap parameters:\n\t uptime trapoid [ var ] ...\n");
 }
 
-int snmp_input(operation, session, reqid, pdu, magic)
-    int operation;
-    struct snmp_session *session;
-    int reqid;
-    struct snmp_pdu *pdu;
-    void *magic;
+int snmp_input(int operation,
+	       struct snmp_session *session,
+	       int reqid,
+	       struct snmp_pdu *pdu,
+	       void *magic)
 {
   return 1;
 }
 
-in_addr_t parse_address(address)
-    char *address;
+in_addr_t parse_address(char *address)
 {
     in_addr_t addr;
     struct sockaddr_in saddr;
@@ -140,180 +120,47 @@ in_addr_t parse_address(address)
 
 }
 
-/*
- * Add a variable with the requested name to the end of the list of
- * variables for this pdu.
- */
-void
-snmp_add_var(pdu, name, name_length, type, value)
-    struct snmp_pdu *pdu;
-    oid *name;
-    int name_length;
-    char type, *value;
-{
-    struct variable_list *vars;
-    char buf[2048];
-
-    if (pdu->variables == NULL){
-	pdu->variables = vars =
-	    (struct variable_list *)malloc(sizeof(struct variable_list));
-    } else {
-	for(vars = pdu->variables;
-	    vars->next_variable;
-	    vars = vars->next_variable)
-	    /*EXIT*/;
-	vars->next_variable =
-	    (struct variable_list *)malloc(sizeof(struct variable_list));
-	vars = vars->next_variable;
-    }
-
-    vars->next_variable = NULL;
-    vars->name = (oid *)malloc(name_length * sizeof(oid));
-    memmove(vars->name, name, name_length * sizeof(oid));
-    vars->name_length = name_length;
-
-    switch((type = tolower (type))){
-	case 'i':
-	    vars->type = INTEGER;
-	    vars->val.integer = (long *)malloc(sizeof(long));
-	    *(vars->val.integer) = atoi(value);
-	    vars->val_len = sizeof(long);
-	    break;
-	case 's':
-	case 'x':
-	case 'd':
-	    vars->type = STRING;
-	    if (type == 'd'){
-		vars->val_len = ascii_to_binary((u_char *)value, buf);
-	    } else if (type == 's'){
-		strcpy(buf, value);
-		vars->val_len = strlen(buf);
-	    } else if (type == 'x'){
-		vars->val_len = hex_to_binary((u_char *)value, buf);
-	    }
-	    vars->val.string = (u_char *)malloc(vars->val_len);
-	    memmove(vars->val.string, buf, vars->val_len);
-	    break;
-	case 'n':
-	    vars->type = NULLOBJ;
-	    vars->val_len = 0;
-	    vars->val.string = NULL;
-	    break;
-	case 'o':
-	    vars->type = OBJID;
-	    vars->val_len = MAX_NAME_LEN;
-	    read_objid(value, (oid *)buf, &vars->val_len);
-	    vars->val_len *= sizeof(oid);
-	    vars->val.objid = (oid *)malloc(vars->val_len);
-	    memmove(vars->val.objid, buf, vars->val_len);
-	    break;
-	case 't':
-	    vars->type = TIMETICKS;
-	    vars->val.integer = (long *)malloc(sizeof(long));
-	    *(vars->val.integer) = atoi(value);
-	    vars->val_len = sizeof(long);
-	    break;
-	case 'a':
-	    vars->type = IPADDRESS;
-	    vars->val.integer = (long *)malloc(sizeof(long));
-	    *(vars->val.integer) = inet_addr(value);
-	    vars->val_len = sizeof(long);
-	    break;
-	default:
-	    fprintf(stderr, "Bad object type: %c\n", type);
-	    exit(1);
-    }
-}
-
-int
-ascii_to_binary(cp, bufp)
-    u_char  *cp;
-    u_char *bufp;
-{
-    int	subidentifier;
-    u_char *bp = bufp;
-
-    for(; *cp != '\0'; cp++){
-	if (isspace(*cp) || *cp == '.')
-	    continue;
-	if (!isdigit(*cp)){
-	    fprintf(stderr, "Input error\n");
-	    return -1;
-	}
-	subidentifier = atoi(cp);
-	if (subidentifier > 255){
-	    fprintf(stderr, "subidentifier %d is too large ( > 255)\n",
-		    subidentifier);
-	    return -1;
-	}
-	*bp++ = (u_char)subidentifier;
-	while(isdigit(*cp))
-	    cp++;
-	cp--;
-    }
-    return bp - bufp;
-}
-
-int
-hex_to_binary(cp, bufp)
-    u_char  *cp;
-    u_char *bufp;
-{
-    int	subidentifier;
-    u_char *bp = bufp;
-
-    for(; *cp != '\0'; cp++){
-	if (isspace(*cp))
-	    continue;
-	if (!isxdigit(*cp)){
-	    fprintf(stderr, "Input error\n");
-	    return -1;
-	}
-	sscanf((char *)cp, "%x", &subidentifier);
-	if (subidentifier > 255){
-	    fprintf(stderr, "subidentifier %d is too large ( > 255)\n",
-		    subidentifier);
-	    return -1;
-	}
-	*bp++ = (u_char)subidentifier;
-	while(isxdigit(*cp))
-	    cp++;
-	cp--;
-    }
-    return bp - bufp;
-}
-
-int
-main(argc, argv)
-    int	    argc;
-    char    *argv[];
+int main(int argc, char *argv[])
 {
     struct snmp_session session, *ss;
-    struct snmp_pdu *pdu;
-    oid name[MAX_NAME_LEN];
+    struct snmp_pdu *pdu, *response;
+    oid name[MAX_OID_LEN];
     int name_length;
     int	arg;
+    int status, inform = 0;
     char *trap = NULL, *specific = NULL, *description = NULL, *agent = NULL;
+#ifdef _DEBUG_MALLOC_INC
+    unsigned long histid1, histid2, orig_size, current_size;
+#endif
 
     /*
      * usage: snmptrap gateway-name srcParty dstParty trap-type specific-type device-description [ -a agent-addr ]
      */
-    init_mib();
     arg = snmp_parse_args(argc, argv, &session);
- 
+
+    SOCK_STARTUP;
+
     session.callback = snmp_input;
     session.callback_magic = NULL;
+    if (session.remote_port == SNMP_DEFAULT_REMPORT)
+	session.remote_port = SNMP_TRAP_PORT;
+    snmp_synch_setup(&session);
     ss = snmp_open(&session);
     if (ss == NULL){
-        snmp_perror("snmptrap: Couldn't open snmp");
-	exit(1);
+        snmp_perror("snmptrap");
+        SOCK_CLEANUP;
+        exit(1);
     }
+#ifdef _DEBUG_MALLOC_INC
+    orig_size = malloc_inuse(&histid1);
+#endif
 
     if (session.version == SNMP_VERSION_1) {
-	pdu = snmp_pdu_create(TRP_REQ_MSG);
+	pdu = snmp_pdu_create(SNMP_MSG_TRAP);
 	if (arg == argc) {
 	    fprintf(stderr, "No enterprise oid\n");
 	    usage();
+            SOCK_CLEANUP;
 	    exit(1);
 	}
 	if (argv[arg][0] == 0) {
@@ -322,10 +169,11 @@ main(argc, argv)
 	    pdu->enterprise_length = sizeof(objid_enterprise)/sizeof (oid);
 	}
 	else {
-	    name_length = MAX_NAME_LEN;
-	    if (!read_objid (argv[arg], name, &name_length)) {
+	    name_length = MAX_OID_LEN;
+	    if (!snmp_parse_oid(argv[arg], name, &name_length)) {
 		fprintf (stderr, "Invalid enterprise id: %s\n", argv[arg]);
 		usage ();
+                SOCK_CLEANUP;
 		exit (1);
 	    }
 	    pdu->enterprise = (oid *)malloc(name_length * sizeof(oid));
@@ -335,6 +183,7 @@ main(argc, argv)
 	if (++arg >= argc) {
 	    fprintf (stderr, "Missing agent parameter\n");
 	    usage ();
+            SOCK_CLEANUP;
 	    exit (1);
 	}
 	agent = argv [arg];
@@ -345,6 +194,7 @@ main(argc, argv)
 	if (++arg == argc) {
 	    fprintf (stderr, "Missing generic-trap parameter\n");
 	    usage ();
+            SOCK_CLEANUP;
 	    exit (1);
 	}
 	trap = argv [arg];
@@ -352,6 +202,7 @@ main(argc, argv)
 	if (++arg == argc) {
 	    fprintf (stderr, "Missing specific-trap parameter\n");
 	    usage ();
+            SOCK_CLEANUP;
 	    exit (1);
 	}
 	specific = argv [arg];
@@ -359,6 +210,7 @@ main(argc, argv)
 	if (++arg == argc) {
 	    fprintf (stderr, "Missing uptime parameter\n");
 	    usage ();
+            SOCK_CLEANUP;
 	    exit (1);
 	}
 	description = argv [arg];
@@ -370,11 +222,18 @@ main(argc, argv)
     else {
 	long sysuptime;
 	char csysuptime [20];
+	char *prognam;
 
-	pdu = snmp_pdu_create(TRP2_REQ_MSG);
+	prognam = strrchr(argv[0], '/');
+	if (prognam) prognam++;
+	else prognam = argv[0];
+
+	if (strcmp(prognam, "snmpinform") == 0) inform = 1;
+	pdu = snmp_pdu_create(inform ? SNMP_MSG_INFORM : SNMP_MSG_TRAP2);
 	if (arg == argc) {
 	    fprintf(stderr, "Missing up-time parameter\n");
 	    usage();
+            SOCK_CLEANUP;
 	    exit(1);
 	}
 	trap = argv[arg];
@@ -388,6 +247,7 @@ main(argc, argv)
 	if (++arg == argc) {
 	    fprintf (stderr, "Missing trap-oid parameter\n");
 	    usage ();
+            SOCK_CLEANUP;
 	    exit (1);
 	}
 	snmp_add_var (pdu, objid_snmptrap, sizeof (objid_snmptrap)/sizeof(oid),
@@ -398,17 +258,28 @@ main(argc, argv)
     while (arg < argc) {
 	arg += 3;
 	if (arg > argc) break;
-	name_length = MAX_NAME_LEN;
-	if (!read_objid (argv [arg-3], name, &name_length)) {
+	name_length = MAX_OID_LEN;
+	if (!snmp_parse_oid(argv [arg-3], name, &name_length)) {
 	    fprintf (stderr, "Invalid object identifier: %s\n", argv [arg-3]);
+            SOCK_CLEANUP;
 	    exit(1);
 	}
 	snmp_add_var (pdu, name, name_length, argv [arg-2][0], argv [arg-1]);
     }
 
-    if (snmp_send(ss, pdu)== 0){
-        snmp_perror("snmptrap: snmp_send error");
+    if (inform) status = snmp_synch_response(ss, pdu, &response);
+    else status = snmp_send(ss, pdu) == 0;
+    if (status) {
+        snmp_perror(inform ? (char*)"snmpinform" : (char*)"snmptrap");
     }
+    if (inform) snmp_free_pdu(response);
+    else snmp_free_pdu(pdu);
+
+#ifdef _DEBUG_MALLOC_INC
+    current_size = malloc_inuse(&histid2);
+    if (current_size != orig_size) malloc_list(2, histid1, histid2);
+#endif
     snmp_close(ss);
-    exit (0);
+    SOCK_CLEANUP;
+    return (0);
 }
