@@ -31,12 +31,13 @@ SOFTWARE.
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_STRINGS_H
-#include <strings.h>
-#else
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/socket.h>
 #if HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
@@ -70,19 +71,26 @@ SOFTWARE.
 #if HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
+#if HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+#include <signal.h>
+#include <errno.h>
 
 #include "asn1.h"
-#include "snmp_impl.h"
 #include "snmp_api.h"
+#include "snmp_impl.h"
 #include "snmp_client.h"
-#include "party.h"
-#include "view.h"
-#include "acl.h"
-#include "context.h"
 #include "mib.h"
 #include "snmp.h"
+#include "party.h"
+#include "context.h"
+#include "acl.h"
 #include "system.h"
 #include "version.h"
+#include "snmptrapd_handlers.h"
+#include "read_config.h"
+#include "snmp_debug.h"
 
 #ifndef BSD4_3
 #define BSD4_2
@@ -99,23 +107,59 @@ typedef long	fd_mask;
 #define FD_ZERO(p)      memset((p), 0, sizeof(*(p)))
 #endif
 
-int main __P((int, char **));
 int Print = 0;
-int Event = 0;
 int Syslog = 0;
+int Event = 0;
+int dropauth = 0;
+int running = 1;
+
+/*
+ * These definitions handle 4.2 systems without additional syslog facilities.
+ */
+#ifndef LOG_CONS
+#define LOG_CONS	0	/* Don't bother if not defined... */
+#endif
+#ifndef LOG_PID
+#define LOG_PID		0	/* Don't bother if not defined... */
+#endif
+#ifndef LOG_LOCAL0
+#define LOG_LOCAL0	0
+#endif
+#ifndef LOG_LOCAL1
+#define LOG_LOCAL1	0
+#endif
+#ifndef LOG_LOCAL2
+#define LOG_LOCAL2	0
+#endif
+#ifndef LOG_LOCAL3
+#define LOG_LOCAL3	0
+#endif
+#ifndef LOG_LOCAL4
+#define LOG_LOCAL4	0
+#endif
+#ifndef LOG_LOCAL5
+#define LOG_LOCAL5	0
+#endif
+#ifndef LOG_LOCAL6
+#define LOG_LOCAL6	0
+#endif
+#ifndef LOG_LOCAL7
+#define LOG_LOCAL7	0
+#endif
+#ifndef LOG_DAEMON
+#define LOG_DAEMON	0
+#endif
+
+/* Include an extra Facility variable to allow command line adjustment of
+   syslog destination */
+int Facility = LOG_LOCAL0;
+
 struct timeval Now;
 
-void init_syslog __P((void));
-char *trap_description __P((int));
-char *uptime_string __P((u_long, char *));
-struct snmp_pdu *snmp_clone_pdu2 __P((struct snmp_pdu *, int));
-void event_input __P((struct variable_list *));
-int snmp_input __P((int, struct snmp_session *, int, struct snmp_pdu *, void *));
-void usage __P((void));
+void init_syslog(void);
 
 char *
-trap_description(trap)
-    int trap;
+trap_description(int trap)
 {
     switch(trap){
 	case SNMP_TRAP_COLDSTART:
@@ -137,10 +181,9 @@ trap_description(trap)
     }
 }
 
+#ifndef CMU_COMPATIBLE
 char *
-uptime_string(timeticks, buf)
-    register u_long timeticks;
-    char *buf;
+uptime_string(u_long timeticks, char *buf)
 {
     int	seconds, minutes, hours, days;
 
@@ -163,17 +206,18 @@ uptime_string(timeticks, buf)
     }
     return buf;
 }
+#endif /* !CMU_COMPATIBLE */
 
 struct snmp_pdu *
-snmp_clone_pdu2(pdu, command)
-    struct snmp_pdu *pdu;
-    int command;
+snmp_clone_pdu2(struct snmp_pdu *pdu,
+		int command)
 {
     struct variable_list *var, *newvar;
     struct snmp_pdu *newpdu;
 
     /* clone the pdu */
     newpdu = (struct snmp_pdu *)malloc(sizeof(struct snmp_pdu));
+    if (newpdu == NULL) return NULL;
     memmove(newpdu, pdu, sizeof(struct snmp_pdu));
     newpdu->variables = NULL;
     newpdu->command = command;
@@ -217,11 +261,10 @@ static oid fallingAlarm[] = {1, 3, 6, 1, 6, 3, 2, 1, 1, 3, 2};
 static oid unavailableAlarm[] = {1, 3, 6, 1, 6, 3, 2, 1, 1, 3, 3};
 
 void
-event_input(vp)	
-    struct variable_list *vp;
+event_input(struct variable_list *vp)
 {
     int eventid;
-    oid variable[MAX_NAME_LEN];
+    oid variable[MAX_OID_LEN];
     int variablelen;
     u_long destip;
     int sampletype;
@@ -268,29 +311,111 @@ event_input(vp)
     
 }
 
+void do_external(char *cmd,
+		 struct hostent *host,
+		 struct snmp_pdu *pdu)
+{
+  struct variable_list tmpvar, *vars;
+  static oid trapoids[10] = {1,3,6,1,6,3,1,1,5};
+  static oid snmpsysuptime[8] = {1,3,6,1,2,1,1,3};
+  static oid snmptrapoid[10] = {1,3,6,1,6,3,1,1,4,1};
+  static oid snmptrapent[10] = {1,3,6,1,6,3,1,1,4,3};
+  int fd[2];
+  int pid, result;
+  FILE *file;
+  char varbuf[SPRINT_MAX_LEN];
+  int oldquick;
+  
+  DEBUGMSGTL(("snmptrapd", "Running: %s\n", cmd));
+  oldquick = snmp_get_quick_print();
+  snmp_set_quick_print(1);
+  if (cmd) {
+    if (pipe(fd)) {
+      perror("pipe");
+    }
+    if ((pid = fork()) == 0) {
+      /* child */
+      close(0);
+      if (dup(fd[0]) != 0) {
+        perror("dup");
+      }
+      close(fd[1]);
+      close(fd[0]);
+      system(cmd);
+      exit(0);
+    } else if (pid > 0) {
+      file = fdopen(fd[1],"w");
+      fprintf(file,"%s\n%s\n",
+              host ? host->h_name : inet_ntoa(pdu->address.sin_addr),
+              inet_ntoa(pdu->address.sin_addr));
+      if (pdu->command == SNMP_MSG_TRAP){
+        /* convert a v1 trap to a v2 variable binding list:
+           The uptime and trapOID go first in the list. */
+        tmpvar.val.integer = (long *) &pdu->time;
+        tmpvar.val_len = sizeof(pdu->time);
+        tmpvar.type = ASN_TIMETICKS;
+        sprint_variable(varbuf, snmpsysuptime, 8, &tmpvar);
+        fprintf(file,"%s\n",varbuf);
+        tmpvar.type = ASN_OBJECT_ID;
+        trapoids[9] = pdu->trap_type+1;
+        tmpvar.val.objid = trapoids;
+        tmpvar.val_len = 10*sizeof(oid);
+        sprint_variable(varbuf, snmptrapoid, 10, &tmpvar);
+        fprintf(file,"%s\n",varbuf);
+      }
+      /* do the variables in the pdu */
+      for(vars = pdu->variables; vars; vars = vars->next_variable) {
+        sprint_variable(varbuf, vars->name, vars->name_length, vars);
+        fprintf(file,"%s\n",varbuf);
+      }
+      if (pdu->command == SNMP_MSG_TRAP){
+        /* convert a v1 trap to a v2 variable binding list:
+           The enterprise goes last. */
+        tmpvar.val.objid = pdu->enterprise;
+        tmpvar.val_len = pdu->enterprise_length*sizeof(oid);
+        sprint_variable(varbuf, snmptrapent, 10, &tmpvar);
+        fprintf(file,"%s\n",varbuf);
+      }
+      fclose(file);
+      close(fd[0]);
+      close(fd[1]);
+      if (waitpid(pid, &result,0) < 0) {
+        perror("waitpid");
+      }
+    } else {
+      perror("fork");
+    }
+  }
+  snmp_set_quick_print(oldquick);
+}
 
-int snmp_input(op, session, reqid, pdu, magic)
-    int op;
-    struct snmp_session *session;
-    int reqid;
-    struct snmp_pdu *pdu;
-    void *magic;
+int snmp_input(int op,
+	       struct snmp_session *session,
+	       int reqid,
+	       struct snmp_pdu *pdu,
+	       void *magic)
 {
     struct variable_list *vars;
-    char buf[64], oid_buf [512];
+    char buf[64], oid_buf [SPRINT_MAX_LEN], *cp;
     struct snmp_pdu *reply;
     struct tm *tm;
     time_t timer;
     struct hostent *host;
-
+    int varbufidx;
+    char varbuf[SPRINT_MAX_LEN];
+    static oid trapoids[10] = {1,3,6,1,6,3,1,1,5};
+    static oid snmptrapoid2[11] = {1,3,6,1,6,3,1,1,4,1,0};
+    struct variable_list tmpvar;
+    char *Command = NULL;
+    tmpvar.type = ASN_OBJECT_ID;
+                  
     if (op == RECEIVED_MESSAGE){
-	if (pdu->command == TRP_REQ_MSG){
-	    if (Print){
+	if (pdu->command == SNMP_MSG_TRAP){
+	    host = gethostbyaddr ((char *)&pdu->agent_addr.sin_addr,
+				  sizeof (pdu->agent_addr.sin_addr), AF_INET);
+	    if (Print && (pdu->trap_type != SNMP_TRAP_AUTHFAIL || dropauth == 0)) {
 		time (&timer);
 		tm = localtime (&timer);
-                host = gethostbyaddr ((char *)&pdu->agent_addr.sin_addr,
-                                      sizeof (pdu->agent_addr.sin_addr),
-                                      AF_INET);
                 printf("%.4d-%.2d-%.2d %.2d:%.2d:%.2d %s [%s] %s:\n",
 		       tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
 		       tm->tm_hour, tm->tm_min, tm->tm_sec,
@@ -298,54 +423,107 @@ int snmp_input(op, session, reqid, pdu, magic)
                        inet_ntoa(pdu->agent_addr.sin_addr),
  		       sprint_objid (oid_buf, pdu->enterprise, pdu->enterprise_length));
 		if (pdu->trap_type == SNMP_TRAP_ENTERPRISESPECIFIC) {
-		    oid trapOid[64];
+		    oid trapOid[MAX_OID_LEN];
 		    int trapOidLen = pdu->enterprise_length;
 		    memcpy(trapOid, pdu->enterprise, sizeof(oid)*trapOidLen);
 		    if (trapOid[trapOidLen-1] != 0) trapOid[trapOidLen++] = 0;
 		    trapOid[trapOidLen++] = pdu->specific_type;
 		    sprint_objid(oid_buf, trapOid, trapOidLen);
+		    cp = strrchr(oid_buf, '.');
+		    if (cp) cp++;
+		    else cp = oid_buf;
 		    printf("\t%s Trap (%s) Uptime: %s\n",
-			   trap_description(pdu->trap_type),
-			   strrchr(oid_buf, '.')+1,
+			   trap_description(pdu->trap_type), cp,
 			   uptime_string(pdu->time, buf));
 		}
 		else
-		    printf("\t%s Trap (%d) Uptime: %s\n",
+		    printf("\t%s Trap (%ld) Uptime: %s\n",
 			   trap_description(pdu->trap_type), pdu->specific_type,
 			   uptime_string(pdu->time, buf));
 		for(vars = pdu->variables; vars; vars = vars->next_variable) {
 		    printf ("\t");
 		    print_variable(vars->name, vars->name_length, vars);
 		}
+                printf("\n");
 	    }
-	    if (Syslog){
-		syslog(LOG_WARNING, "%s: %s Trap (%d) Uptime: %s\n",
+	    if (Syslog && (pdu->trap_type != SNMP_TRAP_AUTHFAIL || dropauth == 0)) {
+	    	varbufidx=0;
+	    	varbuf[varbufidx++]=','; varbuf[varbufidx++]=' ';
+	    	varbuf[varbufidx]='\0';
+
+	    	for(vars = pdu->variables; vars; vars = vars->next_variable) {
+			sprint_variable(varbuf+varbufidx, vars->name,
+                                        vars->name_length, vars);
+			/* Update the length of the string with the
+                           new variable */
+			varbufidx += strlen(varbuf+varbufidx);
+			/* And add a trailing , ... */
+	        	varbuf[varbufidx++]=',';
+	        	varbuf[varbufidx++]=' ';
+	        	varbuf[varbufidx]='\0';
+	    	}
+	    	if ( varbufidx ) {
+			varbufidx -= 2; varbuf[varbufidx]='\0';
+	    	}
+		if (pdu->trap_type == SNMP_TRAP_ENTERPRISESPECIFIC) {
+		    oid trapOid[MAX_OID_LEN];
+		    int trapOidLen = pdu->enterprise_length;
+		    memcpy(trapOid, pdu->enterprise, sizeof(oid)*trapOidLen);
+		    if (trapOid[trapOidLen-1] != 0) trapOid[trapOidLen++] = 0;
+		    trapOid[trapOidLen++] = pdu->specific_type;
+		    sprint_objid(oid_buf, trapOid, trapOidLen);
+		    cp = strrchr(oid_buf, '.');
+		    if (cp) cp++;
+		    else cp = oid_buf;
+		    syslog(LOG_WARNING, "%s: %s Trap (%s) Uptime: %s%s",
+		       inet_ntoa(pdu->agent_addr.sin_addr),
+		       trap_description(pdu->trap_type), cp,
+		       uptime_string(pdu->time, buf), varbuf);
+		} else {
+		    syslog(LOG_WARNING, "%s: %s Trap (%ld) Uptime: %s%s",
 		       inet_ntoa(pdu->agent_addr.sin_addr),
 		       trap_description(pdu->trap_type), pdu->specific_type,
-		       uptime_string(pdu->time, buf));
+		       uptime_string(pdu->time, buf), varbuf);
+		}
 	    }
-	} else if (pdu->command == TRP2_REQ_MSG
-		   || pdu->command == INFORM_REQ_MSG){
-	    if (Print){
-		printf("-------------------------------  Notification  -------------------------------\n");
+            trapoids[9] = pdu->trap_type+1;
+            Command = snmptrapd_get_traphandler(trapoids, 10);
+            if (Command)
+              do_external(Command, host, pdu);
+	} else if (pdu->command == SNMP_MSG_TRAP2
+		   || pdu->command == SNMP_MSG_INFORM){
+	    host = gethostbyaddr ((char *)&pdu->address.sin_addr,
+				  sizeof (pdu->address.sin_addr), AF_INET);
+	    if (Print) {
 		time (&timer);
 		tm = localtime (&timer);
-                host = gethostbyaddr ((char *)&pdu->address.sin_addr,
-                                      sizeof (pdu->address.sin_addr),
-                                      AF_INET);
                 printf("%.4d-%.2d-%.2d %.2d:%.2d:%.2d %s [%s]:\n",
 		       tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
 		       tm->tm_hour, tm->tm_min, tm->tm_sec,
                        host ? host->h_name : inet_ntoa(pdu->address.sin_addr),
                        inet_ntoa(pdu->address.sin_addr));
-		for(vars = pdu->variables; vars; vars = vars->next_variable)
+		for (vars = pdu->variables; vars; vars = vars->next_variable) {
+		    printf("\t");
 		    print_variable(vars->name, vars->name_length, vars);
+		}
+		printf("\n");
 	    }
 	    if (Event) {
 		event_input(pdu->variables);
 	    }
-	    if (pdu->command == INFORM_REQ_MSG){
-		if (!(reply = snmp_clone_pdu2(pdu, GET_RSP_MSG))){
+            for(vars = pdu->variables;
+                vars &&
+                snmp_oid_compare(vars->name, vars->name_length, snmptrapoid2,
+                         sizeof(snmptrapoid2)/sizeof(oid));
+                vars = vars->next_variable);
+            if (vars && vars->type == ASN_OBJECT_ID) {
+              Command = snmptrapd_get_traphandler(vars->val.objid,
+                                                  vars->val_len/sizeof(oid));
+              if (Command)
+                do_external(Command, host, pdu);
+            }
+	    if (pdu->command == SNMP_MSG_INFORM){
+		if (!(reply = snmp_clone_pdu2(pdu, SNMP_MSG_RESPONSE))){
 		    fprintf(stderr, "Couldn't clone PDU for response\n");
 		    return 1;
 		}
@@ -364,16 +542,18 @@ int snmp_input(op, session, reqid, pdu, magic)
 }
 
 
-void usage __P((void))
+void usage(void)
 {
-    fprintf(stderr,"Usage: snmptrapd [-V] [-q] [-p #] [-P] [-s] [-e] [-d]\n");
+    fprintf(stderr,"Usage: snmptrapd [-V] [-q] [-D] [-p #] [-P] [-s] [-f] [-l [d0-7]] [-e] [-d] [-H] [-S] [-a]\n");
+}
+
+RETSIGTYPE term_handler(int sig)
+{
+    running = 0;
 }
 
 
-int
-main(argc, argv)
-    int	    argc;
-    char    *argv[];
+int main(int argc, char *argv[])
 {
     struct snmp_session session, *ss;
     int	arg;
@@ -381,14 +561,18 @@ main(argc, argv)
     fd_set fdset;
     struct timeval timeout, *tvp;
     in_addr_t myaddr;
-    oid src[MAX_NAME_LEN], dst[MAX_NAME_LEN], context[MAX_NAME_LEN];
+    oid src[MAX_OID_LEN], dst[MAX_OID_LEN], context[MAX_OID_LEN];
     int srclen, dstlen, contextlen;
     int local_port = SNMP_TRAP_PORT;
     char ctmp[300];
+    int dofork=1;
+
+    /* register our configuration handlers now so -H properly displays them */
+    register_config_handler("snmptrapd","traphandle",snmptrapd_traphandle,NULL,"oid program [args ...] ");
 
     setvbuf (stdout, NULL, _IOLBF, BUFSIZ);
     /*
-     * usage: snmptrapd [-v 1] [-q] [-p #] [-P] [-s] [-d] [-e]
+     * usage: snmptrapd [-q] [-D] [-p #] [-P] [-s] [-f] [-l [d0-7]] [-d] [-e] [-S] [-a]
      */
     for(arg = 1; arg < argc; arg++){
 	if (argv[arg][0] == '-'){
@@ -403,6 +587,10 @@ main(argc, argv)
 		case 'q':
 		    snmp_set_quick_print(1);
 		    break;
+		case 'D':
+                    debug_register_tokens(&argv[arg][2]);
+                    snmp_set_do_debugging(1);
+                    break;
                 case 'p':
 		    if (++arg == argc) {
 			usage();
@@ -410,6 +598,28 @@ main(argc, argv)
 		    }
                     local_port = atoi(argv[arg]);
                     break;
+		case 'm':
+		    if (argv[arg][2] != 0)
+			setenv("MIBS",&argv[arg][2], 1);
+		    else if (++arg < argc)
+			setenv("MIBS",argv[arg], 1);
+		    else {
+			fprintf(stderr,"Need MIBS after -m flag.\n");
+			usage();
+			exit(1);
+		    }
+		    break;
+		case 'M':
+		    if (argv[arg][2] != 0)
+			setenv("MIBDIRS",&argv[arg][2], 1);
+		    else if (++arg < argc)
+			setenv("MIBDIRS",argv[arg], 1);
+		    else {
+			fprintf(stderr,"Need MIBDIRS after -M flag.\n");
+			usage();
+			exit(1);
+		    }
+		    break;
 		case 'P':
 		    Print++;
 		    break;
@@ -419,6 +629,49 @@ main(argc, argv)
 		case 's':
 		    Syslog++;
 		    break;
+		case 'S':
+		    snmp_set_suffix_only(2);
+		    break;
+		case 'a':
+		    dropauth = 1;
+		    break;
+                case 'f':
+		    dofork = 0;
+		    break;
+		case 'l':
+		    arg++;
+		    switch(argv[arg][0]) {
+			case 'd':
+			   Facility = LOG_DAEMON; break;
+			case '0':
+			   Facility = LOG_LOCAL0; break;
+			case '1':
+			   Facility = LOG_LOCAL1; break;
+			case '2':
+			   Facility = LOG_LOCAL2; break;
+			case '3':
+			   Facility = LOG_LOCAL3; break;
+			case '4':
+			   Facility = LOG_LOCAL4; break;
+			case '5':
+			   Facility = LOG_LOCAL5; break;
+			case '6':
+			   Facility = LOG_LOCAL6; break;
+			case '7':
+			   Facility = LOG_LOCAL7; break;
+			default:
+		    	   fprintf(stderr,"invalid  syslog facility: -l %c\n",
+							argv[arg][0]);
+		    	   usage();
+		    	   exit (1);
+		           break;
+		    }
+		    break;
+                case 'H':
+                    init_snmp();
+                    fprintf(stderr, "Configuration directives understood:\n");
+                    read_config_print_usage("  ");
+                    exit(0);
 		default:
 		    fprintf(stderr,"invalid option: -%c\n", argv[arg][1]);
 		    usage();
@@ -433,27 +686,75 @@ main(argc, argv)
 	}
     }
 
-    if (Syslog) init_syslog();
-    init_mib();
+    init_snmp();
+    if (!Print) Syslog = 1;
 
     myaddr = get_myaddr();
-    srclen = dstlen = contextlen = MAX_NAME_LEN;
+    srclen = dstlen = contextlen = MAX_OID_LEN;
     ms_party_init(myaddr, src, &srclen, dst, &dstlen, context, &contextlen);
 
-    sprintf(ctmp,"%s/party.conf",SNMPLIBPATH);
+    sprintf(ctmp,"%s/party.conf",SNMPSHAREPATH);
     if (read_party_database(ctmp) != 0){
 	fprintf(stderr, "Warning: Couldn't read v2party database from %s\n",ctmp);
     }
-    sprintf(ctmp,"%s/context.conf",SNMPLIBPATH);
+    sprintf(ctmp,"%s/context.conf",SNMPSHAREPATH);
     if (read_context_database(ctmp) != 0){
 	fprintf(stderr, "Warning: Couldn't read v2party's context database from %s\n",ctmp);
     }
-    sprintf(ctmp,"%s/acl.conf",SNMPLIBPATH);
+    sprintf(ctmp,"%s/acl.conf",SNMPSHAREPATH);
     if (read_acl_database(ctmp) != 0){
 	fprintf(stderr,
 		"Warning: Couldn't read v2party's access control database from %s\n",ctmp);
     }
 
+    /* fork the process to the background if we are not printing to stdout */
+    if (!Print && dofork) {
+      int fd, fdnum;
+
+      switch (fork()) {
+	case -1:
+		fprintf(stderr,"bad fork - %s\n",strerror(errno));
+		_exit(1);
+
+	case 0:
+		/* become process group leader */
+		if (setsid() == -1) {
+			fprintf(stderr,"bad setsid - %s\n",strerror(errno));
+			_exit(1);
+		}
+
+        /* if we are forked, we don't want to print out to stdout or stderr */
+		fd=open("/dev/null", O_RDWR);
+		dup2(fd, STDIN_FILENO);
+		dup2(fd, STDOUT_FILENO);
+		dup2(fd, STDERR_FILENO);
+		close(fd);
+                /* Close all unnecessary file descriptors */
+		fdnum = getdtablesize();
+		for ( fd = (STDERR_FILENO + 1); fd < fdnum; fd++ )
+			close(fd);
+		break;
+
+	default:
+		_exit(0);
+      }
+    }
+
+    if (Syslog) {
+	/* open syslog */
+	init_syslog();
+    }
+    if (Print) {
+	struct tm *tm;
+	time_t timer;
+	time (&timer);
+	tm = localtime (&timer);
+	printf("%.4d-%.2d-%.2d %.2d:%.2d:%.2d UCD-snmp version %s Started.\n",
+	       tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
+	       tm->tm_hour, tm->tm_min, tm->tm_sec,
+	       VersionInfo);
+    }
+
     memset(&session, 0, sizeof(struct snmp_session));
     session.peername = NULL;
     session.version = SNMP_DEFAULT_VERSION;
@@ -467,11 +768,18 @@ main(argc, argv)
     session.local_port = local_port;
     ss = snmp_open(&session);
     if (ss == NULL){
-        snmp_perror("snmptrapd: Couldn't open snmp");
+        snmp_perror("snmptrapd");
+        if (Syslog) {
+	    syslog(LOG_ERR,"couldn't open snmp - %m");
+	}
 	exit(1);
     }
 
-    while(1){
+    signal(SIGTERM, term_handler);
+    signal(SIGHUP, term_handler);
+    signal(SIGINT, term_handler);
+
+    while (running) {
 	numfds = 0;
 	FD_ZERO(&fdset);
 	block = 0;
@@ -491,30 +799,43 @@ main(argc, argv)
 		break;
 	    case -1:
 	        perror("select");
-		return 1;
+		running = 0;
 	    default:
 		fprintf(stderr, "select returned %d\n", count);
-		return 1;
+		running = 0;
 	}
     }
+
+    if (Print) {
+	struct tm *tm;
+	time_t timer;
+	time (&timer);
+	tm = localtime (&timer);
+	printf("%.4d-%.2d-%.2d %.2d:%.2d:%.2d UCD-snmp version %s Stopped.\n",
+	       tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday,
+	       tm->tm_hour, tm->tm_min, tm->tm_sec,
+	       VersionInfo);
+    }
+    return 0;
 }
 
 void
-init_syslog __P((void))
+init_syslog(void)
 {
-/*
- * These definitions handle 4.2 systems without additional syslog facilities.
- */
-#ifndef LOG_CONS
-#define LOG_CONS	0	/* Don't bother if not defined... */
-#endif
-#ifndef LOG_LOCAL0
-#define LOG_LOCAL0	0
-#endif
     /*
      * All messages will be logged to the local0 facility and will be sent to
      * the console if syslog doesn't work.
      */
-    openlog("snmptrapd", LOG_CONS, LOG_LOCAL0);
+    openlog("snmptrapd", LOG_CONS|LOG_PID, Facility);
     syslog(LOG_INFO, "Starting snmptrapd");
 }
+
+#ifndef HAVE_GETDTABLESIZE
+#include <sys/resource.h>
+int getdtablesize(void)
+{
+  struct rlimit rl;
+  getrlimit(RLIMIT_NOFILE, &rl);
+  return( rl.rlim_cur );
+}
+#endif
