@@ -32,10 +32,10 @@ SOFTWARE.
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_STRINGS_H
-#include <strings.h>
-#else
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 #include <sys/types.h>
 #if HAVE_NETINET_IN_H
@@ -74,20 +74,12 @@ SOFTWARE.
 #include "mib.h"
 #include "snmp.h"
 #include "snmp_impl.h"
-#include "party.h"
-#include "context.h"
-#include "view.h"
-#include "acl.h"
 #include "system.h"
 #include "snmp_parse_args.h"
 
-void main __P((int, char **));
-int failures;
-
 oid objid_mib[] = {1, 3, 6, 1, 2, 1};
 
-void
-usage __P((void))
+void usage(void)
 {
   fprintf(stderr,"Usage:\n  snmpwalk ");
   snmp_parse_args_usage(stderr);
@@ -95,18 +87,15 @@ usage __P((void))
   snmp_parse_args_descriptions(stderr);
 }
 
-void
-main(argc, argv)
-    int   argc;
-    char  *argv[];
+int main(int argc, char *argv[])
 {
     struct snmp_session  session, *ss;
     struct snmp_pdu *pdu, *response;
     struct variable_list *vars;
     int   arg;
-    oid   name[MAX_NAME_LEN];
+    oid   name[MAX_OID_LEN];
     int   name_length;
-    oid   root[MAX_NAME_LEN];
+    oid   root[MAX_OID_LEN];
     int   rootlen;
     int   count;
     int   running;
@@ -115,17 +104,14 @@ main(argc, argv)
     unsigned long histid1, histid2, orig_size, current_size;
 #endif
 
-    /* read in MIB database */
-    init_mib();
-
     /* get the common command line arguments */
     arg = snmp_parse_args(argc, argv, &session);
 
     /* get the initial object and subtree */
     if (arg < argc) {
       /* specified on the command line */
-      rootlen = MAX_NAME_LEN;
-      if (!read_objid(argv[arg], root, &rootlen)){
+      rootlen = MAX_OID_LEN;
+      if (snmp_parse_oid(argv[arg], root, &rootlen) == NULL) {
         fprintf(stderr, "Invalid object identifier: %s\n", argv[arg]);
         exit(1);
       }
@@ -138,14 +124,13 @@ main(argc, argv)
     orig_size = malloc_inuse(&histid1);
 #endif
 
-
     SOCK_STARTUP;
 
     /* open an SNMP session */
     snmp_synch_setup(&session);
     ss = snmp_open(&session);
     if (ss == NULL){
-      snmp_perror("snmpwalk: Couldn't open snmp");
+      snmp_perror("snmpwalk");
       SOCK_CLEANUP;
       exit(1);
     }
@@ -157,7 +142,7 @@ main(argc, argv)
     running = 1;
     while(running){
       /* create PDU for GETNEXT request and add object name to request */
-      pdu = snmp_pdu_create(GETNEXT_REQ_MSG);
+      pdu = snmp_pdu_create(SNMP_MSG_GETNEXT);
       snmp_add_null_var(pdu, name, name_length);
 
       /* do the request */
@@ -205,10 +190,10 @@ main(argc, argv)
           }
         }
       } else if (status == STAT_TIMEOUT){
-        fprintf(stderr, "No Response from %s\n", session.peername);
+        fprintf(stderr, "Timeout: No Response from %s\n", session.peername);
         running = 0;
       } else {    /* status == STAT_ERROR */
-        snmp_perror("snmpwalk: An error occurred");
+        snmp_perror("snmpwalk");
         running = 0;
       }
       if (response)
@@ -224,41 +209,3 @@ main(argc, argv)
     SOCK_CLEANUP;
     exit (0);
 }
-
-#if 0
-/*
- * to be part of security client library.
- */
-find_params(srcParty, dstParty, context, ipaddress, entity, time, security)
-    struct partyEntry *srcParty, *dstParty;
-    struct contextEntry *context;
-    u_long ipaddress;
-    char *entity;
-    char *time;
-    char *security;
-{
-    struct partyEntry *pp, *goodParties[32];
-    struct contextEntry *cxp, *goodContexts[32];
-    struct aclEntry *ap;
-    int numParties = 0, numContexts = 0;
-
-    party_scanInit();
-    for(pp = party_scanNext(); pp; pp = party_scanNext()){
-	if (pp->partyTDomain == 1 && !memcmp(pp->partyTAddress, &ipaddress, 4)){
-	    if (security == 0 || *security == '\0' || !strcmp(security, "*")){
-		goodParties[numParties++] = pp;
-	    } else if (!strcmp(security, "auth")
-		       && (pp->partyAuthProtocol == 6)){
-		goodParties[numParties++] = pp;
-	    } else if (!strcmp(security, "priv")
-		       && (pp->partyPrivProtocol == 4)){
-		goodParties[numParties++] = pp;
-	    }
-	}
-    }
-    /*
-     * Unfinished ...
-     */
-}
-
-#endif
