@@ -34,10 +34,10 @@ SOFTWARE.
 #include "gw.h"
 #endif
 
-#if HAVE_STRINGS_H
-#include <strings.h>
-#else
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 
 #include <sys/types.h>
@@ -59,6 +59,7 @@ SOFTWARE.
 #define NULL	0
 #endif
 
+#include "snmp_api.h"
 #include "snmp_impl.h" /* to define ERROR_MSG */
 
 /*
@@ -70,14 +71,21 @@ SOFTWARE.
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
- */
+
+  u_char * asn_parse_int(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      long       *intp         IN/OUT - pointer to start of output buffer
+      int         intsize      IN - size of output buffer
+*/
+
 u_char *
-asn_parse_int(data, datalength, type, intp, intsize)
-    register u_char	*data;	/* IN - pointer to start of object */
-    register int	*datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char		*type;	/* OUT - asn type of object */
-    long		*intp;	/* IN/OUT - pointer to start of output buffer */
-    int			intsize;    /* IN - size of output buffer */
+asn_parse_int(u_char *data,
+	      int *datalength,
+	      u_char *type,
+	      long *intp,
+	      int intsize)
 {
 /*
  * ASN.1 integer ::= 0x02 asnlength byte {byte}*
@@ -123,14 +131,20 @@ asn_parse_int(data, datalength, type, intp, intsize)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_parse_unsigned_int(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      u_long     *intp         IN/OUT - pointer to start of output buffer
+      int         intsize      IN - size of output buffer
  */
 u_char *
-asn_parse_unsigned_int(data, datalength, type, intp, intsize)
-    register u_char	    *data;	/* IN - pointer to start of object */
-    register int	    *datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char		    *type;	/* OUT - asn type of object */
-    u_long		    *intp;	/* IN/OUT - pointer to start of output buffer */
-    int			    intsize;    /* IN - size of output buffer */
+asn_parse_unsigned_int(u_char *data,
+		       int *datalength,
+		       u_char *type,
+		       u_long *intp,
+		       int intsize)
 {
 /*
  * ASN.1 integer ::= 0x02 asnlength byte {byte}*
@@ -177,14 +191,20 @@ asn_parse_unsigned_int(data, datalength, type, intp, intsize)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_build_int(
+      u_char     *data         IN - pointer to start of output buffer
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN  - asn type of object
+      long       *intp         IN - pointer to start of long integer
+      int         intsize      IN - size of input buffer
  */
 u_char *
-asn_build_int(data, datalength, type, intp, intsize)
-    register u_char *data;	/* IN - pointer to start of output buffer */
-    register int    *datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	/* IN - asn type of object */
-    register long   *intp;	/* IN - pointer to start of long integer */
-    register int    intsize;    /* IN - size of *intp */
+asn_build_int(u_char *data,
+	      int *datalength,
+	      u_char type,
+	      long *intp,
+	      int intsize)
 {
 /*
  * ASN.1 integer ::= 0x02 asnlength byte {byte}*
@@ -234,14 +254,20 @@ asn_build_int(data, datalength, type, intp, intsize)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_build_unsigned_int(
+      u_char     *data         IN - pointer to start of output buffer
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN  - asn type of object
+      u_long     *intp         IN - pointer to start of long integer
+      int         intsize      IN - size of input buffer
  */
 u_char *
-asn_build_unsigned_int(data, datalength, type, intp, intsize)
-    register u_char *data;	/* IN - pointer to start of output buffer */
-    register int    *datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	/* IN - asn type of object */
-    register u_long *intp;	/* IN - pointer to start of long integer */
-    register int    intsize;    /* IN - size of *intp */
+asn_build_unsigned_int(u_char *data,
+		       int *datalength,
+		       u_char type,
+		       u_long *intp,
+		       int intsize)
 {
 /*
  * ASN.1 integer ::= 0x02 asnlength byte {byte}*
@@ -304,14 +330,20 @@ asn_build_unsigned_int(data, datalength, type, intp, intsize)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_parse_string(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      u_char     *string       IN/OUT - pointer to start of output buffer
+      int        *strlength    IN/OUT - size of output buffer
  */
 u_char *
-asn_parse_string(data, datalength, type, string, strlength)
-    u_char	    *data;	    /* IN - pointer to start of object */
-    register int    *datalength;    /* IN/OUT - number of valid bytes left in buffer */
-    u_char	    *type;	    /* OUT - asn type of object */
-    u_char	    *string;	    /* IN/OUT - pointer to start of output buffer */
-    register int    *strlength;     /* IN/OUT - size of output buffer */
+asn_parse_string(u_char *data,
+		 int *datalength,
+		 u_char *type,
+		 u_char *string,
+		 int    *strlength)
 {
 /*
  * ASN.1 octet string ::= primstring | cmpdstring
@@ -349,14 +381,20 @@ asn_parse_string(data, datalength, type, string, strlength)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_build_string(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
+      u_char     *string       IN - pointer to start of input buffer
+      int         strlength    IN - size of input buffer
  */
 u_char *
-asn_build_string(data, datalength, type, string, strlength)
-    u_char	    *data;	    /* IN - pointer to start of object */
-    register int    *datalength;    /* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	    /* IN - ASN type of string */
-    u_char	    *string;	    /* IN - pointer to start of input buffer */
-    register int    strlength;	    /* IN - size of input buffer */
+asn_build_string(u_char *data,
+		 int *datalength,
+		 u_char	type,
+		 u_char *string,
+		 int strlength)
 {
 /*
  * ASN.1 octet string ::= primstring | cmpdstring
@@ -383,15 +421,19 @@ asn_build_string(data, datalength, type, string, strlength)
  *
  *  Returns a pointer to the first byte of the contents of this object.
  *  Returns NULL on any error.
+
+  u_char * asn_parse_header(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
  */
 u_char *
-asn_parse_header(data, datalength, type)
-    u_char	*data;	/* IN - pointer to start of object */
-    int		*datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char	*type;	/* OUT - ASN type of object */
+asn_parse_header(u_char	*data,
+		 int *datalength,
+		 u_char	*type)
 {
     register u_char *bufp = data;
-    register	    header_len;
+    register int    header_len;
     u_long	    asn_length;
 
     /* this only works on data types < 30, i.e. no extension octets */
@@ -408,6 +450,36 @@ asn_parse_header(data, datalength, type)
 	ERROR_MSG("asn length too long");
 	return NULL;
     }
+#ifdef OPAQUE_SPECIAL_TYPES
+
+    if ((*type == ASN_OPAQUE) &&
+        (*bufp == ASN_OPAQUE_TAG1)) {
+      /* check if 64-but counter */
+      switch(*(bufp+1)) {
+        case ASN_OPAQUE_COUNTER64:
+        case ASN_OPAQUE_U64:
+        case ASN_OPAQUE_FLOAT:
+        case ASN_OPAQUE_DOUBLE:
+        case ASN_OPAQUE_I64:
+          *type = *(bufp+1);
+          break;
+        
+        default:
+          /* just an Opaque */
+          *datalength = (int)asn_length;
+          return bufp;
+      }
+      /* value is encoded as special format */
+      bufp = asn_parse_length(bufp + 2, &asn_length);
+      if (bufp == NULL)
+        return NULL;
+      header_len = bufp - data;
+      if ((header_len + (int)asn_length) > *datalength){
+        ERROR_MSG("asn length too long");
+        return NULL;
+      }
+    }
+#endif /* OPAQUE_SPECIAL_TYPES */
     *datalength = (int)asn_length;
     return bufp;
 }
@@ -424,13 +496,18 @@ asn_parse_header(data, datalength, type)
  *
  *  Returns a pointer to the first byte of the contents of this object.
  *  Returns NULL on any error.
+
+  u_char * asn_build_header(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
+      int         length       IN - length of object
  */
 u_char *
-asn_build_header(data, datalength, type, length)
-    register u_char *data;	/* IN - pointer to start of object */
-    int		    *datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	/* IN - ASN type of object */
-    int		    length;	/* IN - length of object */
+asn_build_header (u_char *data,
+		  int *datalength,
+		  u_char type,
+		  int length)
 {
     if (*datalength < 1)
 	return NULL;
@@ -452,13 +529,18 @@ asn_build_header(data, datalength, type, length)
  *
  *  Returns a pointer to the first byte of the contents of this object.
  *  Returns NULL on any error.
+
+  u_char * asn_build_sequence(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
+      int         length       IN - length of object
  */
 u_char *
-asn_build_sequence(data, datalength, type, length)
-    register u_char *data;	/* IN - pointer to start of object */
-    int		    *datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	/* IN - ASN type of object */
-    int		    length;	/* IN - length of object */
+asn_build_sequence(u_char *data,
+		  int *datalength,
+		  u_char type,
+		  int length)
 {
     *datalength -= 4;
     if (*datalength < 0){
@@ -479,11 +561,14 @@ asn_build_sequence(data, datalength, type, length)
  *  Returns a pointer to the first byte after this length
  *  field (aka: the start of the data field).
  *  Returns NULL on any error.
+
+  u_char * asn_parse_length(
+      u_char     *data         IN - pointer to start of length field
+      u_long     *length       OUT - value of length field
  */
 u_char *
-asn_parse_length(data, length)
-    u_char  *data;	/* IN - pointer to start of length field */
-    u_long  *length;	/* OUT - value of length field */
+asn_parse_length(u_char  *data,
+		 u_long  *length)
 {
     register u_char lengthbyte = *data;
 
@@ -497,9 +582,10 @@ asn_parse_length(data, length)
 	    ERROR_MSG("we can't support data lengths that long");
 	    return NULL;
 	}
+	*length = 0;  /* protect against short lengths */
 	memmove(length, data + 1, (int)lengthbyte);
 	*length = ntohl(*length);
-	*length >>= (8 * ((sizeof(int)) - lengthbyte));
+	*length >>= (8 * ((sizeof(u_long)) - lengthbyte));
 	return data + lengthbyte + 1;
     } else { /* short asnlength */
 	*length = (long)lengthbyte;
@@ -507,11 +593,17 @@ asn_parse_length(data, length)
     }
 }
 
+/*
+
+  u_char * asn_build_length(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      int         length       IN - length of object
+ */
 u_char *
-asn_build_length(data, datalength, length)
-    register u_char *data;	/* IN - pointer to start of object */
-    int		    *datalength;/* IN/OUT - number of valid bytes left in buffer */
-    register int    length;	/* IN - length of object */
+asn_build_length(u_char *data,
+		 int *datalength,
+		 int length)
 {
     u_char    *start_data = data;
 
@@ -554,14 +646,20 @@ asn_build_length(data, datalength, length)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_parse_objid(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      oid        *objid        IN/OUT - pointer to start of output buffer
+      int        *objidlength  IN/OUT - number of sub-id's in objid
  */
 u_char *
-asn_parse_objid(data, datalength, type, objid, objidlength)
-    u_char	*data;		/* IN - pointer to start of object */
-    int		*datalength;	/* IN/OUT - number of valid bytes left in buffer */
-    u_char	*type;		/* OUT - ASN type of object */
-    oid		*objid;		/* IN/OUT - pointer to start of output buffer */
-    int		*objidlength;	/* IN/OUT - number of sub-id's in objid */
+asn_parse_objid(u_char *data,
+		int *datalength,
+		u_char *type,	
+		oid *objid,
+		int *objidlength)
 {
 /*
  * ASN.1 objid ::= 0x06 asnlength subidentifier {subidentifier}*
@@ -623,9 +721,12 @@ asn_parse_objid(data, datalength, type, objid, objidlength)
         } else if (subidentifier < 80) {
             objid[0] = 1;
             objid[1] = subidentifier - 40;
-        } else {
+        } else if (subidentifier < 120) {
             objid[0] = 2;
             objid[1] = subidentifier - 80;
+        } else {
+	    objid[1] = (subidentifier % 40);
+	    objid[0] = ((subidentifier - objid[1]) / 40);
         }
     }
 
@@ -643,14 +744,20 @@ asn_parse_objid(data, datalength, type, objid, objidlength)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_build_objid(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
+      oid        *objid        IN - pointer to start of input buffer
+      int         objidlength  IN - number of sub-id's in objid
  */
 u_char *
-asn_build_objid(data, datalength, type, objid, objidlength)
-    register u_char *data;	    /* IN - pointer to start of object */
-    int		    *datalength;    /* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	    /* IN - ASN type of object */
-    oid		    *objid;	    /* IN - pointer to start of input buffer */
-    int		    objidlength;    /* IN - number of sub-id's in objid */
+asn_build_objid(u_char *data,
+		int *datalength,
+		u_char type,
+		oid *objid,
+		int objidlength)
 {
 /*
  * ASN.1 objid ::= 0x06 asnlength subidentifier {subidentifier}*
@@ -710,7 +817,8 @@ asn_build_objid(data, datalength, type, objid, objidlength)
 
     /* store the encoded OID value */
     for (i = 1, objid_val = first_objid_val, op = objid+2; i < objidlength;
-                i++, objid_val = *op++) {
+                i++) {
+      if (i != 1) objid_val = *op++;
         switch (objid_size[i]) {
         case 1:
             *data++ = (u_char)objid_val;
@@ -742,7 +850,7 @@ asn_build_objid(data, datalength, type, objid, objidlength)
             *data++ = (u_char)(objid_val & 0x07f);
             break;
         }
-    }    
+    }
 
     /* return the length and data ptr */
     *datalength -= asnlength;
@@ -758,12 +866,16 @@ asn_build_objid(data, datalength, type, objid, objidlength)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_parse_null(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
  */
 u_char *
-asn_parse_null(data, datalength, type)
-    u_char	    *data;	    /* IN - pointer to start of object */
-    int		    *datalength;    /* IN/OUT - number of valid bytes left in buffer */
-    u_char	    *type;	    /* OUT - ASN type of object */
+asn_parse_null(u_char *data,
+	       int *datalength,
+	       u_char *type)
 {
 /*
  * ASN.1 null ::= 0x05 0x00
@@ -793,12 +905,16 @@ asn_parse_null(data, datalength, type)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_build_null(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
  */
 u_char *
-asn_build_null(data, datalength, type)
-    u_char	    *data;	    /* IN - pointer to start of object */
-    int		    *datalength;    /* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	    /* IN - ASN type of object */
+asn_build_null(u_char *data,
+	       int *datalength,
+	       u_char type)
 {
 /*
  * ASN.1 null ::= 0x05 0x00
@@ -817,14 +933,20 @@ asn_build_null(data, datalength, type)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_parse_bitstring(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      u_char     *string       IN/OUT - pointer to start of output buffer
+      int        *strlength    IN/OUT - size of output buffer
  */
 u_char *
-asn_parse_bitstring(data, datalength, type, string, strlength)
-    u_char	    *data;	    /* IN - pointer to start of object */
-    register int    *datalength;    /* IN/OUT - number of valid bytes left in buffer */
-    u_char	    *type;	    /* OUT - asn type of object */
-    u_char	    *string;	    /* IN/OUT - pointer to start of output buffer */
-    register int    *strlength;     /* IN/OUT - size of output buffer */
+asn_parse_bitstring(u_char *data,
+		    int *datalength,
+		    u_char *type,
+		    u_char *string,
+		    int *strlength)
 {
 /*
  * bitstring ::= 0x03 asnlength unused {byte}*
@@ -869,14 +991,20 @@ asn_parse_bitstring(data, datalength, type, string, strlength)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_build_bitstring(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
+      u_char     *string       IN - pointer to start of input buffer
+      int         strlength    IN - size of input buffer
  */
 u_char *
-asn_build_bitstring(data, datalength, type, string, strlength)
-    u_char	    *data;	    /* IN - pointer to start of object */
-    register int    *datalength;    /* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	    /* IN - ASN type of string */
-    u_char	    *string;	    /* IN - pointer to start of input buffer */
-    register int    strlength;	    /* IN - size of input buffer */
+asn_build_bitstring(u_char *data,
+		    int *datalength,
+		    u_char type,
+		    u_char *string,
+		    int strlength)
 {
 /*
  * ASN.1 bit string ::= 0x03 asnlength unused {byte}*
@@ -906,14 +1034,20 @@ asn_build_bitstring(data, datalength, type, string, strlength)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_parse_unsigned_int64(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      struct counter64 *cp     IN/OUT - pointer to counter struct
+      int         countersize  IN - size of output buffer
  */
 u_char *
-asn_parse_unsigned_int64(data, datalength, type, cp, countersize)
-    register u_char	    *data;	/* IN - pointer to start of object */
-    register int	    *datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char		    *type;	/* OUT - asn type of object */
-    struct counter64	    *cp;	/* IN/OUT -pointer to counter struct */
-    int			    countersize;/* IN - size of output buffer */
+asn_parse_unsigned_int64(u_char *data,
+			 int *datalength,
+			 u_char *type, 
+			 struct counter64 *cp,
+			 int countersize)
 {
 /*
  * ASN.1 integer ::= 0x02 asnlength byte {byte}*
@@ -937,6 +1071,25 @@ asn_parse_unsigned_int64(data, datalength, type, cp, countersize)
 	ERROR_MSG("overflow of message");
 	return NULL;
     }
+#ifdef OPAQUE_SPECIAL_TYPES
+/* 64 bit counters as opaque */
+    if ((*type == ASN_OPAQUE) &&
+            (asn_length <= ASN_OPAQUE_COUNTER64_MX_BER_LEN) &&
+	    (*bufp == ASN_OPAQUE_TAG1) &&
+	    ((*(bufp+1) == ASN_OPAQUE_COUNTER64) ||
+             (*(bufp+1) == ASN_OPAQUE_U64))) {
+	/* change type to Counter64 or U64 */
+        *type = *(bufp+1);
+        /* value is encoded as special format */
+	bufp = asn_parse_length(bufp + 2, &asn_length);
+        if (bufp == NULL)
+            return NULL;
+        if ((int)(asn_length + (bufp - data)) > *datalength){
+            ERROR_MSG("overflow of message");
+            return NULL;
+        }
+    }
+#endif /* OPAQUE_SPECIAL_TYPES */
     if (((int)asn_length > (intsize * 2 + 1)) ||
 	(((int)asn_length == (intsize * 2 + 1)) && *bufp != 0x00)){
 	ERROR_MSG("I don't support such large integers");
@@ -966,14 +1119,20 @@ asn_parse_unsigned_int64(data, datalength, type, cp, countersize)
  *  Returns a pointer to the first byte past the end
  *   of this object (i.e. the start of the next object).
  *  Returns NULL on any error.
+
+  u_char * asn_build_unsigned_int64(
+      u_char     *data         IN - pointer to start of output buffer
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN  - asn type of object
+      struct counter64 *cp     IN - pointer to counter struct
+      int         countersize  IN - size of input buffer
  */
 u_char *
-asn_build_unsigned_int64(data, datalength, type, cp, countersize)
-    register u_char *data;	/* IN - pointer to start of output buffer */
-    register int    *datalength;/* IN/OUT - number of valid bytes left in buffer */
-    u_char	    type;	/* IN - asn type of object */
-    struct counter64 *cp;	/* IN - pointer to counter struct */
-    register int    countersize; /* IN - size of *intp */
+asn_build_unsigned_int64(u_char *data,
+			 int *datalength,
+			 u_char type,
+			 struct counter64 *cp,
+			 int countersize)
 {
 /*
  * ASN.1 integer ::= 0x02 asnlength byte {byte}*
@@ -1011,11 +1170,49 @@ asn_build_unsigned_int64(data, datalength, type, cp, countersize)
 	    low <<= 8;
 	}
     }
+#ifdef OPAQUE_SPECIAL_TYPES
+/* encode a Counter64 as an opaque (it also works in SNMPv1) */
+    /* turn into Opaque holding special tagged value */
+    if (type == ASN_OPAQUE_COUNTER64) {
+        /* put the tag and length for the Opaque wrapper */
+        data = asn_build_header(data, datalength, ASN_OPAQUE, intsize+3);
+        if (data == NULL)
+            return NULL;
+        if (*datalength < (intsize+3))
+            return NULL;
+	/* put the special tag and length */
+	*data++ = ASN_OPAQUE_TAG1;
+	*data++ = ASN_OPAQUE_COUNTER64;
+	*data++ = (u_char)intsize;
+	*datalength = *datalength - 3;
+    }
+    else
+/* Encode the Unsigned int64 in an opaque */
+    /* turn into Opaque holding special tagged value */
+    if (type == ASN_OPAQUE_U64) {
+        /* put the tag and length for the Opaque wrapper */
+        data = asn_build_header(data, datalength, ASN_OPAQUE, intsize+3);
+        if (data == NULL)
+            return NULL;
+        if (*datalength < (intsize+3))
+            return NULL;
+	/* put the special tag and length */
+	*data++ = ASN_OPAQUE_TAG1;
+	*data++ = ASN_OPAQUE_U64;
+	*data++ = (u_char)intsize;
+	*datalength = *datalength - 3;
+    }
+    else
+    {
+#endif /* OPAQUE_SPECIAL_TYPES */
     data = asn_build_header(data, datalength, type, intsize);
     if (data == NULL)
 	return NULL;
     if (*datalength < intsize)
 	return NULL;
+#ifdef OPAQUE_SPECIAL_TYPES
+    }
+#endif /* OPAQUE_SPECIAL_TYPES */
     *datalength -= intsize;
     if (add_null_byte == 1){
 	*data++ = '\0';
@@ -1030,3 +1227,421 @@ asn_build_unsigned_int64(data, datalength, type, cp, countersize)
     }
     return data;
 }
+
+#ifdef OPAQUE_SPECIAL_TYPES
+
+/*
+
+  u_char * asn_parse_signed_int64(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      struct counter64 *cp     IN/OUT - pointer to counter struct
+      int         countersize  IN - size of output buffer
+ */
+
+u_char *
+asn_parse_signed_int64(u_char *data,
+		       int *datalength,
+		       u_char *type,
+		       struct counter64 *cp,
+		       int countersize)
+{
+  register u_char *bufp = data;
+  u_long	    asn_length;
+  register u_int low = 0, high = 0;
+  int intsize = 4;
+    
+  if (countersize != sizeof(struct counter64)){
+    ERROR_MSG("not right size");
+    return NULL;
+  }
+  *type = *bufp++;
+  bufp = asn_parse_length(bufp, &asn_length);
+  if (bufp == NULL){
+    ERROR_MSG("bad length");
+    return NULL;
+  }
+  if ((asn_length + (bufp - data)) > (u_int)(*datalength)){
+    ERROR_MSG("overflow of message");
+    return NULL;
+  }
+  if ((*type == ASN_OPAQUE) &&
+      (asn_length <= ASN_OPAQUE_COUNTER64_MX_BER_LEN) &&
+      (*bufp == ASN_OPAQUE_TAG1) &&
+       (*(bufp+1) == ASN_OPAQUE_I64)) {
+    /* change type to Int64 */
+    *type = *(bufp+1);
+    /* value is encoded as special format */
+    bufp = asn_parse_length(bufp + 2, &asn_length);
+    if (bufp == NULL)
+      return NULL;
+    if ((int)(asn_length + (bufp - data)) > *datalength){
+      ERROR_MSG("overflow of message");
+      return NULL;
+    }
+  }
+  /* this should always have been true until snmp gets int64 PDU types */
+  else {
+    ERROR_MSG("wrong type");
+    return NULL;
+  }
+  if (((int)asn_length > (intsize * 2 + 1)) ||
+      (((int)asn_length == (intsize * 2 + 1)) && *bufp != 0x00)){
+    ERROR_MSG("I don't support such large integers");
+    return NULL;
+  }
+  *datalength -= (int)asn_length + (bufp - data);
+  if (*bufp & 0x80){
+    low = ~low; /* integer is negative */
+    high = ~high;
+  }
+  while(asn_length--){
+    high = (high << 8) | ((low & 0xFF000000) >> 24);
+    low = (low << 8) | *bufp++;
+  }
+  cp->low = low;
+  cp->high = high;
+  return bufp;
+}
+
+
+/*
+
+  u_char * asn_build_signed_int64(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
+      struct counter64 *cp     IN - pointer to counter struct
+      int         countersize  IN - size of input buffer
+ */
+u_char *
+asn_build_signed_int64(u_char *data,
+		       int *datalength,
+		       u_char type,
+		       struct counter64 *cp,
+		       int countersize)
+{
+/*
+ * ASN.1 integer ::= 0x02 asnlength byte {byte}*
+ */
+
+    struct counter64 c64;
+    register u_int mask, mask2;
+    u_long low, high;
+    int intsize;
+
+    if (countersize != sizeof (struct counter64))
+	return NULL;
+    intsize = 8;
+    memcpy(&c64, cp, sizeof(struct counter64));  /* we're may modify it */
+    low = c64.low;
+    high = c64.high;
+    
+    /*
+     * Truncate "unnecessary" bytes off of the most significant end of this
+     * 2's complement integer.  There should be no sequence of 9
+     * consecutive 1's or 0's at the most significant end of the
+     * integer.
+     */
+    mask = ((u_int) 0xFF) << (8 * (sizeof(u_int) - 1));
+    mask2 = ((u_int) 0x1FF) << ((8 * (sizeof(u_int) - 1)) - 1);
+    /* mask is 0xFF800000 on a big-endian machine */
+    while((((high & mask2) == 0) || ((high & mask2) == mask2)) && intsize > 1){
+      intsize--;
+      high = (high << 8)
+        | ((low & mask) >> (8 * (sizeof(u_int) - 1)));
+      low <<= 8;
+    }
+    /* until a real int64 gets incorperated into SNMP, we are going to
+       encode it as an opaque instead.  First, we build the opaque
+       header and then the int64 tag type we use to mark it as an
+       int64 in the opaque string. */
+    data = asn_build_header(data, datalength, ASN_OPAQUE, intsize+3);
+    if (data == NULL)
+	return NULL;
+    if (*datalength < (intsize+3))
+	return NULL;
+    *data++ = ASN_OPAQUE_TAG1;
+    *data++ = ASN_OPAQUE_I64;
+    *data++ = (u_char)intsize;
+    *datalength -= (3 + intsize);
+    
+    while(intsize--){
+	*data++ = (u_char)((high & mask) >> (8 * (sizeof(u_int) - 1)));
+	high = (high << 8)
+	    | ((low & mask) >> (8 * (sizeof(u_int) - 1)));
+	low <<= 8;
+    }
+    return data;
+}
+
+/*
+ * asn_parse_float - pulls a single precision floating-point out of an opaque type.
+ *
+ *  On entry, datalength is input as the number of valid bytes following
+ *   "data".  On exit, it is returned as the number of valid bytes
+ *   following the end of this object.
+ *
+ *  Returns a pointer to the first byte past the end
+ *   of this object (i.e. the start of the next object).
+ *  Returns NULL on any error.
+
+  u_char * asn_parse_float(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      float      *floatp       IN/OUT - pointer to float
+      int         floatsize    IN - size of output buffer
+ */
+u_char *
+asn_parse_float(u_char *data,
+		int *datalength,
+		u_char *type,
+		float *floatp,
+		int floatsize)
+{
+    register u_char *bufp = data;
+    u_long	    asn_length;
+    union {
+        float  floatVal;
+	long   longVal;
+	u_char c[sizeof(float)];
+    } fu;
+
+    if (floatsize != sizeof(float)){
+	ERROR_MSG("not right size");
+	return NULL;
+    }
+    *type = *bufp++;
+    bufp = asn_parse_length(bufp, &asn_length);
+    if (bufp == NULL){
+	ERROR_MSG("bad length");
+	return NULL;
+    }
+    if ((int)(asn_length + (bufp - data)) > *datalength){
+	ERROR_MSG("overflow of message");
+	return NULL;
+    }
+/* the float is encoded as an opaque */
+    if ((*type == ASN_OPAQUE) &&
+            (asn_length == ASN_OPAQUE_FLOAT_BER_LEN) &&
+	    (*bufp == ASN_OPAQUE_TAG1) &&
+	    (*(bufp+1) == ASN_OPAQUE_FLOAT)) {
+        /* value is encoded as special format */
+	bufp = asn_parse_length(bufp + 2, &asn_length);
+        if (bufp == NULL)
+            return NULL;
+        if ((int)(asn_length + (bufp - data)) > *datalength){
+            ERROR_MSG("overflow of message");
+            return NULL;
+        }
+	/* change type to Float */
+	*type = ASN_OPAQUE_FLOAT;
+    }
+
+    if (asn_length != sizeof(float)) {
+	ERROR_MSG("Size of float incorrect");
+	return NULL;
+    }
+    *datalength -= (int)asn_length + (bufp - data);
+    memcpy(&fu.c[0], bufp, asn_length);
+
+   /* correct for endian differences */
+    fu.longVal = ntohl(fu.longVal);	
+
+    *floatp =  fu.floatVal;
+
+    return bufp;
+}
+
+/*
+ * asn_build_float - builds an ASN object containing a single precision floating-point
+ *                    number in an Opaque value.
+ *
+ *  On entry, datalength is input as the number of valid bytes following
+ *   "data".  On exit, it is returned as the number of valid bytes
+ *   following the end of this object.
+ *
+ *  Returns a pointer to the first byte past the end
+ *   of this object (i.e. the start of the next object).
+ *  Returns NULL on any error.
+
+  u_char * asn_build_float(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
+      float      *floatp       IN - pointer to float
+      int         floatsize    IN - size of input buffer
+ */
+u_char *
+asn_build_float(u_char *data,
+		int *datalength,
+		u_char type,
+		float *floatp,
+		int floatsize)
+{
+    union {
+        float  floatVal;
+        int    intVal;
+	u_char c[sizeof(float)];
+    } fu;
+
+    if (floatsize != sizeof (float))
+	return NULL;
+
+/* encode the float as an opaque */
+    /* turn into Opaque holding special tagged value */
+
+    /* put the tag and length for the Opaque wrapper */
+    data = asn_build_header(data, datalength, ASN_OPAQUE, floatsize+3);
+    if (data == NULL)
+        return NULL;
+    if (*datalength < (floatsize+3))
+        return NULL;
+    /* put the special tag and length */
+    *data++ = ASN_OPAQUE_TAG1;
+    *data++ = ASN_OPAQUE_FLOAT;
+    *data++ = (u_char)floatsize;
+    *datalength = *datalength - 3;
+
+    fu.floatVal = *floatp;
+    /* correct for endian differences */
+    fu.intVal = htonl(fu.intVal);	
+
+    *datalength -= floatsize;
+    memcpy(data, &fu.c[0], floatsize);
+
+    data += floatsize;
+    return data;
+}
+
+/*
+
+  u_char * asn_parse_double(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char     *type         OUT - asn type of object
+      double     *doublep      IN/OUT - pointer to double
+      int         doublesize   IN - size of output buffer
+ */
+u_char *
+asn_parse_double(u_char *data,
+		 int *datalength,
+		 u_char *type,
+		 double *doublep,
+		 int doublesize)
+{
+    register u_char *bufp = data;
+    u_long	    asn_length;
+    long            tmp;
+    union {
+        double doubleVal;
+        int    intVal[2];
+	u_char c[sizeof(double)];
+    } fu;
+    
+
+    if (doublesize != sizeof(double)){
+	ERROR_MSG("not right size");
+	return NULL;
+    }
+    *type = *bufp++;
+    bufp = asn_parse_length(bufp, &asn_length);
+    if (bufp == NULL){
+	ERROR_MSG("bad length");
+	return NULL;
+    }
+    if ((int)(asn_length + (bufp - data)) > *datalength){
+	ERROR_MSG("overflow of message");
+	return NULL;
+    }
+/* the double is encoded as an opaque */
+    if ((*type == ASN_OPAQUE) &&
+            (asn_length == ASN_OPAQUE_DOUBLE_BER_LEN) &&
+	    (*bufp == ASN_OPAQUE_TAG1) &&
+	    (*(bufp+1) == ASN_OPAQUE_DOUBLE)) {
+        /* value is encoded as special format */
+	bufp = asn_parse_length(bufp + 2, &asn_length);
+        if (bufp == NULL)
+            return NULL;
+        if ((int)(asn_length + (bufp - data)) > *datalength){
+            ERROR_MSG("overflow of message");
+            return NULL;
+        }
+	/* change type to Double */
+	*type = ASN_OPAQUE_DOUBLE;
+    }
+
+    if (asn_length != sizeof(double)) {
+	ERROR_MSG("Size of double incorrect");
+	return NULL;
+    }
+    *datalength -= (int)asn_length + (bufp - data);
+    memcpy(&fu.c[0], bufp, asn_length);
+
+   /* correct for endian differences */
+
+    tmp = ntohl(fu.intVal[0]);
+    fu.intVal[0] = ntohl(fu.intVal[1]);
+    fu.intVal[1] = tmp;
+    	
+    *doublep =  fu.doubleVal;
+
+    return bufp;
+}
+
+/*
+
+  u_char * asn_build_double(
+      u_char     *data         IN - pointer to start of object
+      int        *datalength   IN/OUT - number of valid bytes left in buffer
+      u_char      type         IN - asn type of object
+      double     *doublep      IN - pointer to double
+      int         doublesize   IN - size of input buffer
+ */
+u_char *
+asn_build_double(u_char *data,
+		 int *datalength,
+		 u_char type,
+		 double* doublep,
+		 int doublesize)
+{
+    long  tmp;
+    union {
+        double doubleVal;
+	int    intVal[2];
+	u_char c[sizeof(double)];
+    } fu;
+
+    if (doublesize != sizeof (double))
+	return NULL;
+
+/* encode the double as an opaque */
+    /* turn into Opaque holding special tagged value */
+
+    /* put the tag and length for the Opaque wrapper */
+    data = asn_build_header(data, datalength, ASN_OPAQUE, doublesize+3);
+    if (data == NULL)
+        return NULL;
+    if (*datalength < (doublesize+3))
+        return NULL;
+    /* put the special tag and length */
+    *data++ = ASN_OPAQUE_TAG1;
+    *data++ = ASN_OPAQUE_DOUBLE;
+    *data++ = (u_char)doublesize;
+    *datalength = *datalength - 3;
+
+    fu.doubleVal = *doublep;
+    /* correct for endian differences */
+    tmp = htonl(fu.intVal[0]);
+    fu.intVal[0] = htonl(fu.intVal[1]);	
+    fu.intVal[1] = tmp;
+    *datalength -= doublesize;
+    memcpy(data, &fu.c[0], doublesize);
+
+    data += doublesize;
+    return data;
+}
+
+#endif /* OPAQUE_SPECIAL_TYPES */
