@@ -1,3 +1,11 @@
+#ifndef PARSE_H
+#define PARSE_H
+/*
+ * parse.h
+ *
+ * Update: 1998-07-17 <jhy@gsu.edu>
+ * Added prototypes for print_subtree_oid_report* functions.
+ */
 /***********************************************************
         Copyright 1989 by Carnegie Mellon University
 
@@ -19,9 +27,6 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 ******************************************************************/
-/*
- * parse.h
- */
 
 #define MAXLABEL        64      /* maximum characters in a label */
 #define MAXTOKEN        128     /* maximum characters in a token */
@@ -77,9 +82,10 @@ struct tree {
     struct enum_list *enums;    /* (optional) list of enumerated integers */
     char *hint;
     char *units;
-    void (*printer) __P((char *, struct variable_list *, struct enum_list *,
-                         char *, char *));	/* Value printing function */
+    void (*printer) (char *, struct variable_list *, struct enum_list *,
+                         char *, char *);	/* Value printing function */
     char *description;          /* description (a quoted string) */
+    int  reported;              /* 1=report started in print_subtree... */
 };
 
 /*
@@ -126,30 +132,67 @@ struct module_compatability {
 #define TYPE_NSAPADDRESS    13
 #define TYPE_UINTEGER       14
 
-#define ACCESS_READONLY    18
-#define ACCESS_READWRITE   19
-#define	ACCESS_WRITEONLY   20
-#define ACCESS_NOACCESS    21
-#define ACCESS_NOTIFY      67
-#define ACCESS_CREATE      48
+#define MIB_ACCESS_READONLY    18
+#define MIB_ACCESS_READWRITE   19
+#define	MIB_ACCESS_WRITEONLY   20
+#define MIB_ACCESS_NOACCESS    21
+#define MIB_ACCESS_NOTIFY      67
+#define MIB_ACCESS_CREATE      48
 
-#define STATUS_MANDATORY   23
-#define STATUS_OPTIONAL    24
-#define STATUS_OBSOLETE    25
-#define STATUS_DEPRECATED  39
+#define MIB_STATUS_MANDATORY   23
+#define MIB_STATUS_OPTIONAL    24
+#define MIB_STATUS_OBSOLETE    25
+#define MIB_STATUS_DEPRECATED  39
+#define MIB_STATUS_CURRENT     57
 
+#ifdef CMU_COMPATIBLE
+#define ACCESS_READONLY		MIB_ACCESS_READONLY
+#define ACCESS_READWRITE	MIB_ACCESS_READWRITE
+#define ACCESS_WRITEONLY	MIB_ACCESS_WRITEONLY
+#define ACCESS_NOACCESS		MIB_ACCESS_NOACCESS
+#define ACCESS_NOTIFY		MIB_ACCESS_NOTIFY
+#define ACCESS_CREATE		MIB_ACCESS_CREATE
+#define STATUS_MANDATORY	MIB_STATUS_MANDATORY
+#define STATUS_OPTIONAL		MIB_STATUS_OPTIONAL
+#define STATUS_OBSOLETE		MIB_STATUS_OBSOLETE
+#define STATUS_DEPRECATED	MIB_STATUS_DEPRECATED
+#define STATUS_CURRENT		MIB_STATUS_CURRENT
+#endif	/* CMU_COMPATIBLE */
 
 #define	ANON	"anonymous#"
 #define	ANON_LEN  strlen(ANON)
 
-struct tree *read_module __P((char *));
-struct tree *read_mib __P((char *));
-struct tree *read_all_mibs __P((void));
-int  add_mibdir __P((char *));
-void add_module_replacement __P(( char *, char *, char *, int));
-void print_subtree __P((FILE *, struct tree *, int));
-void print_ascii_dump_tree __P((FILE *, struct tree *, int));
+struct tree *read_module (char *);
+struct tree *read_mib (char *);
+struct tree *read_all_mibs (void);
+void init_mib_internals (void);
+int  add_mibdir (char *);
+void add_module_replacement ( char *, char *, char *, int);
+int  which_module (char *);
+char *module_name (int, char *);
+void print_subtree (FILE *, struct tree *, int);
+void print_ascii_dump_tree (FILE *, struct tree *, int);
+void print_mib_tree(FILE *, struct tree *);
+struct tree *find_tree_node (char *, int);
+char *get_tc_descriptor (int);
  /* backwards compatability */
-struct tree *find_node __P((char *, struct tree*));
-void snmp_set_mib_warnings __P((int));
-void snmp_set_save_descriptions __P((int));
+struct tree *find_node (char *, struct tree*);
+struct module *find_module (int);
+void adopt_orphans (void);
+void snmp_set_mib_warnings (int);
+void snmp_set_mib_errors (int);
+void snmp_set_save_descriptions (int);
+void snmp_set_mib_comment_term (int);
+void snmp_set_mib_parse_label (int);
+char *snmp_mib_toggle_options(char *options);
+void snmp_mib_toggle_options_usage(char *lead, FILE *outf);
+void print_subtree_oid_report (FILE *, struct tree *, int);
+void print_subtree_oid_report_enable_labeledoid (void);
+void print_subtree_oid_report_enable_oid (void);
+void print_subtree_oid_report_enable_suffix (void);
+void print_subtree_oid_report_enable_symbolic (void);
+void print_subtree_oid_report_disable_labeledoid (void);
+void print_subtree_oid_report_disable_oid (void);
+void print_subtree_oid_report_disable_suffix (void);
+void print_subtree_oid_report_disable_symbolic (void);
+#endif /* PARSE_H */
