@@ -33,10 +33,10 @@ SOFTWARE.
 #endif
 
 #include <stdio.h>
-#if HAVE_STRINGS_H
-#include <strings.h>
-#else
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 #include <sys/types.h>
 #if TIME_WITH_SYS_TIME
@@ -78,6 +78,7 @@ SOFTWARE.
 
 #include "asn1.h"
 #include "snmp.h"
+#include "snmp_api.h"
 #include "snmp_impl.h"
 #include "party.h"
 #include "context.h"
@@ -85,9 +86,8 @@ SOFTWARE.
 #include "md5.h"
 #include "acl.h"
 #include "system.h"
-#include "snmp_api.h"
 
-static void md5Digest __P((u_char *, int, u_char *));
+static void md5Digest (u_char *, int, u_char *) ;
 
 /** snmp_comstr_parse - parse the header of a community string-based message
 *                       such as that found in SNMPv1 and SNMPv2c.
@@ -95,16 +95,24 @@ static void md5Digest __P((u_char *, int, u_char *));
 *  NOTE: this function will need to be changed to support SNMPv2u and SNMPv3,
 *        since the version field must first be examined to see
 *        if the message contains a community string.
+
+    u_char * snmp_comstr_parse(
+    u_char *data        IN - message
+    int *length         IN/OUT - bytes left in message
+    u_char *sid         OUT - community string
+    int *slen           OUT - length of community string
+    int *version        OUT - message version
 */
+
 u_char *
-snmp_comstr_parse(data, length, sid, slen, version)
-u_char          *data;      /* IN - message */
-int             *length;    /* IN/OUT - bytes left in message */
-u_char          *sid;       /* OUT - community string */
-int             *slen;      /* OUT - length of community string */
-long            *version;   /* OUT - message version */
+snmp_comstr_parse(u_char *data,
+		  int *length,
+		  u_char *psid,
+		  int *slen,
+		  int *version)
 {
     u_char    type;
+    long ver;
 
     /* message is an ASN.1 SEQUENCE */
     data = asn_parse_header(data, length, &type);
@@ -118,38 +126,51 @@ long            *version;   /* OUT - message version */
     }
 
     /* first field is the version */
-    data = asn_parse_int(data, length, &type, version, sizeof(*version));
+    data = asn_parse_int(data, length, &type, &ver, sizeof(ver));
+    *version = ver;
     if (data == NULL){
         ERROR_MSG("bad parse of version");
         return NULL;
     }
 
     /* second field is the community string for SNMPv1 & SNMPv2c */
-    data = asn_parse_string(data, length, &type, sid, slen);
+    data = asn_parse_string(data, length, &type, psid, slen);
     if (data == NULL){
         ERROR_MSG("bad parse of community");
         return NULL;
     }
-    sid[*slen] = '\0';
+    psid[*slen] = '\0';
     return (u_char *)data;
 }
 
+#ifdef USE_V2PARTY_PROTOCOL
 /** snmp_party_parse - parse the header of a party-based message
 *                      such as that found in SNMPv2p
+
+    u_char * snmp_party_parse(
+    u_char          *data;       IN - message 
+    int		    *length;     IN/OUT - bytes left in message 
+    struct packet_info *pi;      OUT - packet info 
+    oid		    *srcParty;   OUT - source party 
+    oid             *dstParty;   OUT - dest party 
+    oid             *context;    OUT - context 
+    int		    *srcPartyLength;  OUT - length of source party 
+    int             *dstPartyLength;  OUT - length of dest party 
+    int             *contextLength;   OUT - length of context 
+    int		    pass;        IN - which pass     
 */
+
 u_char *
-snmp_party_parse(data, length,  pi, srcParty, srcPartyLength,
-                 dstParty, dstPartyLength, context, contextLength, pass)
-u_char          *data;      /* IN - message */
-int		*length;    /* IN/OUT - bytes left in message */
-struct packet_info *pi;     /* OUT - packet info */
-oid		*srcParty;  /* OUT - source party */
-oid             *dstParty;  /* OUT - dest party */
-oid             *context;   /* OUT - context */
-int		*srcPartyLength; /* OUT - length of source party */
-int             *dstPartyLength; /* OUT - length of dest party */
-int             *contextLength;  /* OUT - length of context */
-int		pass;       /* IN - which pass */
+snmp_party_parse(u_char *data,
+		 int *length,
+		 struct packet_info *pi,
+		 oid *srcParty,
+		 int *srcPartyLength,
+		 oid *dstParty,
+		 int *dstPartyLength,
+		 oid *context,		 
+		 int *contextLength,
+		 int pass)
 {
     u_char    type;
     oid	dstParty2[64];
@@ -210,10 +231,14 @@ int		pass;       /* IN - which pass */
     if (type == (ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OCTET_STR)){
 	/* noAuth */
 	pi->version = SNMP_VERSION_2p;
-
+	pi->sec_model = SNMP_SEC_MODEL_SNMPv2p;
+	pi->sec_level = SNMP_SEC_LEVEL_NOAUTH;
     } else if (type == (ASN_CONTEXT | ASN_CONSTRUCTOR | 2)){
 	/* AuthInformation */
 	pi->version = SNMP_VERSION_2p;
+        pi->mp_model = SNMP_MP_MODEL_SNMPv2p;
+	pi->sec_model = SNMP_SEC_MODEL_SNMPv2p;
+	pi->sec_level = SNMP_SEC_LEVEL_AUTHNOPRIV;
 	ismd5 = 1;
 
 	digestStart = data;
@@ -291,6 +316,7 @@ int		pass;       /* IN - which pass */
 	return NULL;
     if ((pass & FIRST_PASS) && (srcp->partyAuthProtocol == SNMPV2MD5AUTHPROT)){
 	/* RFC1446, Pg 18, 3.2.1 */
+	pi->sec_level = SNMP_SEC_LEVEL_AUTHPRIV;
 	if (!ismd5){
 	    /* snmpStatsBadAuths++ */
 	    return NULL;
@@ -361,6 +387,7 @@ int		pass;       /* IN - which pass */
     }
     return data;
 }
+#endif /* USE_V2PARTY_PROTOCOL */
 
 /** snmp_comstr_build - build the header of a community string-based message
 *                       such as that found in SNMPv1 and SNMPv2c.
@@ -369,13 +396,12 @@ int		pass;       /* IN - which pass */
 *        later, if not known.
 */
 u_char *
-snmp_comstr_build(data, length, sid, slen, version, messagelen)
-    u_char      *data;
-    int         *length;
-    u_char      *sid;
-    int         *slen;
-    int         *version;
-    int         messagelen;
+snmp_comstr_build(u_char *data,
+		  int *length,
+		  u_char *psid,
+		  int *slen,
+		  int *version,
+		  int messagelen)
 {
   /* version is an 'int' (CMU had it as a long, but was passing in a *int.
      Grrr.) assign version to verfix and pass in that to asn_build_int
@@ -408,7 +434,7 @@ snmp_comstr_build(data, length, sid, slen, version, messagelen)
     /* store the community string */
     data = asn_build_string(data, length,
             (u_char)(ASN_UNIVERSAL | ASN_PRIMITIVE | ASN_OCTET_STR),
-            sid, *slen);
+            psid, *slen);
     if (data == NULL){
         ERROR_MSG("buildstring");
         return NULL;
@@ -421,27 +447,25 @@ snmp_comstr_build(data, length, sid, slen, version, messagelen)
     return data;
 }
 
+#ifdef USE_V2PARTY_PROTOCOL
 /** snmp_party_build - build the header for a party-based security message.
 *                      In the first pass allocate and store all the fields.
 *                      In the second pass, actually do the encryption and
 *                      message digest creation.
 */
 u_char *
-snmp_party_build(data, length, pi, messagelen, srcParty, srcPartyLen,
-		   dstParty, dstPartyLen, context, contextLen,
-		   packet_len, pass)
-    u_char	    *data;
-    int		    *length;
-    struct packet_info *pi;
-    int		    messagelen;
-    oid		    *srcParty;
-    int		    srcPartyLen;
-    oid		    *dstParty;
-    int		    dstPartyLen;
-    oid		    *context;
-    int             contextLen;
-    int		    *packet_len;    /* OUT - length of complete packet */
-    int		    pass;  /* FIRST_PASS, LAST_PASS, none, or both */
+snmp_party_build(u_char *data,
+		 int *length,
+		 struct packet_info *pi,
+		 int messagelen,
+		 oid *srcParty,
+		 int srcPartyLen,
+		 oid *dstParty,
+		 int dstPartyLen,
+		 oid *context,
+		 int contextLen,
+		 int *packet_len,    /* OUT - length of complete packet */
+		 int pass)  /* FIRST_PASS, LAST_PASS, none, or both */
 {
     struct partyEntry *srcp, *dstp;
     struct timeval now;
@@ -542,7 +566,7 @@ snmp_party_build(data, length, pi, messagelen, srcParty, srcPartyLen,
 	digestEnd = data;
 
 	data = asn_build_unsigned_int(data, length,
-				      (u_char)(UINTEGER),
+				      (u_char)(ASN_UINTEGER),
 				      &dstp->partyAuthClock,
 				      sizeof(dstp->partyAuthClock));
 	if (data == NULL){
@@ -551,7 +575,7 @@ snmp_party_build(data, length, pi, messagelen, srcParty, srcPartyLen,
 	}
 
 	data = asn_build_unsigned_int(data, length,
-				      (u_char)(UINTEGER),
+				      (u_char)(ASN_UINTEGER),
 				      &srcp->partyAuthClock,
 				      sizeof(srcp->partyAuthClock));
 	if (data == NULL){
@@ -644,19 +668,19 @@ snmp_party_build(data, length, pi, messagelen, srcParty, srcPartyLen,
     }
     return (u_char *)endOfPacket;
 }
+#endif /* USE_V2PARTY_PROTOCOL */
 
 static void
-md5Digest(start, length, digest)
-    u_char *start;
-    int length;
-    u_char *digest;
+md5Digest(u_char *start,
+	  int length,
+	  u_char *digest)
 {
     u_char *cp;
     MDstruct MD;
     int i, j;
 #if WORDS_BIGENDIAN
     u_char *buf;
-    static u_char buffer[SNMP_MAX_LEN];
+    u_char buffer[SNMP_MAX_LEN];
 #endif
 
 #if 0
@@ -669,7 +693,7 @@ md5Digest(start, length, digest)
 #endif
 
 #if WORDS_BIGENDIAN
-    /* do the computation in a static array */
+    /* do the computation in an array */
     cp = buf = buffer;
     memmove(buf, start, length);
 #else
@@ -690,10 +714,12 @@ md5Digest(start, length, digest)
 	 *digest++ = (MD.buffer[i]>>j) & 0xFF;
 }
 
+#ifdef USE_V2PARTY_PROTOCOL
 int
-has_access(msg_type, target, subject, resources)
-    u_char msg_type;
-    int target, subject, resources;
+has_access(u_char msg_type,
+	   int target, 
+	   int subject,
+	   int resources)
 {
     struct aclEntry *ap;
 
@@ -704,4 +730,5 @@ has_access(msg_type, target, subject, resources)
 	return 1;
     return 0;
 }
+#endif
 
