@@ -23,19 +23,20 @@ WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 ******************************************************************/
+
 #include <config.h>
 
 #include <stdio.h>
 #include <errno.h>
-#ifdef HAVE_STDLIB_H
+#if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#ifdef HAVE_STRINGS_H
-#include <strings.h>
-#else
+#if HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
-#ifdef HAVE_UNISTD_H
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <sys/types.h>
@@ -56,7 +57,6 @@ SOFTWARE.
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
-#include <stdio.h>
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
@@ -73,8 +73,8 @@ SOFTWARE.
 
 #include "asn1.h"
 #include "snmp.h"
-#include "snmp_impl.h"
 #include "snmp_api.h"
+#include "snmp_impl.h"
 #include "snmp_client.h"
 #include "party.h"
 #include "context.h"
@@ -98,17 +98,15 @@ typedef long	fd_mask;
 #define FD_ZERO(p)	memset((p), 0, sizeof(*(p)))
 #endif
 
-#define PARTY_MIB_BASE	".1.3.6.1.6.3.3.1.3.127.0.0.1.1"
+#define PARTY_MIB_BASE	 ".1.3.6.1.6.3.3.1.3.127.0.0.1.1"
+#define CONTEXT_MIB_BASE ".1.3.6.1.6.3.3.1.4.127.0.0.1.1"
 
-_CRTIMP extern int errno;
-struct synch_state snmp_synch_state;
 
 int
-snmp_synch_input __P((int, struct snmp_session *, int, struct snmp_pdu *, void *));
+snmp_synch_input (int, struct snmp_session *, int, struct snmp_pdu *, void *);
 
 struct snmp_pdu *
-snmp_pdu_create(command)
-    int command;
+snmp_pdu_create(int command)
 {
     struct snmp_pdu *pdu;
 
@@ -132,10 +130,9 @@ snmp_pdu_create(command)
  * Add a null variable with the requested name to the end of the list of
  * variables for this pdu.
  */
-struct variable_list* snmp_add_null_var(pdu, name, name_length)
-    struct snmp_pdu *pdu;
-    oid *name;
-    int name_length;
+struct variable_list* snmp_add_null_var(struct snmp_pdu * pdu, 
+					oid *name, 
+					int name_length)
 {
     struct variable_list *vars;
 
@@ -159,31 +156,33 @@ struct variable_list* snmp_add_null_var(pdu, name, name_length)
 }
 
 int
-snmp_synch_input(op, session, reqid, pdu, magic)
-    int op;
-    struct snmp_session *session;
-    int reqid;
-    struct snmp_pdu *pdu;
-    void *magic;
+snmp_synch_input(int op,
+		 struct snmp_session *session,
+		 int reqid,
+		 struct snmp_pdu *pdu,
+		 void *magic)
 {
     struct synch_state *state = (struct synch_state *)magic;
 
     if (reqid != state->reqid)
 	return 0;
     state->waiting = 0;
-    if (op == RECEIVED_MESSAGE && pdu->command == GET_RSP_MSG){
+    if (op == RECEIVED_MESSAGE && pdu->command == SNMP_MSG_RESPONSE){
 	/* clone the pdu */
 	state->pdu = snmp_clone_pdu(pdu);
 	state->status = STAT_SUCCESS;
+	snmp_errno = 0;  /* XX all OK when msg received ? */
+	session->s_snmp_errno = 0;
     } else if (op == TIMED_OUT){
 	state->pdu = NULL;
 	state->status = STAT_TIMEOUT;
-        snmp_errno = SNMPERR_TIMEOUT;
+	snmp_errno = SNMPERR_TIMEOUT;
+	session->s_snmp_errno = SNMPERR_TIMEOUT;
     }
+
     return 1;
 }
 
-
 /*
  * If there was an error in the input pdu, creates a clone of the pdu
  * that includes all the variables except the one marked by the errindex.
@@ -196,15 +195,14 @@ snmp_synch_input(op, session, reqid, pdu, magic)
  * be returned.
  */
 struct snmp_pdu *
-snmp_fix_pdu(pdu, command)
-    struct snmp_pdu *pdu;
-    int command;
+snmp_fix_pdu(struct snmp_pdu *pdu,
+	     int command)
 {
     struct variable_list *var, *newvar;
     struct snmp_pdu *newpdu;
-    int index, copied = 0;
+    int varindex, copied = 0;
 
-    if (pdu->command != GET_RSP_MSG || pdu->errstat == SNMP_ERR_NOERROR || pdu->errindex <= 0)
+    if (pdu->command != SNMP_MSG_RESPONSE || pdu->errstat == SNMP_ERR_NOERROR || pdu->errindex <= 0)
 	return NULL;
     /* clone the pdu */
     newpdu = (struct snmp_pdu *)malloc(sizeof(struct snmp_pdu));
@@ -215,20 +213,32 @@ snmp_fix_pdu(pdu, command)
     newpdu->errstat = SNMP_DEFAULT_ERRSTAT;
     newpdu->errindex = SNMP_DEFAULT_ERRINDEX;
     if (pdu->enterprise){
-	newpdu->enterprise = (oid *)malloc(pdu->enterprise_length);
-	memmove(newpdu->enterprise, pdu->enterprise, pdu->enterprise_length);
+	newpdu->enterprise = (oid *)malloc(sizeof(oid)*pdu->enterprise_length);
+	memmove(newpdu->enterprise, pdu->enterprise, sizeof(oid)*pdu->enterprise_length);
     }
     if (pdu->community){
 	newpdu->community = (u_char *)malloc(pdu->community_len);
 	memmove(newpdu->community, pdu->community, pdu->community_len);
     }
+   if (pdu->srcParty){
+	newpdu->srcParty = (oid *)malloc(sizeof(oid)*pdu->srcPartyLen);
+	memmove(newpdu->srcParty, pdu->srcParty, sizeof(oid)*pdu->srcPartyLen);
+    }
+    if (pdu->dstParty){
+	newpdu->dstParty = (oid *)malloc(sizeof(oid)*pdu->dstPartyLen);
+	memmove(newpdu->dstParty, pdu->dstParty, sizeof(oid)*pdu->dstPartyLen);
+    }
+    if (pdu->context){
+	newpdu->context = (oid *)malloc(sizeof(oid)*pdu->contextLen);
+	memmove(newpdu->context, pdu->context, sizeof(oid)*pdu->contextLen);
+    }
     var = pdu->variables;
-    index = 1;
-    if (pdu->errindex == index){	/* skip first variable */
+    varindex = 1;
+    if (pdu->errindex == varindex){	/* skip first variable */
       if (var == NULL)
         return NULL;
       var = var->next_variable;
-      index++;
+      varindex++;
     }
     if (var != NULL){
 	newpdu->variables = newvar = (struct variable_list *)malloc(sizeof(struct variable_list));
@@ -246,7 +256,7 @@ snmp_fix_pdu(pdu, command)
 
 	while(var->next_variable){
 	    var = var->next_variable;
-	    if (++index == pdu->errindex)
+	    if (++varindex == pdu->errindex)
 		continue;
 	    newvar->next_variable = (struct variable_list *)malloc(sizeof(struct variable_list));
 	    newvar = newvar->next_variable;
@@ -263,7 +273,7 @@ snmp_fix_pdu(pdu, command)
 	    copied++;
 	}
     }
-    if (index < pdu->errindex || copied == 0){
+    if (varindex < pdu->errindex || copied == 0){
 	snmp_free_pdu(newpdu);
 	return NULL;
     }
@@ -272,88 +282,86 @@ snmp_fix_pdu(pdu, command)
 
 
 /*
+ * snmp_clone_pdu
  * Creates (allocates and copies) a clone of the input PDU.
  */
 struct snmp_pdu *
-snmp_clone_pdu(pdu)
-    struct snmp_pdu *pdu;
+snmp_2clone_pdu(struct snmp_pdu *pdu, struct snmp_pdu *newpdu)
 {
-    struct variable_list *var, *newvar;
-    struct snmp_pdu *newpdu;
-
-    /* clone the pdu */
-    newpdu = (struct snmp_pdu *)malloc(sizeof(struct snmp_pdu));
+    struct variable_list *var, *newvar, *oldvar;
     memmove(newpdu, pdu, sizeof(struct snmp_pdu));
     newpdu->variables = 0;
     var = pdu->variables;
-    if (var != NULL){
-	newpdu->variables = newvar =
+    oldvar = 0;
+    while (var) {
+	newvar =
 	    (struct variable_list *)malloc(sizeof(struct variable_list));
 	memmove(newvar, var, sizeof(struct variable_list));
 	if (var->name != NULL){
 	    newvar->name = (oid *)malloc(var->name_length * sizeof(oid));
 	    memmove(newvar->name, var->name, var->name_length * sizeof(oid));
 	}
-	if (var->val.string != NULL){
+	/* need a pointer and a length to copy a string value. */
+	if ((var->val.string != NULL) && (var->val_len)) {
 	    newvar->val.string = (u_char *)malloc(var->val_len);
 	    memmove(newvar->val.string, var->val.string, var->val_len);
 	}
-	newvar->next_variable = 0;
-
-	while(var->next_variable){
-	    var = var->next_variable;
-	    newvar->next_variable =
-		(struct variable_list *)malloc(sizeof(struct variable_list));
-	    newvar = newvar->next_variable;
-	    memmove(newvar, var, sizeof(struct variable_list));
-	    if (var->name != NULL){
-		newvar->name = (oid *)malloc(var->name_length * sizeof(oid));
-		memmove(newvar->name, var->name, var->name_length * sizeof(oid));
-	    }
-	    if (var->val.string != NULL){
-		newvar->val.string = (u_char *)malloc(var->val_len);
-		memmove(newvar->val.string, var->val.string, var->val_len);
-	    }
-	    newvar->next_variable = 0;
+	else {
+	    newvar->val.string = NULL; newvar->val_len = 0;
 	}
+
+	if (0 == newpdu->variables) newpdu->variables = newvar;
+	if (oldvar) oldvar->next_variable = newvar;
+	oldvar = newvar;
+	oldvar->next_variable = 0;
+	var = var->next_variable;
     }
     if (pdu->enterprise){
-	newpdu->enterprise = (oid *)malloc(pdu->enterprise_length);
-	memmove(newpdu->enterprise, pdu->enterprise, pdu->enterprise_length);
+	newpdu->enterprise = (oid *)malloc(sizeof(oid)*pdu->enterprise_length);
+	memmove(newpdu->enterprise, pdu->enterprise, sizeof(oid)*pdu->enterprise_length);
     }
     if (pdu->community){
 	newpdu->community = (u_char *)malloc(pdu->community_len);
 	memmove(newpdu->community, pdu->community, pdu->community_len);
     }
     if (pdu->srcParty){
-	newpdu->srcParty = (oid *)malloc(pdu->srcPartyLen);
-	memmove(newpdu->srcParty, pdu->srcParty, pdu->srcPartyLen);
+	newpdu->srcParty = (oid *)malloc(sizeof(oid)*pdu->srcPartyLen);
+	memmove(newpdu->srcParty, pdu->srcParty, sizeof(oid)*pdu->srcPartyLen);
     }
     if (pdu->dstParty){
-	newpdu->dstParty = (oid *)malloc(pdu->dstPartyLen);
-	memmove(newpdu->dstParty, pdu->dstParty, pdu->dstPartyLen);
+	newpdu->dstParty = (oid *)malloc(sizeof(oid)*pdu->dstPartyLen);
+	memmove(newpdu->dstParty, pdu->dstParty, sizeof(oid)*pdu->dstPartyLen);
     }
     if (pdu->context){
-	newpdu->context = (oid *)malloc(pdu->contextLen);
-	memmove(newpdu->context, pdu->context, pdu->contextLen);
+	newpdu->context = (oid *)malloc(sizeof(oid)*pdu->contextLen);
+	memmove(newpdu->context, pdu->context, sizeof(oid)*pdu->contextLen);
     }
     return newpdu;
 }
 
+struct snmp_pdu *
+snmp_clone_pdu(struct snmp_pdu *pdu)
+{
+    struct snmp_pdu *newpdu;
+    newpdu = (struct snmp_pdu *)malloc(sizeof(struct snmp_pdu));
+    if (newpdu)
+        snmp_2clone_pdu(pdu, newpdu);
+
+    return (newpdu);
+}
+
 
 int
-snmp_synch_response(ss, pdu, response)
-    struct snmp_session *ss;
-    struct snmp_pdu *pdu;
-    struct snmp_pdu **response;
+snmp_synch_response(struct snmp_session *ss,
+		    struct snmp_pdu *pdu,
+		    struct snmp_pdu **response)
 {
-    struct synch_state *state = &snmp_synch_state;
+    struct synch_state *state = ss->snmp_synch_state;
     int numfds, count;
     fd_set fdset;
     struct timeval timeout, *tvp;
     int block;
 
-
     if ((state->reqid = snmp_send(ss, pdu)) == 0){
 	*response = NULL;
 	snmp_free_pdu(pdu);
@@ -381,8 +389,14 @@ snmp_synch_response(ss, pdu, response)
 		if (errno == EINTR){
 		    continue;
 		} else {
-		    snmp_set_detail(strerror(errno));
 		    snmp_errno = SNMPERR_GENERR;
+		/* CAUTION! if another thread closed the socket(s)
+		   waited on here, the session structure was freed.
+		   It would be nice, but we can't rely on the pointer.
+		    ss->s_snmp_errno = SNMPERR_GENERR;
+		    ss->s_errno = errno;
+		 */
+		    snmp_set_detail(strerror(errno));
 		}
 	    /* FALLTHRU */
 	    default:
@@ -395,39 +409,112 @@ snmp_synch_response(ss, pdu, response)
     return state->status;
 }
 
-void
-snmp_synch_setup(session)
-    struct snmp_session *session;
+int
+snmp_sess_synch_response(void *sessp,
+			 struct snmp_pdu *pdu,
+			 struct snmp_pdu **response)
 {
+    struct snmp_session *ss;
+    struct synch_state *state;
+    int numfds, count;
+    fd_set fdset;
+    struct timeval timeout, *tvp;
+    int block;
+
+    ss = snmp_sess_session(sessp);
+    state = ss->snmp_synch_state;
+
+    if ((state->reqid = snmp_sess_send(sessp, pdu)) == 0){
+	*response = NULL;
+	snmp_free_pdu(pdu);
+	return STAT_ERROR;
+    }
+    state->waiting = 1;
+
+    while(state->waiting){
+	numfds = 0;
+	FD_ZERO(&fdset);
+	block = SNMPBLOCK;
+	tvp = &timeout;
+	timerclear(tvp);
+	snmp_sess_select_info(sessp, &numfds, &fdset, tvp, &block);
+	if (block == 1)
+	    tvp = NULL;	/* block without timeout */
+	count = select(numfds, &fdset, 0, 0, tvp);
+	if (count > 0){
+	    snmp_sess_read(sessp, &fdset);
+	} else switch(count){
+	    case 0:
+		snmp_sess_timeout(sessp);
+		break;
+	    case -1:
+		if (errno == EINTR){
+		    continue;
+		} else {
+		    snmp_errno = SNMPERR_GENERR;
+		/* CAUTION! if another thread closed the socket(s)
+		   waited on here, the session structure was freed.
+		   It would be nice, but we can't rely on the pointer.
+		    ss->s_snmp_errno = SNMPERR_GENERR;
+		    ss->s_errno = errno;
+		 */
+		    snmp_set_detail(strerror(errno));
+		}
+	    /* FALLTHRU */
+	    default:
+		snmp_free_pdu(pdu);
+		*response = NULL;
+		return STAT_ERROR;
+	}
+    }
+    *response = state->pdu;
+    return state->status;
+}
+
+void snmp_synch_reset(struct snmp_session *session)
+{
+    if (session && session->snmp_synch_state) {
+       free((char*)session->snmp_synch_state);
+       session->snmp_synch_state = 0;
+    }
+}
+
+void
+snmp_synch_setup(struct snmp_session *session)
+{
+    struct synch_state *rp = (struct synch_state *)malloc(sizeof(struct synch_state));
+    rp->waiting = 0;
+    rp->pdu = NULL;
+    session->snmp_synch_state = rp;
+
     session->callback = snmp_synch_input;
-    session->callback_magic = (void *)&snmp_synch_state;
+    session->callback_magic = (void *)rp;
 }
 
 char	*error_string[19] = {
-    "(noError) No Error",
-    "(tooBig) Response message would have been too large.",
-    "(noSuchName) There is no such variable name in this MIB.",
-    "(badValue) The value given has the wrong type or length.",
-    "(readOnly) The two parties used do not have access to use the specified SNMP PDU.",
-    "(genError) A general failure occured",
-    "noAccess",
-    "wrongType",
-    "wrongLength",
-    "wrongEncoding",
-    "wrongValue",
-    "noCreation",
-    "inconsistentValue",
-    "resourceUnavailable",
-    "commitFailed",
-    "undoFailed",
-    "authorizationError",
-    "notWritable",
-    "inconsistentName"
+    (char*)"(noError) No Error",
+    (char*)"(tooBig) Response message would have been too large.",
+    (char*)"(noSuchName) There is no such variable name in this MIB.",
+    (char*)"(badValue) The value given has the wrong type or length.",
+    (char*)"(readOnly) The two parties used do not have access to use the specified SNMP PDU.",
+    (char*)"(genError) A general failure occured",
+    (char*)"noAccess",
+    (char*)"wrongType",
+    (char*)"wrongLength",
+    (char*)"wrongEncoding",
+    (char*)"wrongValue",
+    (char*)"noCreation",
+    (char*)"inconsistentValue",
+    (char*)"resourceUnavailable",
+    (char*)"commitFailed",
+    (char*)"undoFailed",
+    (char*)"authorizationError",
+    (char*)"notWritable",
+    (char*)"inconsistentName"
 };
 
 char *
-snmp_errstring(errstat)
-    int	errstat;
+snmp_errstring(int errstat)
 {
     if (errstat <= MAX_SNMP_ERR && errstat >= SNMP_ERR_NOERROR){
 	return error_string[errstat];
@@ -436,6 +523,8 @@ snmp_errstring(errstat)
     }
 }
 
+#ifdef USE_V2PARTY_PROTOCOL
+
 /*
  * In: Dest IP address, src, dst parties and lengths and context and contextlen
  * Initializes a noAuth/noPriv party pair, a context, and 2 acl entries.
@@ -443,10 +532,13 @@ snmp_errstring(errstat)
  * Out: returns 0 if OK, -1 if an error occurred.
  */
 int
-ms_party_init(destaddr, src, srclen, dst, dstlen, context, contextlen)
-    in_addr_t destaddr;
-    oid *src, *dst, *context;
-    int *srclen, *dstlen, *contextlen;
+ms_party_init(in_addr_t destaddr,
+	      oid *src, 
+	      int *srclen,
+	      oid *dst, 
+	      int *dstlen,
+	      oid *context,
+	      int *contextlen)
 {
     u_long addr;
     u_short port;
@@ -493,7 +585,7 @@ ms_party_init(destaddr, src, srclen, dst, dstlen, context, contextlen)
 	pp1->partyAuthPrivateLen = rp->partyAuthPrivateLen = 0;
 	pp1->partyPrivPrivateLen = rp->partyPrivPrivateLen = 0;
 	pp1->partyStorageType = 2; /* volatile */
-	pp1->partyStatus = rp->partyStatus = PARTYACTIVE;
+	pp1->partyStatus = rp->partyStatus = SNMP_ROW_ACTIVE;
 #define PARTYCOMPLETE_MASK              65535
 	/* all collumns - from party_vars.c XXX */
 	pp1->partyBitMask = rp->partyBitMask = PARTYCOMPLETE_MASK;
@@ -532,14 +624,14 @@ ms_party_init(destaddr, src, srclen, dst, dstlen, context, contextlen)
 	pp2->partyAuthPrivateLen = rp->partyAuthPrivateLen = 0;
 	pp2->partyPrivPrivateLen = rp->partyPrivPrivateLen = 0;
 	pp2->partyStorageType = 2; /* volatile */
-	pp2->partyStatus = rp->partyStatus = PARTYACTIVE;
+	pp2->partyStatus = rp->partyStatus = SNMP_ROW_ACTIVE;
 #define PARTYCOMPLETE_MASK              65535
 	/* all collumns - from party_vars.c XXX */
 	pp2->partyBitMask = rp->partyBitMask = PARTYCOMPLETE_MASK;
     }
     twoIndex = pp2->partyIndex;
 
-    if (!read_objid(PARTY_MIB_BASE, context, contextlen)){
+    if (!read_objid(CONTEXT_MIB_BASE, context, contextlen)){
 	snmp_errno = SNMPERR_BAD_CONTEXT;
 	snmp_set_detail(PARTY_MIB_BASE);
 	return -1;
@@ -560,7 +652,7 @@ ms_party_init(destaddr, src, srclen, dst, dstlen, context, contextlen)
 	cxp->contextLocalTime = CURRENTTIME;
 	cxp->contextProxyContextLen = 0;
 	cxp->contextStorageType = 2;
-	cxp->contextStatus = rxp->contextStatus = CONTEXTACTIVE;
+	cxp->contextStatus = rxp->contextStatus = SNMP_ROW_ACTIVE;
 #define CONTEXTCOMPLETE_MASK              0x03FF
 	/* all collumns - from context_vars.c XXX */
 	cxp->contextBitMask = rxp->contextBitMask = CONTEXTCOMPLETE_MASK;
@@ -572,7 +664,7 @@ ms_party_init(destaddr, src, srclen, dst, dstlen, context, contextlen)
 	ap = acl_createEntry(oneIndex, twoIndex, cxindex);
 	ap->aclPriveleges = 132;
 	ap->aclStorageType = 2; /* volatile */
-	ap->aclStatus = ACLACTIVE;
+	ap->aclStatus = SNMP_ROW_ACTIVE;
 #define ACLCOMPLETE_MASK              0x3F
 	/* all collumns - from acl_vars.c XXX */
 	ap->aclBitMask = ACLCOMPLETE_MASK;
@@ -581,3 +673,4 @@ ms_party_init(destaddr, src, srclen, dst, dstlen, context, contextlen)
     return 0; /* SUCCESS */
 }
 
+#endif /* USE_V2PARTY_PROTOCOL */
