@@ -27,6 +27,9 @@ SOFTWARE.
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#if HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #if TIME_WITH_SYS_TIME
 # ifdef WIN32
 #  include <sys/timeb.h>
@@ -58,8 +61,10 @@ SOFTWARE.
 #include <sys/ioctl.h>
 #endif
 #ifndef WIN32
+#ifdef HAVE_NLIST_H
 #include <nlist.h>
 #endif
+#endif
 #if HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif
@@ -72,6 +77,15 @@ SOFTWARE.
 #if HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
 #endif
+
+#if HAVE_STRING_H
+#include <string.h>
+#else
+#include <strings.h>
+#endif
+
+#include "asn1.h"
+#include "snmp_api.h"
 #include "system.h"
 
 #define NUM_NETWORKS    32   /* max number of interfaces to check */
@@ -123,11 +137,6 @@ opendir(char *filename)
  *	downcase = TRUE;
  *  }
  */
-    /* Get us a DIR structure */
-    p = (DIR*)malloc(sizeof(DIR));
-    /* Newz(1303, p, 1, DIR); */
-    if(p == NULL)
-	return NULL;
 
     /* Create the search pattern */
     strcpy(scanname, filename);
@@ -143,6 +152,12 @@ opendir(char *filename)
 	return NULL;
     }
 
+    /* Get us a DIR structure */
+    p = (DIR*)malloc(sizeof(DIR));
+    /* Newz(1303, p, 1, DIR); */
+    if(p == NULL)
+	return NULL;
+
     /* now allocate the first part of the string table for
      * the filenames that we find.
      */
@@ -150,8 +165,8 @@ opendir(char *filename)
     p->start = (char*)malloc(idx * sizeof(char));
     /* New(1304, p->start, idx, char);*/
     if(p->start == NULL) {
-		fprintf(stderr,"opendir: malloc failed!\n");
-		exit(1);
+		free(p);
+		return NULL;
     }
     strcpy(p->start, FindData.cFileName);
 /*  if(downcase)
@@ -173,8 +188,8 @@ opendir(char *filename)
 			idx+len+1 * sizeof(char));
 	/* Renew(p->start, idx+len+1, char);*/
 	if(p->start == NULL) {
-	    fprintf(stderr,"opendir: malloc failed!\n");
-		exit(1);
+		free(p);
+	    return NULL;
 	}
 	strcpy(&p->start[idx], FindData.cFileName);
 /*	if (downcase) 
@@ -229,22 +244,21 @@ closedir(DIR *dirp)
     return 1;
 }
 
+#ifndef HAVE_GETTIMEOFDAY
 
-
-int gettimeofday(tv, tz)
-struct timeval *tv;
-struct timezone *tz;
+int gettimeofday(struct timeval *tv,
+		 struct timezone *tz)
 {
     struct _timeb timebuffer;
 
     _ftime(&timebuffer);
-    tv->tv_usec = timebuffer.millitm;
+    tv->tv_usec = timebuffer.millitm * 1000;
     tv->tv_sec = timebuffer.time;
-    return(1);
+    return(0);
 }
+#endif
 
-
-in_addr_t get_myaddr()
+in_addr_t get_myaddr(void)
 {
   char local_host[130];
   int result;
@@ -285,13 +299,13 @@ in_addr_t get_myaddr()
   return((in_addr_t)in_addr.sin_addr.s_addr);
 }
 
-long get_uptime __P((void))
+long get_uptime (void)
 {
     return (0); /* not implemented */
 }
 
 char *
-winsock_startup __P((void))
+winsock_startup (void)
 {
  WORD VersionRequested;
  WSADATA stWSAData;
@@ -313,13 +327,13 @@ winsock_startup __P((void))
  return(NULL);
 }
 
-void winsock_cleanup __P((void))
+void winsock_cleanup (void)
 {
    WSACleanup();
 }
 
 #else
-in_addr_t get_myaddr __P((void))
+in_addr_t get_myaddr (void)
 {
     int sd;
     struct ifconf ifc;
@@ -360,23 +374,23 @@ in_addr_t get_myaddr __P((void))
     return 0;
 }
 
-
+#ifndef rtems
 /*
  * Returns uptime in centiseconds(!).
  */
-long get_uptime __P((void))
+long get_uptime (void)
 {
-#ifdef bsdlike
+#if defined(bsdlike) && !defined(solaris2) && !defined(linux)
     struct timeval boottime, now, diff;
 #ifndef CAN_USE_SYSCTL
     int kmem;
     static struct nlist nl[] = {
-#if !defined(hpux) && !defined(solaris2)
-	    { "_boottime" },
+#if !defined(hpux)
+	    { (char*)"_boottime" },
 #else
 	    { "boottime" },
 #endif
-	    { "" }
+	    { (char*)"" }
 	};
 
     if ((kmem = open("/dev/kmem", 0)) < 0)
@@ -444,13 +458,15 @@ long get_uptime __P((void))
    }
    return uptim;
 #endif /* linux */
+
+   return (0); /* not implemented */
 }
+#endif rtems
 #endif
 
 #ifndef HAVE_STRDUP
 char *
-strdup(src)
-    char *src;
+strdup(char *src)
 {
     int len;
     char *dst;
@@ -461,4 +477,23 @@ strdup(src)
     strcpy(dst, src);
     return(dst);
 }
-#endif
+#endif	/* HAVE_STRDUP */
+
+#ifndef HAVE_SETENV
+int setenv(char *name,
+	   char *value,
+	   int overwrite)
+{
+    char *cp;
+    int ret;
+
+    if (overwrite == 0) {
+	if (getenv(name)) return 0;
+    }
+    cp = (char*)malloc(strlen(name)+strlen(value)+2);
+    if (cp == NULL) return -1;
+    sprintf(cp, "%s=%s", name, value);
+    ret = putenv(cp);
+    return ret;
+}
+#endif /* HAVE_SETENV */
